Unit CommWrap;
     { ४祭   㯮    ࠡ⮩ १ ᨫ }

Interface
  Function _OpenCOM            {- Open a COMport for communication        -}
    (_Nr         : Byte;       {- Internal portnumber: 0-7                -}
     _Address    : Word;       {- Port address in hex: 000-3F8            -}
     _IrqNum     : Byte;       {- Port Irq number: 0-15  (255 for no Irq) -}
     _Baudrate   : LongInt;    {- Baudrate: (see table)                   -}
     _ParityBit  : Char;       {- Parity  : 'O','E' or 'N'                -}
     _Databits   : Byte;       {- Databits: 5-8                           -}
     _Stopbits   : Byte;       {- Stopbits: 1-2                           -}
     _BufferSize : Word;       {- Size of input buffer: 0-65535           -}
     _Handshake  : Boolean)    {- True to use hardware handshake          -}
       : Boolean;              {- Returns true if ok                      -}
                               {- Note that the DTR signal is set if      -}
                               {- handshake is enabled (true).            -}

  Procedure _CloseCOM          {- Close a open COMport                    -}
    (_Nr : Byte);              {- Internal portnumber: 0-7                -}

  Procedure _ResetCOM          {- Reset a open COMport incl. buffer       -}
    (_Nr : Byte);              {- Internal portnumber: 0-7                -}

  Procedure _COMSettings       {- Change settings for a open COMport      -}
    (_Nr        : Byte;        {- Internal portnumber: 0-7                -}
     _Baudrate  : LongInt;     {- Baudrate: (see table)                   -}
     _Paritybit : Char;        {- Parity  : 'O','E' or 'N'                -}
     _Databits  : Byte;        {- Databits: 5-8                           -}
     _Stopbits  : Byte;        {- Stopbits: 1-2                           -}
     _Handshake : Boolean);    {- True to use hardware handshake          -}
                               {- Note that the DTR signal is set if      -}
                               {- handshake is enabled (true).            -}

  Function _WriteCOM           {- Writes a character to a port            -}
    (_Nr : Byte;               {- Internal portnumber: 0-7                -}
     _Ch : Char)               {- Character to be written to port         -}
      : Boolean;               {- True if character send                  -}

  Function _WriteCOMString     {- Writes a string to a port               -}
    (_Nr : Byte;               {- Internal portnumber: 0-7                -}
     _St : String)             {- String to be written to port            -}
      : Boolean;               {- True if string send                     -}

  Function _CheckCOM           {- Check if any character is arrived       -}
    (_Nr : Byte;               {- Internal portnumber: 0-7                -}
     Var _Ch : Char)           {- Character arrived                       -}
      : Boolean;               {- Returns true and character if any       -}

  Procedure AT_Cmd(cmd,ans0: string);


Implementation
{-   /////////////////////////////////////////////////   -}
Uses EtAsync, Foscom, AonVars;

Function _OpenCOM(_Nr: Byte; _Address: Word; _IrqNum: Byte;
                  _Baudrate: LongInt; _ParityBit: Char; _Databits: Byte;
                  _Stopbits: Byte; _BufferSize: Word; _Handshake: Boolean): Boolean;
Begin
  If UseFossil=1 Then Begin
    Fos_Init(_Nr);
    Fos_Parms(_Nr,_Baudrate,_Databits,_ParityBit,_Stopbits);
    _OpenCOM:=True;
  End Else Begin
    _OpenCOM:=OpenCOM(_Nr,
                      _Address,
                      _IrqNum,
                      _BaudRate,
                      _ParityBit,
                      _DataBits,
                      _StopBits,
                      8192,True);
  End;

End;


Procedure _CloseCOM(_Nr: Byte);
Begin
  If UseFossil=1 Then Fos_Close(_Nr) Else CloseCOM(_Nr);
End;


Procedure _ResetCOM(_Nr : Byte);
Begin
  If UseFossil=0 Then ResetCOM(_Nr);
End;

Procedure _COMSettings(_Nr        : Byte;
                       _Baudrate  : LongInt;
                       _Paritybit : Char;
                       _Databits  : Byte;
                       _Stopbits  : Byte;
                       _Handshake : Boolean);
Begin
  If UseFossil=1 Then Fos_Parms(_Nr,_Baudrate,_Databits,_ParityBit,_Stopbits)
  Else COMSettings(_Nr,_Baudrate,_Paritybit,_Databits,_Stopbits,_Handshake);
End;

Function _WriteCOM(_Nr: Byte; _Ch: Char): Boolean;
Begin
  If UseFossil=1 Then Begin
    Fos_Write(_Nr,_Ch);
    _WriteCOM:=True;
  End
  Else _WriteCOM:=WriteCOM(_Nr,_Ch);

End;

Function _WriteCOMString(_Nr: Byte; _St: String): Boolean;
Begin
  If UseFossil=1 Then Begin
    Fos_String(_Nr,_St);
    _WriteCOMString:=True;
  End
  Else
   _WriteCOMString:=WriteCOMString(_Nr,_St+#13#10);

End;


Function _CheckCOM(_Nr: Byte; Var _Ch: Char): Boolean;
Begin
  If UseFossil=1 Then Begin
    Repeat
    Until Fos_Avail(_Nr);
    _Ch:=Fos_Receive(_Nr);
    _CheckCOM:=True;
  End
  Else _CheckCOM:=CheckCOM(_Nr,_Ch);
End;

Procedure AT_Cmd(cmd,ans0: string);
Var answ: string;
    ChOUT,ChIN: Char;
    DosTicker: longint absolute $0040:$006C;
    OldTick: longint;

    l_ans0,l_ans0_1: word;

Begin
  answ:='';
  l_ans0:=Length(ans0); l_ans0_1:=l_ans0-1;

  _WriteCOMString(ModemPort,cmd+#13#10);
  OldTick:=DosTicker;
  Repeat
    repeat
      if DosTicker-OldTick>WaitForModem*18 then begin
         _CloseCOM(ModemPort);
         If TerminalEcho=1 Then
           WriteLn('Modem not responding !');
         Halt(0);
      end;
    until _CheckCOM(ModemPort,ChIn);
    If TerminalEcho=1 Then Write(ChIn);
    If (ChIn<>Chr(10)) AND (ChIn<>Chr(13)) then answ:=answ+ChIn;

    If pos('ERROR',answ)<>0 then begin
       _CloseCOM(ModemPort);
       Halt(0);
    end;

  Until (ChIn=Chr(10)) AND (Copy(answ,length(answ)-l_ans0_1,l_ans0)=ans0);
End;

BEGIN

END.

