	PAGE	82,124

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;              ROCKWELL CONFIDENTIAL AND PROPRIETARY
;        This source is the sole property of ROCKWELL Inc.
;    Reproduction or utilization of this source in whole or in 
;  part is forbidden without the written consent of ROCKWELL Inc.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;                 (c) Copyright ROCKWELL, INC. 1991
;                       All Rights Reserved
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;       Last update:  10-13-93
;	Last PVCS update: $Date:   17 Oct 1991 10:36:18  $ 
;
;	TITLE:		d.asm
;
;	FACILITY:	ADPCM decompression and compression
;
;	MODULE DESCRIPTION:
;
;	Decompression routines for 2, 3, and 4 bits per sample (BPS).
;
;	Debugging macros for the decompression routines that show
;	various partial results.
;
;	Upsampling routines for 14.4k and 22.1k Hz. Assumes original
;	samples are at 7200.  This fixes some alaising problems on
;	signal reconstructions on some sound cards, like the Sound
;	Blaster, where there is no low pass filter after the DAC.
;
;	Optional, noise shaping filters for decompression.
;
;	Compression routines for 2, 3, and 4 BPS.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;	Revision History:
;
;	$Log: $
;  
; 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	TITLE	d.asm
	NAME	d
;	OPTION	SCOPED

	DOSSEG
	.8086

ifdef		mem_s
	memS	equ	1	; small model
	memM	equ	0	; medium model
elseifdef	mem_m
	memS	equ	0	; small model
	memM	equ	1	; medium model
else
	memS	equ	1	; small model
	memM	equ	0	; medium model
endif

if 	memS
	.model SMALL,PASCAL
elseif memM
	.model MEDIUM,PASCAL
else
	.model SMALL,PASCAL	; default to SMALL
endif

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Design Notes: Small model, 8086 opcodes, 'PASCAL' calling convenctions
;		and 'PASCAL' segments.
;
; 'PASCAL' calling convention (hidden by the .MODEL small,PASCAL declaration)
;
;	push	bp
;	mov	bp,sp
;
;	mov	ax,[bp+8]	; get first parameter of three
;	mov	bx,[bp+6]	; get second parameter of three
;		      		; etc....
;	... other code
;
;	pop	bp
;	ret	6		; three parameters
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	.DATA
	ASSUME DS:@data, ES:@data, SS:@data

	; Constants.
	;
PNT98		EQU	32113		;0.98
PNT012		EQU	393		;0.012
PNT006		EQU	197		;0.006

	; Maximum Nu for the adaptive quantizer.
	;
QDLMX4		EQU	02ED5h		;(I)QDLMX = 1.11V
QDLMX3		EQU	03B7Ah		;(I)QDLMX = 1.41V
QDLMX2		EQU	054C4h		;(I)QDLMX = 2.01V

	; Minimum Nu for the adaptive quantizer.
	;
QDLMN		EQU	0001Fh		;QDLMN = IQDLMN = 2.87 mV.

	; Emphasis and De-emphasis filter coefficients.
	;
DEMPCF		EQU	03333h		;0.4
PDEMPCF		EQU	03333h		;0.4

	; Total delay line length for the pole and zero delay
	; lines.
	;
QORDER		EQU	8

	; Silence codewords
	;
SL2CW		EQU	013ech		; Silence codeword for 2 BPS.
SL3CW		EQU	023deh		; Silence codeword for 3 BPS.
SL4CW		EQU	0c11ch		; Silence codeword for 4 BPS.

	; Debug equates for the partial results in the decompression
	; routines.
	;
IQDELTA		EQU	0
IQLEVEL		EQU	1
IQOUT		EQU	2
APFZB0		EQU	3
DCDPPZ1		EQU	4
DCDPPZ2		EQU	5
DCDZPZ1		EQU	6
DCDZPZ6		EQU	7
DCDSGA1		EQU	8
DCDSGA2		EQU	9
DCDSGB1		EQU    10
DCDSGB6		EQU    11

	; Constants needed for the 7200 to 11025 Interpolator. Do
	; not change this value unless you know exactly what you are
	; doing.  784 = LCYCLE * 8 * 2.
	;
LCYCLE		EQU	98
MCYCLE		EQU	64

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Design Notes: Relation of QDataIndex to position in QData Buffer.
;
; Delay data is not rotated by shuffling; the rotation is done by
; the QDataIndex.  This variable always points to the data to be
; multiplied by the coefficient a1.
;
; The coefficients, a1..a2 and b1..b6, stay in the same relative
; position in the coefficient array.  Updates to these values are
; done in place.
;
; Illustration belows shows the value of QDataIndex and the Delay
; Line data in relation to the coefficient array.
;
; Position
; in Qdata	Start	2nd	3rd	4th
; -----------------------------------------
; 0	a1	Y(n-1)	Y(n-2)	Q(n-1)	Q(n-2)
; 1	a2	Y(n-2)	Q(n-1)	Q(n-2)	Q(n-3)
; 2	b1	Q(n-1)	Q(n-2)	Q(n-3)	Q(n-4)
; 3	b2	Q(n-2)	Q(n-3)	Q(n-4)	Q(n-5)
; 4	b3	Q(n-3)	Q(n-4)	Q(n-5)	Q(n-6)
; 5	b4	Q(n-4)	Q(n-5)	Q(n-6)	Y(n-1)
; 6	b5	Q(n-5)	Q(n-6)	Y(n-1)	Y(n-2)
; 7	b6	Q(n-6)	Y(n-1)	Y(n-2)	Q(n-1)
; -----------------------------------------
; QDataIndex	0	7	6	5
; -----------------------------------------
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	; Coefficient Table for the pole and zero linear predictor.
	;
;PUBLIC	_pzTable
_pzTable	dw	0	; a1
		dw	0	; a2
		dw	0	; b1
		dw	0	; b2
		dw	0	; b3
		dw	0	; b4
		dw	0	; b5
		dw	0	; b6

	; Delay line pointer to the coefficient a1.
	;
QdataIndex	dw	0

	; Delay line.
	;
Qdata		dw	QORDER dup (0)

	; Last Nu value.
	;
LastNu		dw	0

	; De-emphasis filter delay line (one element).
	;
Dempz		dw	0

	; Adaptive quantizier output.
	;
NewQdata	dw	0

	; Multiplier table to calculate new Nu for 4 BPS.
	;
ML4bps		dw	04CCDh		;ML0 = 2.4/4
		dw	04000h		;ML1 = 2.0/4
		dw	03333h		;ML2 = 1.6/4
		dw	02666h		;ML3 = 1.2/4
		dw	01CCDh		;ML4 = 0.9/4
		dw	01CCDh		;ML5 = 0.9/4
		dw	01CCDh		;ML6 = 0.9/4
		dw	01CCDh		;ML7 = 0.9/4
		dw	01CCDh		;ML8 = 0.9/4
		dw	01CCDh		;ML9 = 0.9/4
		dw	01CCDh		;ML10 = 0.9/4
		dw	01CCDh		;ML11 = 0.9/4
		dw	02666h		;ML12 = 1.2/4
		dw	03333h		;ML13 = 1.6/4
		dw	04000h		;ML14 = 2.0/4
		dw	04CCDh		;ML15 = 2.4/4

	; Multiplier table for 4 BPS to calculate inverse
	; quantizer output.  This number, index by the code
	; word, times Nu is the inverse quantizer output.
	;
Zeta4bps	dw	0A88Bh		;Zeta0 = -2.733/4 
		dw	0BDCBh		;Zeta1 = -2.069/4 
		dw	0CC29h		;Zeta2 = -1.618/4 
		dw	0D7CFh		;Zeta3 = -1.256/4 
		dw	0E1D8h		;Zeta4 = -0.9424/4
		dw	0EAFBh		;Zeta5 = -6568/4  
		dw	0F395h		;Zeta6 = -0.3881/4
		dw	0FBE4h		;Zeta7 = -0.1284/4
		dw	0041Ch		;Zeta8 = 0.1284/4 
		dw	00C6Bh		;Zeta9 = 0.3881/4 
		dw	01505h		;Zeta10 = 0.6568/4
		dw	01E28h		;Zeta11 = 0.9424/4
		dw	02831h		;Zeta12 = 1.256/4 
		dw	033C7h		;Zeta13 = 1.618/4 
		dw	04235h		;Zeta14 = 2.069/4 
		dw	05775h		;Zeta15 = 2.733/4 

	; Multiplier table to calculate new Nu for 3 BPS.
	;
ML3bps		dw	03800h		;ML0 = 1.75/4
		dw	02800h		;ML1 = 1.25/4
		dw	01CCDh		;ML2 = 0.9/4
		dw	01CCDh		;ML3 = 0.9/4
		dw	01CCDh		;ML4 = 0.9/4
		dw	01CCDh		;ML5 = 0.9/4
		dw	02800h		;ML6 = 1.25/4
		dw	03800h		;ML7 = 1.75/4

	; Multiplier table for 3 BPS to calculate inverse
	; quantizer output.  This number, index by the code
	; word, times Nu is the inverse quantizer output.
	;
Zeta3bps	dw	0BB23h		;Zeta0 = -2.152/4
		dw	0D4FEh		;Zeta1 = -1.344/4
		dw	0E7CFh		;Zeta2 = -0.7560/4
		dw	0F828h		;Zeta3 = -0.2451/4
		dw	007D8h		;Zeta4 = 0.2451/4
		dw	01831h		;Zeta5 = 0.7560/4
		dw	02B02h		;Zeta6 = 1.344/4
		dw	044DDh		;Zeta7 = 2.152/4

	; Multiplier table to calculate new Nu for 2 BPS.
	;
ML2bps		dw	03333h		;ML0 = 1.6/4
		dw	0199Ah		;ML1 = 0.8/4
		dw	0199Ah		;ML2 = 0.8/4
		dw	03333h		;ML3 = 1.6/4

	; Multiplier table for 2 BPS to calculate inverse
	; quantizer output.  This number, index by the code
	; word, times Nu is the inverse quantizer output.
	;
Zeta2bps	dw	0CFAEh		;Zeta0 = -1.510/4
		dw	0F183h		;Zeta1 = -0.4528/4
		dw	00E7Dh		;Zeta2 = 0.4528/4
		dw	03052h		;Zeta3 = 1.510/4

	; Maximum limit for Nu.  Changes based on 2, 3, or 4
	; BPS selected.  Initialization routine updates this value.
	;
QDelayMX	dw	QDLMX3

	; Array index by BPS for updating QDelayMX
	;
QDelayTable	dw	0
		dw	0
		dw	QDLMX2
		dw	QDLMX3
		dw	QDLMX4

	; Flag set by the initialization routine to indicate whether
	; the decompression routines shall look for the silence
	; codewords in the data stream.
	;
SLFlag 		db	0

	; Flag to indicate that a silence codeword was detected in the
	; data stream, and that the number of samples of silence
	; codeword, which follows the silence codeword, was not seen.
	;
	; This occurs when the silence codeword was the last word
	; processed by the decompression routines.
	;
SLSilenceOn	db	0		; if 1, looking for silence length

	; Dummy call back function.  When the decompression routines
	; see the silence codeword and the number of samples of silence 
	; codeword, the routine calls a function to announce the number
	; of silence data points.  This dummy routine makes sure that a
 	; call to outer space does not occur.
	;
SLCallBack 	dd	SLDummy

	.CODE

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Macro definitions used in the decompression, interpolation, and
; compression functions.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; ----------------------------------------------------------------
; Mod adder.
;
addm	MACRO	reg,value
	add	reg,value
	and	reg,0fh
ENDM

; ----------------------------------------------------------------
; 64 Mod adder.
;
addm64	MACRO	reg,value
	add	reg,value
	and	reg,3fh
ENDM

; ----------------------------------------------------------------
; 128 Mod adder.
;
addm128	MACRO	reg,value
	add	reg,value
	and	reg,7fh
ENDM

; ----------------------------------------------------------------
; Small Mod adder.
;
addmsml	MACRO	reg,value
	add	reg,value

IFDEF BIG_FILTERS
	and	reg,3fh
ELSE
	and	reg,1fh
ENDIF

ENDM

; ----------------------------------------------------------------
; Large Mod adder.
;
addmlrg	MACRO	reg,value
	add	reg,value

IFDEF BIG_FILTERS
	and	reg,7fh
ELSE
	and	reg,3fh
ENDIF

ENDM

; ----------------------------------------------------------------
; Clip 16 bit register on over flow.
;
clip	MACRO	hreg,lreg,address
	LOCAL	NoClip

	jno	NoClip

	or	hreg,hreg
ifnb <lreg>
	mov	lreg,0
endif
	mov	hreg,8000h

ifnb <address>
	jns	address		; sign reversal on overflow
else
	jns	NoClip		; sign reversal on overflow
endif

ifnb <lreg>
	dec	lreg
endif
	dec	hreg

ifnb <address>
	jmp	SHORT address
endif

NoClip:
ENDM

; ----------------------------------------------------------------
; Clip 8 bit register on over flow.
;
cliplow	MACRO	hreg,lreg,address
	LOCAL	NoClip

	jno	NoClip

	or	hreg,hreg
ifnb <lreg>
	mov	lreg,0
endif
	mov	hreg,80h

ifnb <address>
	jns	address		; sign reversal on overflow
else
	jns	NoClip		; sign reversal on overflow
endif

ifnb <lreg>
	dec	lreg
endif
	dec	hreg

ifnb <address>
	jmp	SHORT address
endif

NoClip:
ENDM

; ----------------------------------------------------------------
; Adjust decimal points after a multiplication.
;
adjust	MACRO	hreg,lreg
ifnb <lreg>
	shl	lreg,1
	rcl	hreg,1
else
	shl	hreg,1
endif
ENDM

; ----------------------------------------------------------------
; Round.
;
round	MACRO	hreg,lreg
	shl	lreg,1
	adc	hreg,0
ENDM

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Macro definitions for decompression debug.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

DEBUGGER	= 0

MASK_IQDELTA	EQU	(1 SHL IQDELTA)
MASK_IQLEVEL	EQU	(1 SHL IQLEVEL)
MASK_IQOUT	EQU	(1 SHL IQOUT)
MASK_APFZB0	EQU	(1 SHL APFZB0)
MASK_DCDPPZ1	EQU	(1 SHL DCDPPZ1)
MASK_DCDPPZ2	EQU	(1 SHL DCDPPZ2)
MASK_DCDZPZ1	EQU	(1 SHL DCDZPZ1)
MASK_DCDZPZ6	EQU	(1 SHL DCDZPZ6)
MASK_DCDSGA1	EQU	(1 SHL DCDSGA1)
MASK_DCDSGA2	EQU	(1 SHL DCDSGA2)
MASK_DCDSGB1	EQU	(1 SHL DCDSGB1)
MASK_DCDSGB6	EQU	(1 SHL DCDSGB6)

;DEBUGGER	= DEBUGGER OR MASK_APFZB0

IFDEF DEBUG_IQDELTA
DEBUGGER	= DEBUGGER OR MASK_IQDELTA
ENDIF 

IFDEF DEBUG_IQLEVEL
DEBUGGER	= DEBUGGER OR MASK_IQLEVEL
ENDIF 

IFDEF DEBUG_IQOUT
DEBUGGER	= DEBUGGER OR MASK_IQOUT
ENDIF 

IFDEF DEBUG_APFZB0
DEBUGGER	= DEBUGGER OR MASK_APFZB0
ENDIF 

IFDEF DEBUG_DCDPPZ1
DEBUGGER	= DEBUGGER OR MASK_DCDPPZ1
ENDIF 

IFDEF DEBUG_DCDPPZ2
DEBUGGER	= DEBUGGER OR MASK_DCDPPZ2
ENDIF 

IFDEF DEBUG_DCDZPZ1
DEBUGGER	= DEBUGGER OR MASK_DCDZPZ1
ENDIF 

IFDEF DEBUG_DCDZPZ6
DEBUGGER	= DEBUGGER OR MASK_DCDZPZ6
ENDIF 

IFDEF DEBUG_DCDSGA1
DEBUGGER	= DEBUGGER OR MASK_DCDSGA1
ENDIF 

IFDEF DEBUG_DCDSGA2
DEBUGGER	= DEBUGGER OR MASK_DCDSGA2
ENDIF 

IFDEF DEBUG_DCDSGB1
DEBUGGER	= DEBUGGER OR MASK_DCDSGB1
ENDIF 

IFDEF DEBUG_DCDSGB6
DEBUGGER	= DEBUGGER OR MASK_DCDSGB6
ENDIF 

IFDEF DEBUG_DECOMP_ALL
DEBUGGER	= 0ffffh
ENDIF

; ----------------------------------------------------------------
; First delay element in the pole predicator.
;
SaveDCDPPZ1	MACRO	value
IF (DEBUGGER AND MASK_DCDPPZ1) NE 0
	push	ax
	push	bx
	push	cx
	push	dx

	mov	ax,4
	push	ax
	push	value
	call	UpdateVar

	pop	dx
	pop	cx
	pop	bx
	pop	ax
ENDIF
ENDM

; ----------------------------------------------------------------
; Second delay element in the pole predicator.
;
SaveDCDPPZ2	MACRO	value
IF (DEBUGGER AND MASK_DCDPPZ2) NE 0
	push	ax
	push	bx
	push	cx
	push	dx

	mov	ax,5
	push	ax
	push	value
	call	UpdateVar

	pop	dx
	pop	cx
	pop	bx
	pop	ax
ENDIF
ENDM

; ----------------------------------------------------------------
; First delay element in the zero predicator.
;
SaveDCDZPZ1	MACRO	value
IF (DEBUGGER AND MASK_DCDZPZ1) NE 0
	push	ax
	push	bx
	push	cx
	push	dx

	mov	ax,6
	push	ax
	push	value
	call	UpdateVar

	pop	dx
	pop	cx
	pop	bx
	pop	ax
ENDIF
ENDM

; ----------------------------------------------------------------
; Sixth (and last) delay element in the zero predicator.
;
SaveDCDZPZ6	MACRO	value
IF (DEBUGGER AND MASK_DCDZPZ6) NE 0
	push	ax
	push	bx
	push	cx
	push	dx

	mov	ax,7
	push	ax
	push	value
	call	UpdateVar

	pop	dx
	pop	cx
	pop	bx
	pop	ax
ENDIF
ENDM

; ----------------------------------------------------------------
; First coefficient in the pole predicator.
;
SaveDCDSGA1	MACRO	value
IF (DEBUGGER AND MASK_DCDSGA1) NE 0
	push	ax
	push	bx
	push	cx
	push	dx

	mov	ax,8
	push	ax
	push	value
	call	UpdateVar

	pop	dx
	pop	cx
	pop	bx
	pop	ax
ENDIF
ENDM

; ----------------------------------------------------------------
; Second coefficient in the pole predicator.
;
SaveDCDSGA2	MACRO	value
IF (DEBUGGER AND MASK_DCDSGA2) NE 0
	push	ax
	push	bx
	push	cx
	push	dx

	mov	ax,9
	push	ax
	push	value
	call	UpdateVar

	pop	dx
	pop	cx
	pop	bx
	pop	ax
ENDIF
ENDM

; ----------------------------------------------------------------
; Sixth coefficient in the zero predicator.
;
SaveDCDSGB6	MACRO	value
IF (DEBUGGER AND MASK_DCDSGB6) NE 0
	push	ax
	push	bx
	push	cx
	push	dx

	mov	ax,11
	push	ax
	push	value
	call	UpdateVar

	pop	dx
	pop	cx
	pop	bx
	pop	ax
ENDIF
ENDM

; ----------------------------------------------------------------
; Nu value.
;
SaveIQDELTA	MACRO	value
IF (DEBUGGER AND MASK_IQDELTA) NE 0
	push	ax
	push	bx
	push	cx
	push	dx

	sub	ax,ax
	push	ax
	push	value
	call	UpdateVar

	pop	dx
	pop	cx
	pop	bx
	pop	ax
ENDIF
ENDM

; ----------------------------------------------------------------
; Inverse quantizer output.
;
SaveIQOut	MACRO	value
IF (DEBUGGER AND MASK_IQOut) NE 0
	push	ax
	push	bx
	push	cx
	push	dx

	mov	ax,2
	push	ax
	push	value
	call	UpdateVar

	pop	dx
	pop	cx
	pop	bx
	pop	ax
ENDIF
ENDM

; ----------------------------------------------------------------
; First coefficient in the zero predicator.
;
SaveDCDSGB1	MACRO	value
IF (DEBUGGER AND MASK_DCDSGB1) NE 0
	push	ax
	push	bx
	push	cx
	push	dx

	mov	ax,10
	push	ax
	push	value
	call	UpdateVar

	pop	dx
	pop	cx
	pop	bx
	pop	ax
ENDIF
ENDM

; ----------------------------------------------------------------
; Input ADPCM codeword to the decompressor.
;
SaveIQLevel	MACRO	value
IF (DEBUGGER AND MASK_IQLevel) NE 0
	push	ax
	push	bx
	push	cx
	push	dx

	mov	ax,1
	push	ax
	shr	value,1
	push	value
	call	UpdateVar

	pop	dx
	pop	cx
	pop	bx
	pop	ax
ENDIF
ENDM

; ----------------------------------------------------------------
; Output ADCPM data point from the compressor, but before the
; noise shaper, if conditionly assembled, and the de-emphasis
; filter.
;
SaveAPFZB0	MACRO	value
IF (DEBUGGER AND MASK_APFZB0) NE 0
	push	ax
	push	bx
	push	cx
	push	dx

	mov	ax,3
	push	ax
	push	value
	call	UpdateVar

	pop	dx
	pop	cx
	pop	bx
	pop	ax
ENDIF
ENDM

; ----------------------------------------------------------------
; Deliever debug values and tags to the 'C' main routine, where the
; disk functions are.
;
SaveUpdates	MACRO
IF DEBUGGER NE 0
	push	ax
	push	bx
	push	cx
	push	dx

	call	WriteDebug

	pop	dx
	pop	cx
	pop	bx
	pop	ax
ENDIF
ENDM

; ----------------------------------------------------------------
;
IF DEBUGGER NE 0
extrn		UpdateVar : PROC
extrn		WriteDebug : PROC
ENDIF

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Utility!!  Routines that both the decompressor and the compressor
; use.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; pzPred
;
; Linear predictor coefficient update routine.  Local to this module.
;
; Inputs:
;
; CX = Q(n), i.e. WORD PTR NewQData
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	EVEN
pzPred	PROC USES SI DI

	; Load pointers to the predictor table and the pointer
	; to the coefficient a1.
	;
	mov	si,OFFSET @data:_pzTable
	mov	di,QDataIndex

	; Calculate coefficient a1(n).
	;
	lodsw
	mov	bx,PNT98
	imul	bx

	; Adjust the decimal point and round 0.98*a1(n-1)
	;
	adjust	dx,ax
	round	dx,ax
	mov	bx,dx

	; Determine sign of Q(n) * Y(n-1)
	;
	; Do not change the sign determination method!
	;
	mov	ax,cx
	imul	Qdata[di]
	adjust	dx,ax
	round	dx,ax

	mov	ax,PNT012
	jns	Noa1
	neg	ax

Noa1:
	add	bx,ax

	; Check for overflows, clip if necessary.
	;
	clip	bx
	mov	[si-2],bx		; save new a1
	SaveDCDSGA1	bx

	; Calculate coefficient a2(n).
	;
	lodsw
	mov	bx,PNT98		; use this alot
	imul	bx

	; Adjust the decimal point and round 0.98*a2(n-1)
	;
	adjust	dx,ax
	round	dx,ax
	mov	bx,dx

	; Determine sign of Q(n) * Y(n-2)
	;
	; Do not change the sign determination method!
	;
	addm	di,2			; point to Y(n-2)
	mov	ax,cx
	imul	Qdata[di]
	adjust	dx,ax
	round	dx,ax

	mov	ax,PNT012
	jns	Noa2
	neg	ax

Noa2:
	add	bx,ax

	; Check for overflows, clip if necessary.
	;
	clip	bx
	mov	[si-2],bx		; save new a2
	SaveDCDSGA2	bx

	; Calculate coefficient b1(n).
	;
	lodsw
	mov	bx,PNT98		; use this alot
	imul	bx

	; Adjust the decimal point and round 0.98*b1(n-1)
	;
	adjust	dx,ax
	round	dx,ax
	mov	bx,dx

	; Determine sign of Q(n) * Q(n-1)
	;
	; Do not change the sign determination method!
	;
	addm	di,2			; point to Q(n-1)
	mov	ax,cx
	imul	Qdata[di]
	adjust	dx,ax
	round	dx,ax

	mov	ax,PNT006
	jns	Nob1
	neg	ax

Nob1:
	add	bx,ax

	; Check for overflows, clip if necessary.
	;
	clip	bx
	mov	[si-2],bx		; save new b1
	SaveDCDSGB1	bx

	; Calculate coefficient b2(n).
	;
	lodsw
	mov	bx,PNT98		; use this alot
	imul	bx

	; Adjust the decimal point and round 0.98*b2(n-1)
	;
	adjust	dx,ax
	round	dx,ax
	mov	bx,dx

	; Determine sign of Q(n) * Q(n-2)
	;
	; Do not change the sign determination method!
	;
	addm	di,2			; point to Q(n-2)
	mov	ax,cx
	imul	Qdata[di]
	adjust	dx,ax
	round	dx,ax

	mov	ax,PNT006
	jns	Nob2
	neg	ax

Nob2:
	add	bx,ax

	; Check for overflows, clip if necessary.
	;
	clip	bx
	mov	[si-2],bx 		; save new b2

	; Calculate coefficient b3(n).
	;
	lodsw
	mov	bx,PNT98		; use this alot
	imul	bx

	; Adjust the decimal point and round 0.98*b3(n-1)
	;
	adjust	dx,ax
	round	dx,ax
	mov	bx,dx

	; Determine sign of Q(n) * Q(n-3)
	;
	; Do not change the sign determination method!
	;
	addm	di,2			; point to Q(n-3)
	mov	ax,cx
	imul	Qdata[di]
	adjust	dx,ax
	round	dx,ax

	mov	ax,PNT006
	jns	Nob3
	neg	ax

Nob3:
	add	bx,ax

	; Check for overflows, clip if necessary.
	;
	clip	bx
	mov	[si-2],bx 		; save new b3

	; Calculate coefficient b4(n).
	;
	lodsw
	mov	bx,PNT98		; use this alot
	imul	bx

	; Adjust the decimal point and round 0.98*b4(n-1)
	;
	adjust	dx,ax
	round	dx,ax
	mov	bx,dx

	; Determine sign of Q(n) * Q(n-4)
	;
	; Do not change the sign determination method!
	;
	addm	di,2			; point to Q(n-4)
	mov	ax,cx
	imul	Qdata[di]
	adjust	dx,ax
	round	dx,ax

	mov	ax,PNT006
	jns	Nob4
	neg	ax

Nob4:
	add	bx,ax

	; Check for overflows, clip if necessary.
	;
	clip	bx
	mov	[si-2],bx 		; save new b4

	; Calculate coefficient b5(n).
	;
	lodsw
	mov	bx,PNT98		; use this alot
	imul	bx

	; Adjust the decimal point and round 0.98*b5(n-1)
	;
	adjust	dx,ax
	round	dx,ax
	mov	bx,dx

	; Determine sign of Q(n) * Q(n-5)
	;
	; Do not change the sign determination method!
	;
	addm	di,2			; point to Q(n-5)
	mov	ax,cx
	imul	Qdata[di]
	adjust	dx,ax
	round	dx,ax

	mov	ax,PNT006
	jns	Nob5
	neg	ax

Nob5:
	add	bx,ax

	; Check for overflows, clip if necessary.
	;
	clip	bx
	mov	[si-2],bx 		; save new b5

	; Calculate coefficient b6(n).
	;
	lodsw
	mov	bx,PNT98		; use this alot
	imul	bx

	; Adjust the decimal point and round 0.98*b6(n-1)
	;
	adjust	dx,ax
	round	dx,ax
	mov	bx,dx

	; Determine sign of Q(n) * Q(n-6)
	;
	; Do not change the sign determination method!
	;
	addm	di,2			; point to Q(n-6)
	mov	ax,cx
	imul	Qdata[di]
	adjust	dx,ax
	round	dx,ax

	mov	ax,PNT006
	jns	Nob6
	neg	ax

Nob6:
	add	bx,ax

	; Check for overflows, clip if necessary.
	;
	clip	bx
	mov	[si-2],bx		; save new b6
	SaveDCDSGB6	bx

	ret

pzPred	ENDP

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Design Notes: Sum of Products Multiplications.
;
;	lodsw				; a2(n)
;	addm	di,2			; Y(n-2)
;	SaveDCDPPZ2	Qdata[di]
;	imul	Qdata[di]
;	adjust	dx,ax			; decimal point adjustment.
;	add	bx,ax			; add to 32 bit accumulator.
;	adc	cx,dx
;	clip	cx,bx			; check for overflow.
;
; Multiplications are 16-bit signed numbers producing a signed
; 32-bit result.  The two operands are usually numbers less than
; one; this requires a 32-bit shift by the macro "adjust" to bring the
; decimal point in line.  The 32-bit addition is two 16-bit additions
; with carry.  The "clip" macro checks for overflow and limits the
; result of the addition to 0x7fffffff or 0x80000000 (for 32-bit
; results), or 0x7fff or 0x8000 (for 16-bit results).  Note that
; the "clip" macro depends on the flags being set because of an
; addition; the 80x86 processor does not update these flags because
; of a move operation.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; XpzCalc
;
; Linear pole and zero predictor calculate.  CX,BX register pair is the
; 32 bit accumulator.  Local to this module.
;
; Inputs:
;
; CX = Initial Value.  BX set to zero.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	EVEN
XpzCalc	PROC

	; Load pointers to the predictor table and the pointer
	; to the coefficient a1.
	;
	mov	si,OFFSET @data:_pzTable
	mov	di,QDataIndex

	; Do the sum of products.  Eight multiplications done
	; in line to save time.
	;
	lodsw				; a1(n)
	SaveDCDPPZ1	Qdata[di]
	imul	Qdata[di]		; Y(n-1)
	adjust	dx,ax			; decimal point adjustment.
	mov	bx,ax			; add to 32 bit accumulator,
	add	cx,dx			; with BX set to zero.
	clip	cx,bx			; check for overflow.

	lodsw				; a2(n)
	addm	di,2			; Y(n-2)
	SaveDCDPPZ2	Qdata[di]
	imul	Qdata[di]
	adjust	dx,ax			; decimal point adjustment.
	add	bx,ax			; add to 32 bit accumulator.
	adc	cx,dx
	clip	cx,bx			; check for overflow.

	lodsw				; b1(n)
	addm	di,2			; Q(n-1)
	SaveDCDZPZ1	Qdata[di]
	imul	Qdata[di]
	adjust	dx,ax			; decimal point adjustment.
	add	bx,ax			; add to 32 bit accumulator.
	adc	cx,dx
	clip	cx,bx			; check for overflow.

	lodsw				; b2(n)
	addm	di,2			; Q(n-2)
	imul	Qdata[di]
	adjust	dx,ax			; decimal point adjustment.
	add	bx,ax			; add to 32 bit accumulator.
	adc	cx,dx
	clip	cx,bx			; check for overflow.

	lodsw				; b3(n)
	addm	di,2			; Q(n-3)
	imul	Qdata[di]
	adjust	dx,ax			; decimal point adjustment.
	add	bx,ax			; add to 32 bit accumulator.
	adc	cx,dx
	clip	cx,bx			; check for overflow.

	lodsw				; b4(n)
	addm	di,2			; Q(n-4)
	imul	Qdata[di]
	adjust	dx,ax			; decimal point adjustment.
	add	bx,ax			; add to 32 bit accumulator.
	adc	cx,dx
	clip	cx,bx			; check for overflow.

	lodsw				; b5(n)
	addm	di,2			; Q(n-5)
	imul	Qdata[di]
	adjust	dx,ax			; decimal point adjustment.
	add	bx,ax			; add to 32 bit accumulator.
	adc	cx,dx
	clip	cx,bx			; check for overflow.

	lodsw				; b6(n)
	addm	di,2			; Q(n-6)
	SaveDCDZPZ6	Qdata[di]
	imul	Qdata[di]
	adjust	dx,ax			; decimal point adjustment.
	add	bx,ax			; add to 32 bit accumulator.
	adc	cx,dx
	clip	cx,bx			; check for overflow.

	ret
XpzCalc	ENDP

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Post Filter!!
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

IFDEF POSTFILTER
	.DATA

	; Adaptive post filter number 1 coefficients.
	;
P8Table		dw	06666h	;0.8
		dw	051ebh	;0.8**2
		dw	04189h	;0.8**3
		dw	0346dh	;0.8**4
		dw	029f1h	;0.8**5
		dw	0218eh	;0.8**6

	; Adaptive post filter number 2 coefficients.
	;
PM5Table	dw	0c000h	;-0.5
		dw	0e000h	;-(0.5**2)
		dw	0f000h	;-(0.5**3)
		dw	0f800h	;-(0.5**4)
		dw	0fc00h	;-(0.5**5)
		dw	0fe00h	;-(0.5**6)

	; Temporay data storage.
	;
NewAppData	dw	0

	; Delay line for the first stage of the post filter.
	;
QPdata		dw	QORDER dup (0)

	; Delay line for the second stage of the post filter.
	;
QPPdata		dw	QORDER dup (0)

	.CODE
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; App1Calc
;
; Adaptive post filter number 1.  Local to this module.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	EVEN
App1Calc	PROC USES SI DI

	; Load pointers to the predictor table and the pointer
	; to the coefficient a1.
	;
	mov	si,OFFSET @data:_pzTable
	mov	di,QDataIndex

	mov	NewAppData,cx

	lodsw				; a1(n)
	imul	P8Table[0]		; make a1'(n)
	adjust	dx,ax
	round	dx,ax
	mov	ax,dx
	imul	QPdata[di]		; Y'(n-1)
	adjust	dx,ax
	mov	bx,ax
	add	cx,dx
	clip	cx,bx

	lodsw				; a2(n)
	imul	P8Table[2]		; make a2'(n)
	adjust	dx,ax
	round	dx,ax
	mov	ax,dx
	addm	di,2			; Y'(n-2)
	imul	QPdata[di]
	adjust	dx,ax
	add	bx,ax
	adc	cx,dx
	clip	cx,bx

	lodsw				; b1(n)
	imul	P8Table[0]		; make b1'(n)
	adjust	dx,ax
	round	dx,ax
	mov	ax,dx
	addm	di,2			; QP(n-1)
	imul	QPdata[di]
	adjust	dx,ax
	add	bx,ax
	adc	cx,dx
	clip	cx,bx

	lodsw				; b2(n)
	imul	P8Table[2]		; make b2'(n)
	adjust	dx,ax
	round	dx,ax
	mov	ax,dx
	addm	di,2			; QP(n-2)
	imul	QPdata[di]
	adjust	dx,ax
	add	bx,ax
	adc	cx,dx
	clip	cx,bx

	lodsw				; b3(n)
	imul	P8Table[4]		; make b3'(n)
	adjust	dx,ax
	round	dx,ax
	mov	ax,dx
	addm	di,2			; QP(n-3)
	imul	QPdata[di]
	adjust	dx,ax
	add	bx,ax
	adc	cx,dx
	clip	cx,bx

	lodsw				; b4(n)
	imul	P8Table[6]		; make b4'(n)
	adjust	dx,ax
	round	dx,ax
	mov	ax,dx
	addm	di,2			; QP(n-4)
	imul	QPdata[di]
	adjust	dx,ax
	add	bx,ax
	adc	cx,dx
	clip	cx,bx

	lodsw				; b5(n)
	imul	P8Table[8]		; make b5'(n)
	adjust	dx,ax
	round	dx,ax
	mov	ax,dx
	addm	di,2			; QP(n-5)
	imul	QPdata[di]
	adjust	dx,ax
	add	bx,ax
	adc	cx,dx
	clip	cx,bx

	lodsw				; b6(n)
	imul	P8Table[10]		; make b6'(n)
	adjust	dx,ax
	round	dx,ax
	mov	ax,dx
	addm	di,2			; QP(n-6)
	imul	QPdata[di]
	adjust	dx,ax
	add	bx,ax
	adc	cx,dx
	clip	cx,bx

	; Update delay line at the a1'(n) table entry position.
	;
	mov	QPdata[di],cx		; drop b6, now a1 Qdata.

	; Update delay line at the b1'(n) table entry position.
	;
	addm	di,4
	mov	ax,NewAppData
	mov	QPdata[di],ax		; drop a2, now b1 Qdata.

	ret
App1Calc	ENDP

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; App2Calc
;
; Adaptive post filter number 2.  Local to this module.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	EVEN
App2Calc	PROC USES SI DI

	; Load pointers to the predictor table and the pointer
	; to the coefficient a1.
	;
	mov	si,OFFSET @data:_pzTable
	mov	di,QDataIndex

	mov	NewAppData,cx

	lodsw				; a1(n)
	imul	PM5Table[0]		; make a1''(n)
	adjust	dx,ax
	round	dx,ax
	mov	ax,dx
	imul	QPPData[di]		; Y''(n-1)
	adjust	dx,ax
	mov	bx,ax
	add	cx,dx
	clip	cx,bx

	lodsw				; a2(n)
	imul	PM5Table[2]		; make a2''(n)
	adjust	dx,ax
	round	dx,ax
	mov	ax,dx
	addm	di,2			; Y''(n-2)
	imul	QPPData[di]
	adjust	dx,ax
	add	bx,ax
	adc	cx,dx
	clip	cx,bx

	lodsw				; b1(n)
	imul	PM5Table[0]		; make b1''(n)
	adjust	dx,ax
	round	dx,ax
	mov	ax,dx
	addm	di,2			; QPP(n-1)
	imul	QPPData[di]
	adjust	dx,ax
	add	bx,ax
	adc	cx,dx
	clip	cx,bx

	lodsw				; b2(n)
	imul	PM5Table[2]		; make b2''(n)
	adjust	dx,ax
	round	dx,ax
	mov	ax,dx
	addm	di,2			; QPP(n-2)
	imul	QPPData[di]
	adjust	dx,ax
	add	bx,ax
	adc	cx,dx
	clip	cx,bx

	lodsw				; b3(n)
	imul	PM5Table[4]		; make b3''(n)
	adjust	dx,ax
	round	dx,ax
	mov	ax,dx
	addm	di,2			; QPP(n-3)
	imul	QPPData[di]
	adjust	dx,ax
	add	bx,ax
	adc	cx,dx
	clip	cx,bx

	lodsw				; b4(n)
	imul	PM5Table[6]		; make b4''(n)
	adjust	dx,ax
	round	dx,ax
	mov	ax,dx
	addm	di,2			; QPP(n-4)
	imul	QPPData[di]
	adjust	dx,ax
	add	bx,ax
	adc	cx,dx
	clip	cx,bx

	lodsw				; b5(n)
	imul	PM5Table[8]		; make b5''(n)
	adjust	dx,ax
	round	dx,ax
	mov	ax,dx
	addm	di,2			; QPP(n-5)
	imul	QPPData[di]
	adjust	dx,ax
	add	bx,ax
	adc	cx,dx
	clip	cx,bx

	lodsw				; b6(n)
	imul	PM5Table[10]		; make b6''(n)
	adjust	dx,ax
	round	dx,ax
	mov	ax,dx
	addm	di,2			; QPP(n-6)
	imul	QPPData[di]
	adjust	dx,ax
	add	bx,ax
	adc	cx,dx
	clip	cx,bx

	; Update delay line at the a1''(n) table entry position.
	;
	mov	QPPData[di],cx		; drop b6, now a1 Qdata.

	; Update delay line at the b1''(n) table entry position.
	;
	addm	di,4
	mov	ax,NewAppData
	mov	QPPData[di],ax		; drop a2, now b1 Qdata.

	ret
App2Calc	ENDP
ENDIF	; POSTFILTER

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Decompression!!
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; RVDecomReset
;
; Reset local variables for ADPCM decoder.
;
; No Function Prototype.  This routine is local.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	EVEN
RVDecomReset	PROC USES SI DI

	mov	ax,@data
	mov	es,ax

	sub	ax,ax

	mov	QdataIndex,ax		; clear pointer to a1.
	mov	cx,QORDER
	mov	di,OFFSET @data:QData
	rep 	stosw			; clear delay line.
	mov	Dempz,ax		; clear de-emphasis delay line.
	mov	WORD PTR NewQdata,ax	; clear inverse quantizier.

IFDEF POSTFILTER
	mov	cx,QORDER
	mov	di,OFFSET @data:QPData
	rep 	stosw			; clear QPData delay line.

	mov	cx,QORDER
	mov	di,OFFSET @data:QPPData
	rep 	stosw			; clear QPPData delay line.

	mov	NewAppData,ax
ENDIF

	mov	SLSilenceOn,al		; not looking for silence length.

	mov	LastNu,QDLMN		; reset Nu.

	ret
RVDecomReset	ENDP

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; RVDecomInit
;
; Reset local variables for ADPCM decoder.  Public routine.
;
; Function Prototype:
;
; void pascal
;    RVDecomInit(int SetSLFlag, unsigned int bps,
;    	unsigned int (far pascal *SLCallBack)());
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	EVEN
RVDecomInit	PROC USES SI DI, SetSLFlag : WORD, bps : WORD, \
				 SLCallBackPtr : FAR PTR


	call	RVDecomReset		; reset the decoder proper.

	mov	ax,SetSLFlag		; move silence insertion check flag.
	mov	SLFlag,al

	mov	bx,bps			; set Nu maximum limit.
	shl	bx,1
	mov	ax,QDelayTable[bx]
	mov	QDelayMX,ax

	les	ax,SLCallBackPtr	; save silence insertion pointer.
	mov	WORD PTR SLCallBack,ax
	mov	WORD PTR SLCallBack+2,es

	ret
RVDecomInit	ENDP

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; SLDummy
;
; Dummy routine to keep the silence callback pointer to something
; harmless when not in use.
;
; No Function Prototype.  This routine is local.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	EVEN
SLDummy		PROC
	ret
SLDummy		ENDP

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; RVDecomOne
;
; Decode a code word.  Local to this module.
;
; Inputs:
;
; BX = Zeta, base inverse quantizer value, modified by Nu.
; AX = ML, adaptive multiplier for Nu.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	EVEN
RVDecomOne	PROC USES SI DI

	; Save LastNu in SI for later use in the following two
	; calculations.
	;
	mov	si,LastNu
	SaveIQDelta	si

	; Update Nu.
	;
	imul	si			; ax = ML

	; Decimal adjust, round, and multiple by four.
	;
	; Do not change this sequence!
	;
	adjust	dx,ax
	round	dx,ax
	shl	dx,1			; multiply by four
	clip	dx,<>,Ov2
	shl	dx,1
	clip	dx

Ov2:
	; Check Nu bounds.
	;
	cmp	dx,QDLMN
	jg	GDelta

	mov	dx,QDLMN
	jmp	SHORT RVdoFini

GDelta:
	mov	ax,QDelayMX
	cmp	dx,ax
	jle	RVdoFini

	mov	dx,ax

RVdoFini:
	mov	LastNu,dx		; save new Nu.

	; Make a new inverse quantizer value.
	;
	mov	ax,si	 		; si = LastNu
	imul	bx			; bx = Zeta

	; Decimal adjust, round, and multiple by four.
	;
	; Do not change this sequence!
	;
	adjust	dx,ax
	round	dx,ax
	shl	dx,1			; multiply by four
	clip	dx,<>,Ov1
	shl	dx,1
	clip	dx

Ov1:
	SaveIQOut	dx

	; Save new inverse quantizier output.
	;
	mov	WORD PTR NewQData,dx

	; Move inverse quantizer output into the upper summer
	; register.
	;
	mov	cx,dx

	; Compute (Xp+z)(n) + Q(n)
	;
	call	XpzCalc

	; Y(n) done, save in SI for later
	;
	mov	si,cx

IFDEF POSTFILTER
	; Adaptive post filtering.  The input to these routines
	; are in CX.
	;
	call	App1Calc
	call	App2Calc
ENDIF

	; Use a de-emphasis filter on Y(n) to remove the effects
	; from the emphasis filter used during compression.
	;
	mov	ax,DEMPCF
	imul	Dempz
	adjust	dx,ax
	add	bx,ax
	adc	cx,dx
	clip	cx,bx
	mov	Dempz,cx

	; Update predictor filter coefficients.
	;
	mov	cx,WORD PTR NewQData	; load inverse quantizer output.
	call	pzPred

	; Update delay line at the a1(n) table entry position.
	;
	mov	Qdata[di],si		; drop b6, now a1 Qdata
	SaveAPFZB0	si

	; Update delay line at the b1(n) table entry position.
	;
	mov	QDataIndex,di		; save for next iteration
	addm	di,4
	mov	Qdata[di],cx		; drop a2, now b1 Qdata

	; Return Y(n) + de-emphasis filter.
	;
	mov	ax,Dempz
	SaveUpdates
	ret

RVDecomOne	ENDP

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; RVDecom3bps
;
; Decode 3 codewords, Qdata0, Qdata1, and Qdata2.  Store 16 results
; at the location pointed to YnPtr.  Use the callback function to
; return the amount of silence to the calling routine if a silence
; codeword is detected.  Public routine.
;
; Function Prototype:
;
; unsigned long pascal
;    RVDecom3bps(
;    	unsigned int Qdata0,unsigned int Qdata1,unsigned int Qdata2,
;	   unsigned int far *YnPtr);
;
; Outputs:
;
; AX = Qdata words processed, ranges from 0 to 3.
; DX = number of decompressed words produced.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	EVEN
RVDecom3bps	PROC USES SI DI, Qdata0 : WORD, Qdata1 : WORD, \
			         Qdata2 : WORD, YnPtr : FAR PTR WORD

	; Check if the routine is waiting for the second word in
	; a silence deletion codeword - the number of zero data
	; points.
	;
	cmp	SLSilenceOn,0
	je	NoSLOn

	call	RVDecomReset		; reset the decoder proper.

	; Routine was waiting for the second word in a silence
	; deletion codeword - the number of zero data points.
	; Return this value back to the application.
	;
	push	Qdata0
	call	SLCallBack

	; Return number of code words consumed in AX, and the
	; number of decompressed data points produced in DX.
	;
	mov	ax,1
	sub	dx,dx
	ret

NoSLOn:
	; Load destination pointer.
	;
	les	di,YnPtr

	; Get first 16 bit code word.
	;
	mov	si,Qdata0

	; Check if routine is looking for silence codewords.
	;
	cmp	SLFlag,0
	je	NoSL0

	; Routine should look for silence codewords.  Check if
	; a codeword is found.
	;
	cmp	si,SL3CW
	jne	NoSL0

	call	RVDecomReset		; reset the decoder proper.

	; Routine found a silence codeword.  Return the number
	; of zero data points back to the application.
	;
	push	Qdata1
	call	SLCallBack

	; Return number of code words consumed in AX, and the
	; number of decompressed data points produced in DX.
	;
	mov	ax,2
	mov	dx,di
	sub	dx,WORD PTR [YnPtr]
	shr	dx,1
	ret

NoSL0:
	; No silence codeword found.  Work on the first word by
	; unpacking ADPCM codewords.
	;
	mov	bx,si		; left shift for index.
	shl	bl,1
	and	bx,0eh

	SaveIQLevel	bx
	mov	ax,ML3bps[bx]
	mov	bx,Zeta3bps[bx]
	call	RVDecomOne	; decompress.
	stosw

	mov	bx,si		; shift by 3, then left shift for index.
	mov	cl,2
	shr	bl,cl
	and	bx,0eh

	SaveIQLevel	bx
	mov	ax,ML3bps[bx]
	mov	bx,Zeta3bps[bx]
	call	RVDecomOne	; decompress.
	stosw

	mov	bx,si		; shift by 6, then left shift for index.
	mov	cl,3
	shl	bx,cl
	mov	bl,bh
	and	bx,0eh

	SaveIQLevel	bx
	mov	ax,ML3bps[bx]
	mov	bx,Zeta3bps[bx]
	call	RVDecomOne	; decompress.
	stosw

	mov	bx,si		; shift by 9, then left shift for index.
	mov	bl,bh
	and	bx,0eh

	SaveIQLevel	bx
	mov	ax,ML3bps[bx]
	mov	bx,Zeta3bps[bx]
	call	RVDecomOne	; decompress.
	stosw

	mov	bx,si		; shift by 12, then left shift for index.
	mov	bl,bh
	mov	cl,3
	shr	bl,cl
	and	bx,0eh

	SaveIQLevel	bx
	mov	ax,ML3bps[bx]
	mov	bx,Zeta3bps[bx]
	call	RVDecomOne	; decompress.
	stosw

	; Get second 16 bit code word.
	;
	mov	bx,Qdata1
	
	; Check if routine is looking for silence codewords.
	;
	cmp	SLFlag,0
	je	NoSL1

	; Routine should look for silence codewords.  Check if
	; a codeword is found.
	;
	cmp	bx,SL3CW
	jne	NoSL1

	call	RVDecomReset		; reset the decoder proper.

	; Routine found a silence codeword.  Return the the number
	; of zero data points back to the application.
	;
	push	Qdata2
	call	SLCallBack

	; Return number of code words consumed in AX, and the
	; number of decompressed data points produced in DX.
	;
	mov	ax,3
	mov	dx,di
	sub	dx,WORD PTR [YnPtr]
	shr	dx,1
	ret

NoSL1:
	; No silence codeword found.  Work on the second word by
	; unpacking ADPCM codewords.  A fragment of the last
	; ADPCM codeword remains in SI.
	;
	shl	si,1 		; get last bit into carry.
	mov	si,bx		; transfer to more permanent location.
	rcl	bl,1
	shl	bx,1		; left shift for index.
	and	bx,0eh

	SaveIQLevel	bx
	mov	ax,ML3bps[bx]
	mov	bx,Zeta3bps[bx]
	call	RVDecomOne	; decompress.
	stosw

	mov	bx,si	    	; shift by 2, then left shift for index.
	shr	bl,1
	and	bx,0eh

	SaveIQLevel	bx
	mov	ax,ML3bps[bx]
	mov	bx,Zeta3bps[bx]
	call	RVDecomOne	; decompress.
	stosw

	mov	bx,si	    	; shift by 5, then left shift for index.
	mov	cl,4
	shr	bl,cl
	and	bx,0eh

	SaveIQLevel	bx
	mov	ax,ML3bps[bx]
	mov	bx,Zeta3bps[bx]
	call	RVDecomOne	; decompress.
	stosw

	mov	bx,si	    	; shift by 8, then left shift for index.
	mov	bl,bh
	shl	bl,1
	and	bx,0eh

	SaveIQLevel	bx
	mov	ax,ML3bps[bx]
	mov	bx,Zeta3bps[bx]
	call	RVDecomOne	; decompress.
	stosw

	mov	bx,si	    	; shift by 11, then left shift for index.
	mov	bl,bh
	mov	cl,2
	shr	bl,cl
	and	bx,0eh

	SaveIQLevel	bx
	mov	ax,ML3bps[bx]
	mov	bx,Zeta3bps[bx]
	call	RVDecomOne	; decompress.
	stosw

	; Get third 16 bit code word.
	;
	mov	ax,si		; save last 2 bits.
	xchg	al,ah
	mov	cl,3
	shl	ax,cl
	and	ah,6
	mov	si,Qdata2

	; Check if routine is looking for silence codewords.
	;
	cmp	SLFlag,0
	je	NoSL2

	; Routine should look for silence codewords.  Check if
	; a codeword is found.
	;
	cmp	si,SL3CW
	jne	NoSL2

	; Routine found a silence codeword.  Cannot return the number
	; of zero data points back to the application at this time.
	; Set a flag to remind us that the first word on the next
	; group of three words will contain the number of data points.
	;
	mov	SLSilenceOn,1

	; Return number of code words consumed in AX, and the
	; number of decompressed data points produced in DX.
	;
	mov	ax,3
	mov	dx,di
	sub	dx,WORD PTR [YnPtr]
	shr	dx,1
	ret

NoSL2:
	; No silence codeword found.  Work on the second word by
	; unpacking ADPCM codewords.  A fragment of the last
	; ADPCM codeword remains in SI.
	;
	mov	bx,si		; left shift for index.
	mov	cl,3
	shl	bl,cl
	or	bl,ah
	and	bx,0eh

	SaveIQLevel	bx
	mov	ax,ML3bps[bx]
	mov	bx,Zeta3bps[bx]
	call	RVDecomOne	; decompress.
	stosw

	mov	bx,si		; shift by 1, then left shift for index.
	and	bx,0eh

	SaveIQLevel	bx
	mov	ax,ML3bps[bx]
	mov	bx,Zeta3bps[bx]
	call	RVDecomOne	; decompress.
	stosw

	mov	bx,si		; shift by 4, then left shift for index.
	mov	cl,3
	shr	bl,cl
	and	bx,0eh

	SaveIQLevel	bx
	mov	ax,ML3bps[bx]
	mov	bx,Zeta3bps[bx]
	call	RVDecomOne	; decompress.
	stosw

	mov	bx,si 		; shift by 7, then left shift for index.
	mov	cl,2
	shl	bx,cl
	mov	bl,bh
	and	bx,0eh

	SaveIQLevel	bx
	mov	ax,ML3bps[bx]
	mov	bx,Zeta3bps[bx]
	call	RVDecomOne	; decompress.
	stosw

	mov	bx,si 		; shift by 10, then left shift for index.
	mov	bl,bh
	shr	bl,1
	and	bx,0eh

	SaveIQLevel	bx
	mov	ax,ML3bps[bx]
	mov	bx,Zeta3bps[bx]
	call	RVDecomOne	; decompress.
	stosw

	mov	bx,si 		; shift by 13, then left shift for index.
	mov	bl,bh
	mov	cl,4
	shr	bl,cl
	and	bx,0eh

	SaveIQLevel	bx
	mov	ax,ML3bps[bx]
	mov	bx,Zeta3bps[bx]
	call	RVDecomOne	; decompress.
	stosw

	; Return number of code words consumed in AX, and the
	; number of decompressed data points produced in DX.
	;
	mov	ax,3
	mov	dx,di
	sub	dx,WORD PTR [YnPtr]
	shr	dx,1
	ret

RVDecom3bps	ENDP

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; RVDecom4bps
;
; Decode 3 codewords, Qdata0, Qdata1, and Qdata2.  Store 12 results
; at the location pointed to YnPtr.  Use the callback function to
; return the amount of silence to the calling routine if a silence
; codeword is detected.  Public routine.
;
; Function Prototype:
;
; unsigned long pascal
;    RVDecom4bps(
;    	unsigned int Qdata0,unsigned int Qdata1,unsigned int Qdata2,
;	   unsigned int far *YnPtr);
;
; Outputs:
;
; AX = Qdata words processed, ranges from 0 to 3.
; DX = number of decompressed words produced.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	EVEN
RVDecom4bps	PROC USES SI DI, Qdata0 : WORD, Qdata1 : WORD, \
			         Qdata2 : WORD, YnPtr : FAR PTR WORD

	; Check if the routine is waiting for the second word in
	; a silence deletion codeword - the number of zero data
	; points.
	;
	cmp	SLSilenceOn,0
	je	NoSLOn

	call	RVDecomReset		; reset the decoder proper.

	; Routine was waiting for the second word in a silence
	; deletion codeword - the number of zero data points.
	; Return this value back to the application.
	;
	push	Qdata0
	call	SLCallBack

	; Return number of code words consumed in AX, and the
	; number of decompressed data points produced in DX.
	;
	mov	ax,1
	sub	dx,dx
	ret

NoSLOn:
	; Load destination pointer.
	;
	les	di,YnPtr

	; Get first 16 bit code word.
	;
	mov	si,Qdata0

	; Check if routine is looking for silence codewords.
	;
	cmp	SLFlag,0
	je	NoSL0

	; Routine should look for silence codewords.  Check if
	; a codeword is found.
	;
	cmp	si,SL4CW
	jne	NoSL0

	call	RVDecomReset		; reset the decoder proper.

	; Routine found a silence codeword.  Return the the number
	; of zero data points back to the application.
	;
	push	Qdata1
	call	SLCallBack

	; Return number of code words consumed in AX, and the
	; number of decompressed data points produced in DX.
	;
	mov	ax,2
	mov	dx,di
	sub	dx,WORD PTR [YnPtr]
	shr	dx,1
	ret

NoSL0:
	; No silence codeword found.  Work on the first word by
	; unpacking ADPCM codewords.
	;
	mov	bx,si		; left shift for index.
	shl	bl,1
	and	bx,1eh

	SaveIQLevel	bx
	mov	ax,ML4bps[bx]
	mov	bx,Zeta4bps[bx]
	call	RVDecomOne	; decompress.
	stosw

	mov	bx,si		; shift by 4, then left shift for index.
	mov	cl,3
	shr	bl,cl
	and	bx,1eh

	SaveIQLevel	bx
	mov	ax,ML4bps[bx]
	mov	bx,Zeta4bps[bx]
	call	RVDecomOne	; decompress.
	stosw

	mov	bx,si		; shift by 8, then left shift for index.
	mov	bl,bh
	shl	bl,1
	and	bx,1eh

	SaveIQLevel	bx
	mov	ax,ML4bps[bx]
	mov	bx,Zeta4bps[bx]
	call	RVDecomOne	; decompress.
	stosw

	mov	bx,si		; shift by 12, then left shift for index.
	mov	bl,bh
	mov	cl,3
	shr	bl,cl
	and	bx,1eh

	SaveIQLevel	bx
	mov	ax,ML4bps[bx]
	mov	bx,Zeta4bps[bx]
	call	RVDecomOne	; decompress.
	stosw

	; Get second 16 bit code word.
	;
	mov	si,Qdata1
	
	; Check if routine is looking for silence codewords.
	;
	cmp	SLFlag,0
	je	NoSL1

	; Routine should look for silence codewords.  Check if
	; a codeword is found.
	;
	cmp	si,SL4CW
	jne	NoSL1

	call	RVDecomReset		; reset the decoder proper.

	; Routine found a silence codeword.  Return the the number
	; of zero data points back to the application.
	;
	push	Qdata2
	call	SLCallBack

	; Return number of code words consumed in AX, and the
	; number of decompressed data points produced in DX.
	;
	mov	ax,3
	mov	dx,di
	sub	dx,WORD PTR [YnPtr]
	shr	dx,1
	ret

NoSL1:
	; No silence codeword found.  Work on the second word by
	; unpacking ADPCM codewords.
	;
	mov	bx,si		; left shift for index.
	shl	bl,1
	and	bx,1eh

	SaveIQLevel	bx
	mov	ax,ML4bps[bx]
	mov	bx,Zeta4bps[bx]
	call	RVDecomOne	; decompress.
	stosw

	mov	bx,si		; shift by 4, then left shift for index.
	mov	cl,3
	shr	bl,cl
	and	bx,1eh

	SaveIQLevel	bx
	mov	ax,ML4bps[bx]
	mov	bx,Zeta4bps[bx]
	call	RVDecomOne	; decompress.
	stosw

	mov	bx,si		; shift by 8, then left shift for index.
	mov	bl,bh
	shl	bl,1
	and	bx,1eh

	SaveIQLevel	bx
	mov	ax,ML4bps[bx]
	mov	bx,Zeta4bps[bx]
	call	RVDecomOne	; decompress.
	stosw

	mov	bx,si		; shift by 12, then left shift for index.
	mov	bl,bh
	mov	cl,3
	shr	bl,cl
	and	bx,1eh

	SaveIQLevel	bx
	mov	ax,ML4bps[bx]
	mov	bx,Zeta4bps[bx]
	call	RVDecomOne	; decompress.
	stosw

	; Get third 16 bit code word.
	;
	mov	si,Qdata2

	; Check if routine is looking for silence codewords.
	;
	cmp	SLFlag,0
	je	NoSL2

	; Routine should look for silence codewords.  Check if
	; a codeword is found.
	;
	cmp	si,SL4CW
	jne	NoSL2

	; Routine found a silence codeword.  Cannot return the number
	; of zero data points back to the application at this time.
	; Set a flag to remind us that the first word on the next
	; group of three words will contain the number of data points.
	;
	mov	SLSilenceOn,1

	; Return number of code words consumed in AX, and the
	; number of decompressed data points produced in DX.
	;
	mov	ax,3
	mov	dx,di
	sub	dx,WORD PTR [YnPtr]
	shr	dx,1
	ret

NoSL2:
	; No silence codeword found.  Work on the second word by
	; unpacking ADPCM codewords.
	;
	mov	bx,si		; left shift for index.
	shl	bl,1
	and	bx,1eh

	SaveIQLevel	bx
	mov	ax,ML4bps[bx]
	mov	bx,Zeta4bps[bx]
	call	RVDecomOne	; decompress.
	stosw

	mov	bx,si		; shift by 4, then left shift for index.
	mov	cl,3
	shr	bl,cl
	and	bx,1eh

	SaveIQLevel	bx
	mov	ax,ML4bps[bx]
	mov	bx,Zeta4bps[bx]
	call	RVDecomOne	; decompress.
	stosw

	mov	bx,si		; shift by 8, then left shift for index.
	mov	bl,bh
	shl	bl,1
	and	bx,1eh

	SaveIQLevel	bx
	mov	ax,ML4bps[bx]
	mov	bx,Zeta4bps[bx]
	call	RVDecomOne	; decompress.
	stosw

	mov	bx,si		; shift by 12, then left shift for index.
	mov	bl,bh
	mov	cl,3
	shr	bl,cl
	and	bx,1eh

	SaveIQLevel	bx
	mov	ax,ML4bps[bx]
	mov	bx,Zeta4bps[bx]
	call	RVDecomOne	; decompress.
	stosw

	; Return number of code words consumed in AX, and the
	; number of decompressed data points produced in DX.
	;
	mov	ax,3
	mov	dx,di
	sub	dx,WORD PTR [YnPtr]
	shr	dx,1
	ret

RVDecom4bps	ENDP

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; RVDecom2bps
;
; Decode 3 codewords, Qdata0, Qdata1, and Qdata2.  Store 24 results
; at the location pointed to YnPtr.  Use the callback function to
; return the amount of silence to the calling routine if a silence
; codeword is detected.  Public routine.
;
; Function Prototype:
;
; unsigned long pascal
;    RVDecom2bps(
;    	unsigned int Qdata0,unsigned int Qdata1,unsigned int Qdata2,
;	   unsigned int far *YnPtr);
;
; Outputs:
;
; AX = Qdata words processed, ranges from 0 to 3.
; DX = number of decompressed words produced.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	EVEN
RVDecom2bps	PROC USES SI DI, Qdata0 : WORD, Qdata1 : WORD, \
			         Qdata2 : WORD, YnPtr : FAR PTR WORD

	; Check if the routine is waiting for the second word in
	; a silence deletion codeword - the number of zero data
	; points.
	;
	cmp	SLSilenceOn,0
	je	NoSLOn

	call	RVDecomReset		; reset the decoder proper.

	; Routine was waiting for the second word in a silence
	; deletion codeword - the number of zero data points.
	; Return this value back to the application.
	;
	push	Qdata0
	call	SLCallBack

	; Return number of code words consumed in AX, and the
	; number of decompressed data points produced in DX.
	;
	mov	ax,1
	sub	dx,dx
	ret

NoSLOn:
	; Load destination pointer.
	;
	les	di,YnPtr

	; Get first 16 bit code word.
	;
	mov	si,Qdata0

	; Check if routine is looking for silence codewords.
	;
	cmp	SLFlag,0
	je	NoSL0

	; Routine should look for silence codewords.  Check if
	; a codeword is found.
	;
	cmp	si,SL2CW
	jne	NoSL0

	call	RVDecomReset		; reset the decoder proper.

	; Routine found a silence codeword.  Return the the number
	; of zero data points back to the application.
	;
	push	Qdata1
	call	SLCallBack

	; Return number of code words consumed in AX, and the
	; number of decompressed data points produced in DX.
	;
	mov	ax,2
	mov	dx,di
	sub	dx,WORD PTR [YnPtr]
	shr	dx,1
	ret

NoSL0:
	; No silence codeword found.  Work on the first word by
	; unpacking ADPCM codewords.
	;
	mov	bx,si		; left shift for index.
	shl	bl,1
	and	bx,06h

	SaveIQLevel	bx
	mov	ax,ML2bps[bx]
	mov	bx,Zeta2bps[bx]
	call	RVDecomOne	; decompress.
	stosw

	mov	bx,si		; shift by 2, then left shift for index.
	shr	bl,1
	and	bx,06h

	SaveIQLevel	bx
	mov	ax,ML2bps[bx]
	mov	bx,Zeta2bps[bx]
	call	RVDecomOne	; decompress.
	stosw

	mov	bx,si		; shift by 4, then left shift for index.
	mov	cl,3
	shr	bl,cl
	and	bx,06h

	SaveIQLevel	bx
	mov	ax,ML2bps[bx]
	mov	bx,Zeta2bps[bx]
	call	RVDecomOne	; decompress.
	stosw

	mov	bx,si		; shift by 6, then left shift for index.
	mov	cl,3
	shl	bx,cl
	mov	bl,bh
	and	bx,06h

	SaveIQLevel	bx
	mov	ax,ML2bps[bx]
	mov	bx,Zeta2bps[bx]
	call	RVDecomOne	; decompress.
	stosw

	mov	bx,si		; shift by 8, then left shift for index.
	mov	bl,bh
	mov	si,bx		; save some moves in the future
	shl	bl,1
	and	bx,06h

	SaveIQLevel	bx
	mov	ax,ML2bps[bx]
	mov	bx,Zeta2bps[bx]
	call	RVDecomOne	; decompress.
	stosw

	mov	bx,si		; shift by 10, then left shift for index.
	shr	bl,1
	and	bx,06h

	SaveIQLevel	bx
	mov	ax,ML2bps[bx]
	mov	bx,Zeta2bps[bx]
	call	RVDecomOne	; decompress.
	stosw

	mov	bx,si		; shift by 12, then left shift for index.
	mov	cl,3
	shr	bl,cl
	and	bx,06h

	SaveIQLevel	bx
	mov	ax,ML2bps[bx]
	mov	bx,Zeta2bps[bx]
	call	RVDecomOne	; decompress.
	stosw

	mov	bx,si		; shift by 14, then left shift for index.
	mov	cl,3
	shl	bx,cl
	mov	bl,bh
	and	bx,06h

	SaveIQLevel	bx
	mov	ax,ML2bps[bx]
	mov	bx,Zeta2bps[bx]
	call	RVDecomOne	; decompress.
	stosw

	; Get second 16 bit code word.
	;
	mov	si,Qdata1
	
	; Check if routine is looking for silence codewords.
	;
	cmp	SLFlag,0
	je	NoSL1

	; Routine should look for silence codewords.  Check if
	; a codeword is found.
	;
	cmp	si,SL2CW
	jne	NoSL1

	call	RVDecomReset		; reset the decoder proper.

	; Routine found a silence codeword.  Return the the number
	; of zero data points back to the application.
	;
	push	Qdata2
	call	SLCallBack

	; Return number of code words consumed in AX, and the
	; number of decompressed data points produced in DX.
	;
	mov	ax,3
	mov	dx,di
	sub	dx,WORD PTR [YnPtr]
	shr	dx,1
	ret

NoSL1:
	; No silence codeword found.  Work on the second word by
	; unpacking ADPCM codewords.
	;
	mov	bx,si		; left shift for index.
	shl	bl,1
	and	bx,06h

	SaveIQLevel	bx
	mov	ax,ML2bps[bx]
	mov	bx,Zeta2bps[bx]
	call	RVDecomOne	; decompress.
	stosw

	mov	bx,si		; shift by 2, then left shift for index.
	shr	bl,1
	and	bx,06h

	SaveIQLevel	bx
	mov	ax,ML2bps[bx]
	mov	bx,Zeta2bps[bx]
	call	RVDecomOne	; decompress.
	stosw

	mov	bx,si		; shift by 4, then left shift for index.
	mov	cl,3
	shr	bl,cl
	and	bx,06h

	SaveIQLevel	bx
	mov	ax,ML2bps[bx]
	mov	bx,Zeta2bps[bx]
	call	RVDecomOne	; decompress.
	stosw

	mov	bx,si		; shift by 6, then left shift for index.
	mov	cl,3
	shl	bx,cl
	mov	bl,bh
	and	bx,06h

	SaveIQLevel	bx
	mov	ax,ML2bps[bx]
	mov	bx,Zeta2bps[bx]
	call	RVDecomOne	; decompress.
	stosw

	mov	bx,si		; shift by 8, then left shift for index	.
	mov	bl,bh
	mov	si,bx		; save some moves in the future
	shl	bl,1
	and	bx,06h

	SaveIQLevel	bx
	mov	ax,ML2bps[bx]
	mov	bx,Zeta2bps[bx]
	call	RVDecomOne	; decompress.
	stosw

	mov	bx,si		; shift by 10, then left shift for index .
	shr	bl,1
	and	bx,06h

	SaveIQLevel	bx
	mov	ax,ML2bps[bx]
	mov	bx,Zeta2bps[bx]
	call	RVDecomOne	; decompress.
	stosw

	mov	bx,si		; shift by 12, then left shift for index.
	mov	cl,3
	shr	bl,cl
	and	bx,06h

	SaveIQLevel	bx
	mov	ax,ML2bps[bx]
	mov	bx,Zeta2bps[bx]
	call	RVDecomOne	; decompress.
	stosw

	mov	bx,si		; shift by 14, then left shift for index.
	mov	cl,3
	shl	bx,cl
	mov	bl,bh
	and	bx,06h

	SaveIQLevel	bx
	mov	ax,ML2bps[bx]
	mov	bx,Zeta2bps[bx]
	call	RVDecomOne	; decompress.
	stosw

	; Get third 16 bit code word.
	;
	mov	si,Qdata2

	; Check if routine is looking for silence codewords.
	;
	cmp	SLFlag,0
	je	NoSL2

	; Routine should look for silence codewords.  Check if
	; a codeword is found.
	;
	cmp	si,SL2CW
	jne	NoSL2

	; Routine found a silence codeword.  Cannot return the number
	; of zero data points back to the application at this time.
	; Set a flag to remind us that the first word on the next
	; group of three words will contain the number of data points.
	;
	mov	SLSilenceOn,1

	; Return number of code words consumed in AX, and the
	; number of decompressed data points produced in DX.
	;
	mov	ax,3
	mov	dx,di
	sub	dx,WORD PTR [YnPtr]
	shr	dx,1
	ret

NoSL2:
	; No silence codeword found.  Work on the second word by
	; unpacking ADPCM codewords.
	;
	mov	bx,si		; left shift for index.
	shl	bl,1
	and	bx,06h

	SaveIQLevel	bx
	mov	ax,ML2bps[bx]
	mov	bx,Zeta2bps[bx]
	call	RVDecomOne	; decompress.
	stosw

	mov	bx,si		; shift by 2, then left shift for index.
	shr	bl,1
	and	bx,06h

	SaveIQLevel	bx
	mov	ax,ML2bps[bx]
	mov	bx,Zeta2bps[bx]
	call	RVDecomOne	; decompress.
	stosw

	mov	bx,si		; shift by 4, then left shift for index.
	mov	cl,3
	shr	bl,cl
	and	bx,06h

	SaveIQLevel	bx
	mov	ax,ML2bps[bx]
	mov	bx,Zeta2bps[bx]
	call	RVDecomOne	; decompress.
	stosw

	mov	bx,si		; shift by 6, then left shift for index.
	mov	cl,3
	shl	bx,cl
	mov	bl,bh
	and	bx,06h

	SaveIQLevel	bx
	mov	ax,ML2bps[bx]
	mov	bx,Zeta2bps[bx]
	call	RVDecomOne	; decompress.
	stosw

	mov	bx,si		; shift by 8, then left shift for index.
	mov	bl,bh
	mov	si,bx		; save some moves in the future
	shl	bl,1
	and	bx,06h

	SaveIQLevel	bx
	mov	ax,ML2bps[bx]
	mov	bx,Zeta2bps[bx]
	call	RVDecomOne	; decompress.
	stosw

	mov	bx,si		; shift by 10, then left shift for index.
	shr	bl,1
	and	bx,06h

	SaveIQLevel	bx
	mov	ax,ML2bps[bx]
	mov	bx,Zeta2bps[bx]
	call	RVDecomOne	; decompress.
	stosw

	mov	bx,si		; shift by 12, then left shift for index.
	mov	cl,3
	shr	bl,cl
	and	bx,06h

	SaveIQLevel	bx
	mov	ax,ML2bps[bx]
	mov	bx,Zeta2bps[bx]
	call	RVDecomOne	; decompress.
	stosw

	mov	bx,si		; shift by 14, then left shift for index.
	mov	cl,3
	shl	bx,cl
	mov	bl,bh
	and	bx,06h

	SaveIQLevel	bx
	mov	ax,ML2bps[bx]
	mov	bx,Zeta2bps[bx]
	call	RVDecomOne	; decompress.
	stosw

	; Return number of code words consumed in AX, and the
	; number of decompressed data points produced in DX.
	;
	mov	ax,3
	mov	dx,di
	sub	dx,WORD PTR [YnPtr]
	shr	dx,1
	ret

RVDecom2bps	ENDP

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Compression!!
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

.DATA

CD4X_LENGTH	EQU	7
CD3X_LENGTH	EQU	3
CD2X_LENGTH	EQU	1

; 16 level adaptive quantizer 4 bits per sample input constants.
;
cd4x		dw	00843h		;Nu9 = 0.2582/4
		dw	010B8h		;Nu10 = 0.5224/4
		dw	01996h		;Nu11 = 0.7996/4
		dw	0232Bh		;Nu12 = 1.099/4,
		dw	02DFCh		;Nu13 = 1.437/4,
		dw	03B02h		;Nu14 = 1.844/4,
		dw	04CD6h		;Nu15 = 2.401/4,

; 8 level adaptive quantizer 3 bits per sample input constants.
;
cd3x		dw	01005h		;Nu5 = 0.5006/4
		dw	0219Ah		;Nu6 = 1.050/4
		dw	037F0h		;Nu7 = 1.748/4

; 4 level adaptive quantizer 2 bits per sample input constants.
;
cd2x		dw	01F69h		;Nu3 = 0.9816/4

; Current X(n).
;
adc16		dw	0

; X(n-1).
;
adc16z1		dw	0

; Xp+z(n).
;
NewXpz		dw	0

; QOut, compressed data.
;
SaveQ		dw	0

.CODE

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; RVComInit
;
; Reset local variables for the ADPCM encoder.  Public routine.
;
; Function Prototype:
;
; void pascal
;    RVComInit(unsigned int bps);
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	EVEN
RVComInit	PROC bps : WORD

	; The compressor uses part of the decompression routines,
	; initialize its local variables too.
	;
	sub	ax,ax
	push	ax
	push	bps
	mov	ax,SEG @code:SLDummy
	push	ax
	mov	ax,OFFSET @code:SLDummy
	push	ax
	call	RVDecomInit

	; Initialize the compression routines local variables.
	;
	sub	ax,ax
	mov	adc16,ax
	mov	adc16z1,ax
	mov	NewXpz,ax
	mov	SaveQ,ax

	ret
RVComInit	ENDP

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; RVComOne3bps
;
; Code a word into a 3 BPS codeword.  Local to this module.
;
; Input:
;
; AX = X(n)
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	EVEN
RVComOne3bps	PROC USES SI DI

	; Save X(n) for later.
	;
	mov	adc16,ax

	; Compute Xp+z(n).
	;
	sub	cx,cx
	call	XpzCalc

	; Xp+z(n) done, save for later.
	;
	mov	NewXpz,cx

	; Use the emphasis filter on Xp+z(n).
	;
IFNDEF	DEBUG_COM
	mov	ax,PDEMPCF
	imul	adc16z1			; X(n-1).
	adjust	dx,ax
	add	bx,ax
	adc	cx,dx
	clip	cx,bx
ENDIF

	mov	ax,adc16
	sub	cx,ax			; X(n).
	clip	cx

	; Make ABS(X(n) - Xp+z(n) - (PDEMPZ)X(n-1)) in BX.
	;
	mov	bx,cx			; make another copy.
	jns	NoNt

	neg	bx

NoNt:
	; Make X(n) - Xp+z(n) - (PDEMPZ)X(n-1).
	;
	neg	cx

	; Make new X(n-1).
	;
	mov	adc16z1,ax

	; Find Q(n) from a table lookup.  Use several short cuts.
	; First search half the table (ignore negative numbers, hence
	; the ABS() in BX).  Second, avoid an unnecessary
	; multiplication by zero.
	;
	mov	si,OFFSET @data:cd3x
	mov	di,LastNu
	mov	cl,CD3X_LENGTH

	; Q(n) represents a interval under a Gaussian distribution
	; curve.  The shape of the curve is modified by Nu.  Find
	; the appropriate interval by scanning a table.
	;
	; Q(n) is found when Nu times the interval is greater than
	; the table lookup value.
	;
LoopSigma:
	lodsw
	imul	di

	; Decimal adjust, round, and multiple by four.
	;
	; Do not change this sequence!
	;
	adjust	dx,ax
	round	dx,ax
	shl	dx,1			; multiply by four.
	clip	dx,<>,Ov1
	shl	dx,1
	clip	dx

Ov1:
	cmp	bx,dx
	jl	YtQout

	; No match.  Check if there are more table entries to examine.
	;
	dec	cl
	jnz	LoopSigma

	; No more table entries.  Assume the largest table value.
	;
	add	si,2

YtQout:
	; Interval found, make Q(n) (compressed data) from index in SI.
	;
	sub	si,OFFSET @data:cd3x
	sar	si,1

	mov	ax,CD3X_LENGTH

	; Offset depends on if X(n) - Xp+z(n) - (PDEMPZ)X(n-1) was
	; positive or negative.
	;
	or	ch,ch
	js	NegQOut

	add	si,ax		; number positive.
	jmp	SHORT updQ

NegQOut:
	sub	si,ax		; number negative.
	dec	si
	neg	si

updQ:
	; Q(n) found. Save for later, other house keeping chores
	; are necessary.
	;
	mov	SaveQ,si

	; Update Nu.
	;
	shl	si,1			; make an index.
	mov	ax,ML3bps[si]
	imul	di			; di = LastNu.

	; Decimal adjust, round, and multiple by four.
	;
	; Do not change this sequence!
	;
	adjust	dx,ax
	round	dx,ax
	shl	dx,1			; multiply by four.
	clip	dx,<>,Ov2
	shl	dx,1
	clip	dx

Ov2:
	; Check Nu bounds.
	;
	cmp	dx,QDLMN
	jg	GDelta

	mov	dx,QDLMN
	jmp	SHORT RVdoFini

GDelta:
	mov	ax,QDelayMX
	cmp	dx,ax
	jle	RVdoFini

	mov	dx,ax

RVdoFini:
	mov	LastNu,dx		; save new Nu.

	; Make a new inverse quantizer value.
	;
	mov	ax,Zeta3bps[si]
	imul	di			; si = LastNu.

	; Decimal adjust, round, and multiple by four.
	;
	; Do not change this sequence!
	;
	adjust	dx,ax
	round	dx,ax
	shl	dx,1			; multiply by four.
	clip	dx,<>,Ov3
	shl	dx,1
	clip	dx

Ov3:
	; Update predictor filter coefficients.
	;
	mov	cx,dx			; use inverse quantizer output.
	call	pzPred

	; Update delay line at the a1(n) table entry position.
	;
	mov	di,QDataIndex
	addm	di,0eh
	mov	ax,NewXpz
	add	ax,cx
	clip	ax
	mov	Qdata[di],ax		; drop b6, now a1 Qdata.

	; Update delay line at the b1(n) table entry position.
	;
	mov	QDataIndex,di		; save for next iteration.
	addm	di,4
	mov	Qdata[di],cx		; drop a2, now b1 Qdata.

	; Return QOut.
	;
	mov	ax,SaveQ
	ret

RVComOne3bps	ENDP

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; RVCom3bps
;
; Encode 16 words pointed to by XnPtr.  Put 3 results at
; location pointed to YnPtr.  Public routine.
;
; Function Prototype:
;
; unsigned long pascal
;    RVCom3bps(
;       unsigned int *XnPtr,unsigned int far *YnPtr);
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	EVEN
RVCom3bps	PROC USES SI DI, XnPtr : PTR WORD, YnPtr : FAR PTR WORD

	; Load source pointer to uncompressed data.
	;
	mov	si,XnPtr

	; Make first compressed codeword.
	;
	lodsw
	call	RVComOne3bps	; produce compressed codeword.
	mov	di,ax

	lodsw			; shift 3 places.
	call	RVComOne3bps
	mov	cl,3		; pack the codeword.
	shl	ax,cl
	or	di,ax

	lodsw			; shift 6 places.
	call	RVComOne3bps	; produce compressed codeword.
	xchg	ah,al		; pack the codeword.
	mov	cl,2
	shr	ax,cl
	or	di,ax

	lodsw			; shift 9 places.
	call	RVComOne3bps	; produce compressed codeword.
	xchg	ah,al		; pack the codeword.
	shl	ax,1
	or	di,ax

	lodsw			; shift 12 places.
	call	RVComOne3bps	; produce compressed codeword.
	xchg	ah,al		; pack the codeword.
	mov	cl,4
	shl	ax,cl
	or	di,ax

	lodsw			; shift 15 places.
	call	RVComOne3bps	; produce compressed codeword.
	shl	di,1		; pack the codeword.
	shr	ax,1
	mov	bx,ax		; save code fragment for later.
	rcr	di,1

	; Save first codeword.
	;
	mov	ax,di
	les	di,YnPtr
	stosw
	mov	WORD PTR [YnPtr],di

	; Make second compressed codeword.  Start with
	; code fragment from last finished code word.
	;
	mov	di,bx
	lodsw			; shift 2 places.
	call	RVComOne3bps	; produce compressed codeword.
	mov	cl,2		; pack the codeword.
	shl	ax,cl
	or	di,ax

	lodsw			; shift 5 places.
	call	RVComOne3bps	; produce compressed codeword.
	xchg	ah,al		; pack the codeword.
	mov	cl,3
	shr	ax,cl
	or	di,ax

	lodsw			; shift 8 places.
	call	RVComOne3bps	; produce compressed codeword.
	xchg	ah,al		; pack the codeword.
	or	di,ax

	lodsw			; shift 11 places.
	call	RVComOne3bps	; produce compressed codeword.
	xchg	ah,al		; pack the codeword.
	mov	cl,3
	shl	ax,cl
	or	di,ax

	lodsw			; shift 14 places.
	call	RVComOne3bps	; produce compressed codeword.
	xchg	ah,al		; pack the codeword.
	mov	cl,2
	shr	ax,cl
	xchg	ah,al
	mov	bx,ax		; save code fragment for later.
	sub	bh,bh
	sub	al,al
	or	ax,di

	; Save second codeword.
	;
	les	di,YnPtr
	stosw
	mov	WORD PTR [YnPtr],di

	; Make third compressed codeword.  Start with
	; code fragment from last finished code word.
	;
	mov	di,bx
	lodsw			; shift 1 place.
	call	RVComOne3bps	; produce compressed codeword.
	shl	ax,1		; pack the codeword.
	or	di,ax

	lodsw			; shift 4 places.
	call	RVComOne3bps	; produce compressed codeword.
	mov	cl,4		; pack the codeword.
	shl	ax,cl
	or	di,ax

	lodsw			; shift 7 places.
	call	RVComOne3bps	; produce compressed codeword.
	xchg	ah,al		; pack the codeword.
	shr	ax,1
	or	di,ax

	lodsw			; shift 10 places.
	call	RVComOne3bps	; produce compressed codeword.
	xchg	ah,al		; pack the codeword.
	mov	cl,2
	shl	ax,cl
	or	di,ax

	lodsw			; shift 13 places.
	call	RVComOne3bps	; produce compressed codeword.
	xchg	ah,al		; pack the codeword.
	mov	cl,3
	shr	ax,cl
	xchg	ah,al
	or	di,ax

	; Save third codeword.
	;
	mov	ax,di
	les	di,YnPtr
	stosw

	; Return number of non compressed data points consumed in AX,
	; and the number of compressed data points produced in DX.
	;
	mov	ax,si
	sub	ax,XnPtr
	shr	ax,1
	mov	dx,3
	ret

RVCom3bps	ENDP

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; RVComOne4bps
;
; Code a word into a 4 BPS codeword.  Local to this module.
;
; Input:
;
; AX = X(n)
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	EVEN
RVComOne4bps	PROC USES SI DI

	; Save X(n) for later.
	;
	mov	adc16,ax

	; Compute Xp+z(n).
	;
	sub	cx,cx
	call	XpzCalc

	; Xp+z(n) done, save later.
	;
	mov	NewXpz,cx

	; Use the emphasis filter on Xp+z(n).
	;
IFNDEF	DEBUG_COM
	mov	ax,PDEMPCF
	imul	adc16z1			; X(n-1).
	adjust	dx,ax
	add	bx,ax
	adc	cx,dx
	clip	cx,bx
ENDIF

	mov	ax,adc16
	sub	cx,ax			; X(n).
	clip	cx

	; Make ABS(X(n) - Xp+z(n) - (PDEMPZ)X(n-1)) in BX.
	;
	mov	bx,cx
	jns	NoNt

	neg	bx

NoNt:
	; Make X(n) - Xp+z(n) - (PDEMPZ)X(n-1).
	;
	neg	cx

	; Make new X(n-1).
	;
	mov	adc16z1,ax

	; Find Q(n) from a table lookup.  Use several short cuts.
	; First search half the table (ignore negative numbers, hence
	; the ABS() in BX).  Second, avoid an unnecessary
	; multiplication by zero.
	;
	mov	si,OFFSET @data:cd4x
	mov	di,LastNu
	mov	cl,CD4X_LENGTH

	; Q(n) represents a interval under a Gaussian distribution
	; curve.  The shape of the curve is modified by Nu.  Find
	; the appropriate interval by scanning a table.
	;
	; Q(n) is found when Nu times the interval is greater than
	; the table lookup value.
	;
LoopSigma:
	lodsw
	imul	di

	; Decimal adjust, round, and multiple by four.
	;
	; Do not change this sequence!
	;
	adjust	dx,ax
	round	dx,ax
	shl	dx,1			; multiply by four
	clip	dx,<>,Ov1
	shl	dx,1
	clip	dx

Ov1:
	cmp	bx,dx
	jl	YtQout

	; No match.  Check if there are more table entries to examine.
	;
	dec	cl
	jnz	LoopSigma

	; No more table entries.  Assume the largest table value.
	;
	add	si,2

YtQout:
	; Interval found, make Q(n) (compressed data) from index in SI.
	;
	sub	si,OFFSET @data:cd4x
	sar	si,1

	mov	ax,CD4X_LENGTH

	; Offset depends on if X(n) - Xp+z(n) - (PDEMPZ)X(n-1) was
	; positive or negative.
	;
	or	ch,ch
	js	NegQOut

	add	si,ax		; number positive.
	jmp	SHORT updQ

NegQOut:
	sub	si,ax		; number negative.
	dec	si
	neg	si

updQ:
	; Q(n) found. Save for later, other house keeping chores
	; are necessary.
	;
	mov	SaveQ,si

	; Update Nu.
	;
	shl	si,1			; make an index.
	mov	ax,ML4bps[si]
	imul	di			; di = LastNu.

	; Decimal adjust, round, and multiple by four.
	;
	; Do not change this sequence!
	;
	adjust	dx,ax
	round	dx,ax
	shl	dx,1			; multiply by four
	clip	dx,<>,Ov2
	shl	dx,1
	clip	dx

Ov2:
	; Check Nu bounds.
	;
	cmp	dx,QDLMN
	jg	GDelta

	mov	dx,QDLMN
	jmp	SHORT RVdoFini

GDelta:
	mov	ax,QDelayMX
	cmp	dx,ax
	jle	RVdoFini

	mov	dx,ax

RVdoFini:
	mov	LastNu,dx		; save new Nu.

	; Make a new inverse quantizer value.
	;
	mov	ax,Zeta4bps[si]
	imul	di			; si = LastNu

	; Decimal adjust, round, and multiple by four.
	;
	; Do not change this sequence!
	;
	adjust	dx,ax
	round	dx,ax
	shl	dx,1			; multiply by four.
	clip	dx,<>,Ov3
	shl	dx,1
	clip	dx

Ov3:
	; Update predictor filter coefficients.
	;
	mov	cx,dx			; use inverse quantizer output.
	call	pzPred

	; Update delay line at the a1(n) table entry position.
	;
	mov	di,QDataIndex
	addm	di,0eh
	mov	ax,NewXpz
	add	ax,cx
	clip	ax
	mov	Qdata[di],ax		; drop b6, now a1 Qdata.

	; Update delay line at the b1(n) table entry position.
	;
	mov	QDataIndex,di		; save for next iteration.
	addm	di,4
	mov	Qdata[di],cx		; drop a2, now b1 Qdata.

	; Return QOut.
	;
	mov	ax,SaveQ
	ret

RVComOne4bps	ENDP

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; RVCom4bps
;
; Encode 12 words pointed to by XnPtr.  Put 3 results at
; location pointed to YnPtr.  Public routine.
;
; Function Prototype:
;
; unsigned long pascal
;    RVCom4bps(
;       unsigned int *XnPtr,unsigned int far *YnPtr);
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	EVEN
RVCom4bps	PROC USES SI DI, XnPtr : PTR WORD, YnPtr : FAR PTR WORD

	; Load source pointer to uncompressed data.
	;
	mov	si,XnPtr

	; Make first compressed codeword.
	;
	lodsw
	call	RVComOne4bps	; produce compressed codeword.
	mov	di,ax

	lodsw			; shift 4 places.
	call	RVComOne4bps	; produce compressed codeword.
	mov	cl,4		; pack the codeword.
	shl	ax,cl
	or	di,ax

	lodsw			; shift 8 places.
	call	RVComOne4bps	; produce compressed codeword.
	xchg	ah,al		; pack the codeword.
	or	di,ax

	lodsw			; shift 12 places.
	call	RVComOne4bps	; produce compressed codeword.
	xchg	ah,al		; pack the codeword.
	mov	cl,4
	shl	ax,cl
	or	ax,di

	; Save first codeword.
	;
	les	di,YnPtr
	stosw
	mov	WORD PTR [YnPtr],di

	; Make second compressed codeword.
	;
	lodsw
	call	RVComOne4bps	; produce compressed codeword.
	mov	di,ax		; pack the codeword.

	lodsw			; shift 4 places.
	call	RVComOne4bps	; produce compressed codeword.
	mov	cl,4		; pack the codeword.
	shl	ax,cl
	or	di,ax

	lodsw			; shift 8 places.
	call	RVComOne4bps	; produce compressed codeword.
	xchg	ah,al		; pack the codeword.
	or	di,ax

	lodsw			; shift 12 places.
	call	RVComOne4bps	; produce compressed codeword.
	xchg	ah,al		; pack the codeword.
	mov	cl,4
	shl	ax,cl
	or	ax,di

	; Save second codeword.
	;
	les	di,YnPtr
	stosw
	mov	WORD PTR [YnPtr],di

	; Make third compressed codeword.
	;
	lodsw
	call	RVComOne4bps	; produce compressed codeword.
	mov	di,ax		; pack the codeword.

	lodsw			; shift 4 places.
	call	RVComOne4bps	; produce compressed codeword.
	mov	cl,4		; pack the codeword.
	shl	ax,cl
	or	di,ax

	lodsw			; shift 8 places.
	call	RVComOne4bps	; produce compressed codeword.
	xchg	ah,al		; pack the codeword.
	or	di,ax

	lodsw			; shift 12 places.
	call	RVComOne4bps	; produce compressed codeword.
	xchg	ah,al		; pack the codeword.
	mov	cl,4
	shl	ax,cl
	or	ax,di

	; Save third codeword.
	;
	les	di,YnPtr
	stosw

	; Return number of non compressed data points consumed in AX,
	; and the number of compressed data points produced in DX.
	;
	mov	ax,si
	sub	ax,XnPtr
	shr	ax,1
	mov	dx,3
	ret

RVCom4bps	ENDP

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; RVComOne2bps
;
; Code a word into a 2 BPS codeword.  Local to this module.
;
; Input:
;
; AX = X(n)
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	EVEN
RVComOne2bps	PROC USES SI DI

	; Save X(n) for later.
	;
	mov	adc16,ax

	; Compute Xp+z(n).
	;
	sub	cx,cx
	call	XpzCalc

	; Xp+z(n) done, save for later.
	;
	mov	NewXpz,cx

	; Use the emphasis filter on Xp+z(n).
	;
IFNDEF	DEBUG_COM
	mov	ax,PDEMPCF
	imul	adc16z1			; X(n-1).
	adjust	dx,ax
	add	bx,ax
	adc	cx,dx
	clip	cx,bx
ENDIF

	mov	ax,adc16
	sub	cx,ax			; X(n).
	clip	cx

	; Make ABS(X(n) - Xp+z(n) - (PDEMPZ)X(n-1)) in BX.
	;
	mov	bx,cx
	jns	NoNt

	neg	bx

NoNt:
	; Make X(n) - Xp+z(n) - (PDEMPZ)X(n-1).
	;
	neg	cx

	; Make new X(n-1).
	;
	mov	adc16z1,ax

	; Find Q(n) from a table lookup.  Use several short cuts.
	; First search half the table (ignore negative numbers, hence
	; the ABS() in BX).  Second, avoid an unnecessary
	; multiplication by zero.
	;
	mov	si,OFFSET @data:cd2x
	mov	di,LastNu
	mov	cl,CD2X_LENGTH

	; Q(n) represents a interval under a Gaussian distribution
	; curve.  The shape of the curve is modified by Nu.  Find
	; the appropriate interval by scanning a table.
	;
	; Q(n) is found when Nu times the interval is greater than
	; the table lookup value.
	;
LoopSigma:
	lodsw
	imul	di

	; Decimal adjust, round, and multiple by four.
	;
	; Do not change this sequence!
	;
	adjust	dx,ax
	round	dx,ax
	shl	dx,1			; multiply by four
	clip	dx,<>,Ov1
	shl	dx,1
	clip	dx

Ov1:
	cmp	bx,dx
	jl	YtQout

	; No match.  Check if there are more table entries to examine.
	;
	dec	cl
	jnz	LoopSigma

	; No more table entries.  Assume the largest table value.
	;
	add	si,2

YtQout:
	; Interval found, make Q(n) (compressed data) from index in SI.
	;
	sub	si,OFFSET @data:cd2x
	sar	si,1

	mov	ax,CD2X_LENGTH

	; Offset depends on if X(n) - Xp+z(n) - (PDEMPZ)X(n-1) was
	; positive or negative.
	;
	or	ch,ch
	js	NegQOut

	add	si,ax		; number positive.
	jmp	SHORT updQ

NegQOut:
	sub	si,ax		; number negative.
	dec	si
	neg	si

updQ:
	; Q(n) found. Save for later, other house keeping chores
	; are necessary.
	;
	mov	SaveQ,si

	; Update Nu.
	;
	shl	si,1			; make an index.
	mov	ax,ML2bps[si]
	imul	di			; di = LastNu.

	; Decimal adjust, round, and multiple by four.
	;
	; Do not change this sequence!
	;
	adjust	dx,ax
	round	dx,ax
	shl	dx,1			; multiply by four.
	clip	dx,<>,Ov2
	shl	dx,1
	clip	dx

Ov2:
	; Check Nu bounds.
	;
	cmp	dx,QDLMN
	jg	GDelta

	mov	dx,QDLMN
	jmp	SHORT RVdoFini

GDelta:
	mov	ax,QDelayMX
	cmp	dx,ax
	jle	RVdoFini

	mov	dx,ax

RVdoFini:
	mov	LastNu,dx		; save new Nu.

	; Make a new inverse quantizer value.
	;
	mov	ax,Zeta2bps[si]
	imul	di			; si = LastNu.

	; Decimal adjust, round, and multiple by four.
	;
	; Do not change this sequence!
	;
	adjust	dx,ax
	round	dx,ax
	shl	dx,1			; multiply by four.
	clip	dx,<>,Ov3
	shl	dx,1
	clip	dx

Ov3:
	; Update predictor filter coefficients.
	;
	mov	cx,dx			; use inverse quantizer output.
	call	pzPred

	; Update delay line at the a1(n) table entry position.
	;
	mov	di,QDataIndex
	addm	di,0eh
	mov	ax,NewXpz
	add	ax,cx
	clip	ax
	mov	Qdata[di],ax		; drop b6, now a1 Qdata.

	; Update delay line at the b1(n) table entry position.
	;
	mov	QDataIndex,di		; save for next iteration.
	addm	di,4
	mov	Qdata[di],cx		; drop a2, now b1 Qdata.

	; Return QOut.
	;
	mov	ax,SaveQ
	ret

RVComOne2bps	ENDP

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; RVCom2bps
;
; Encode 24 words pointed to by XnPtr.  Put 3 results at
; location pointed to YnPtr.  Public routine.
;
; Function Prototype:
;
; unsigned long pascal
;    RVCom2bps(
;       unsigned int *XnPtr,unsigned int far *YnPtr);
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	EVEN
RVCom2bps	PROC USES SI DI, XnPtr : PTR WORD, YnPtr : FAR PTR WORD

	; Load source pointer to uncompressed data.
	;
	mov	si,XnPtr

	; Make first compressed codeword.
	;
	lodsw
	call	RVComOne2bps	; produce compressed codeword.
	mov	di,ax		; pack the codeword.

	lodsw			; shift 2 places.
	call	RVComOne2bps	; produce compressed codeword.
	mov	cl,2		; pack the codeword.
	shl	ax,cl
	or	di,ax

	lodsw			; shift 4 places.
	call	RVComOne2bps	; produce compressed codeword.
	mov	cl,4		; pack the codeword.
	shl	ax,cl
	or	di,ax

	lodsw			; shift 6 places.
	call	RVComOne2bps	; produce compressed codeword.
	xchg	ah,al		; pack the codeword.
	mov	cl,2
	shr	ax,cl
	or	di,ax

	lodsw			; shift 8 places.
	call	RVComOne2bps	; produce compressed codeword.
	xchg	ah,al		; pack the codeword.
	or	di,ax

	lodsw			; shift 10 places.
	call	RVComOne2bps	; produce compressed codeword.
	xchg	ah,al		; pack the codeword.
	mov	cl,2
	shl	ax,cl
	or	di,ax

	lodsw			; shift 12 places.
	call	RVComOne2bps	; produce compressed codeword.
	xchg	ah,al		; pack the codeword.
	mov	cl,4
	shl	ax,cl
	or	di,ax

	lodsw			; shift 14 places.
	call	RVComOne2bps	; produce compressed codeword.
	xchg	ah,al		; pack the codeword.
	mov	cl,2
	shr	ax,cl
	xchg	ah,al
	or	ax,di

	; Save first codeword.
	;
	les	di,YnPtr
	stosw
	mov	WORD PTR [YnPtr],di

	; Make second compressed codeword.
	;
	lodsw
	call	RVComOne2bps	; produce compressed codeword.
	mov	di,ax		; pack the codeword.

	lodsw			; shift 2 places.
	call	RVComOne2bps	; produce compressed codeword.
	mov	cl,2		; pack the codeword.
	shl	ax,cl
	or	di,ax

	lodsw			; shift 4 places.
	call	RVComOne2bps	; produce compressed codeword.
	mov	cl,4		; pack the codeword.
	shl	ax,cl
	or	di,ax

	lodsw			; shift 6 places.
	call	RVComOne2bps	; produce compressed codeword.
	xchg	ah,al		; pack the codeword.
	mov	cl,2
	shr	ax,cl
	or	di,ax

	lodsw			; shift 8 places.
	call	RVComOne2bps	; produce compressed codeword.
	xchg	ah,al		; pack the codeword.
	or	di,ax

	lodsw			; shift 10 places.
	call	RVComOne2bps	; produce compressed codeword.
	xchg	ah,al		; pack the codeword.
	mov	cl,2
	shl	ax,cl
	or	di,ax

	lodsw			; shift 12 places.
	call	RVComOne2bps	; produce compressed codeword.
	xchg	ah,al		; pack the codeword.
	mov	cl,4
	shl	ax,cl
	or	di,ax

	lodsw			; shift 14 places.
	call	RVComOne2bps	; produce compressed codeword.
	xchg	ah,al		; pack the codeword.
	mov	cl,2
	shr	ax,cl
	xchg	ah,al
	or	ax,di

	; Save second codeword.
	;
	les	di,YnPtr
	stosw
	mov	WORD PTR [YnPtr],di

	; Make third compressed codeword.
	;
	lodsw
	call	RVComOne2bps	; produce compressed codeword.
	mov	di,ax		; pack the codeword.

	lodsw			; shift 2 places.
	call	RVComOne2bps	; produce compressed codeword.
	mov	cl,2		; pack the codeword.
	shl	ax,cl
	or	di,ax

	lodsw			; shift 4 places.
	call	RVComOne2bps	; produce compressed codeword.
	mov	cl,4		; pack the codeword.
	shl	ax,cl
	or	di,ax

	lodsw			; shift 6 places.
	call	RVComOne2bps	; produce compressed codeword.
	xchg	ah,al		; pack the codeword.
	mov	cl,2
	shr	ax,cl
	or	di,ax

	lodsw			; shift 8 places.
	call	RVComOne2bps	; produce compressed codeword.
	xchg	ah,al		; pack the codeword.
	or	di,ax

	lodsw			; shift 10 places.
	call	RVComOne2bps	; produce compressed codeword.
	xchg	ah,al		; pack the codeword.
	mov	cl,2
	shl	ax,cl
	or	di,ax

	lodsw			; shift 12 places.
	call	RVComOne2bps	; produce compressed codeword.
	xchg	ah,al		; pack the codeword.
	mov	cl,4
	shl	ax,cl
	or	di,ax

	lodsw			; shift 14 places.
	call	RVComOne2bps	; produce compressed codeword.
	xchg	ah,al		; pack the codeword.
	mov	cl,2
	shr	ax,cl
	xchg	ah,al
	or	ax,di

	; Save third codeword.
	;
	les	di,YnPtr
	stosw

	; Return number of non compressed data points consumed in AX,
	; and the number of compressed data points produced in DX.
	;
	mov	ax,si
	sub	ax,XnPtr
	shr	ax,1
	mov	dx,3
	ret

RVCom2bps	ENDP

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Interpolation!!  Change sample rate from 7200 to 14.4k, 21.1k,
;		   11.025k, or 22.05k Hz
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	.DATA

	; Filter delay line length
	;
LPQORDER	EQU	64


IFNDEF Kaiser

IFDEF BIG_FILTERS

	; Sampling= 14400  Cutoff= 3600  Taps= 64  Multiplication= 2.0
	; Window= 0
	; Atten @ 3600 Hz = 0.4950
	;
LPInter14400	dw	0fe2ch
		dw	0fe1dh
		dw	01f4h
		dw	0205h
		dw	0fde8h
		dw	0fdd4h
		dw	0242h
		dw	025ah
		dw	0fd8dh
		dw	0fd71h
		dw	02aeh
		dw	02cfh
		dw	0fd0ch
		dw	0fce3h
		dw	034ah
		dw	037dh
		dw	0fc49h
		dw	0fc07h
		dw	0444h
		dw	049ch
		dw	0fafeh
		dw	0fa84h
		dw	0610h
		dw	06c7h
		dw	0f852h
		dw	0f723h
		dw	0a79h
		dw	0ccdh
		dw	0ef8ah
		dw	0e8f4h
		dw	02669h
		dw	0733ch

ELSE

	; Sampling= 14400  Cutoff= 3600  Taps= 32  Multiplication=  2.00
	; Window= 0
	; Atten @ 3600 Hz = 0.4901
	;
LPInter14400	dw	0fc49h
		dw	0fc07h
		dw	0444h
		dw	049ch
		dw	0fafeh
		dw	0fa84h
		dw	0610h
		dw	06c7h
		dw	0f852h
		dw	0f723h
		dw	0a79h
		dw	0ccdh
		dw	0ef8ah
		dw	0e8f4h
		dw	02669h
		dw	0733ch

ENDIF

ELSE

IFDEF BIG_FILTERS

	; Sampling= 14400  Cutoff= 3160  Taps= 64  Multiplication= 2.0
	; Window= 5  Beta=     6.0000
	; Atten @ 3592 Hz = 0.0010
	;
LPInter14400	dw	0fffbh
		dw	0fff1h
		dw	04h
		dw	025h
		dw	0bh
		dw	0ffc1h
		dw	0ffcdh
		dw	052h
		dw	07bh
		dw	0ffbah
		dw	0ff21h
		dw	02h
		dw	0149h
		dw	093h
		dw	0fe6dh
		dw	0fe7bh
		dw	0182h
		dw	02c6h
		dw	0ff2bh
		dw	0fbd7h
		dw	0ff48h
		dw	055ah
		dw	0360h
		dw	0fa24h
		dw	0f8a3h
		dw	04f8h
		dw	0d20h
		dw	0fea9h
		dw	0e9a4h
		dw	0f65eh
		dw	02f71h
		dw	06797h

ELSE

	; Sampling= 14400  Cutoff= 2725  Taps= 32  Multiplication=  2.00
	; Window= 5  Beta=     6.0000
	; Atten @ 3592 Hz = 0.0015
	;
LPInter14400	dw	0fff8h
		dw	0ffcbh
		dw	0ffdch
		dw	08eh
		dw	0116h
		dw	0ffd9h
		dw	0fd5eh
		dw	0fd84h
		dw	02adh
		dw	0787h
		dw	02a2h
		dw	0f482h
		dw	0eec7h
		dw	051bh
		dw	033c0h
		dw	05b04h

ENDIF

ENDIF


IFNDEF Kaiser

IFDEF BIG_FILTERS

	; Sampling= 21600  Cutoff= 3600  Taps= 96  Multiplication= 3.00
	; Window= 0
	; Atten @ 3601 Hz = 0.4882
	;
LPInter21100	dw	0feb7h
		dw	0fd60h
		dw	0fea9h
		dw	015fh
		dw	02cfh
		dw	0170h
		dw	0fe88h
		dw	0fcfch
		dw	0fe74h
		dw	0196h
		dw	0342h
		dw	01ach
		dw	0fe48h
		dw	0fc76h
		dw	0fe2dh
		dw	01e1h
		dw	03e1h
		dw	0200h
		dw	0fdeeh
		dw	0fbb7h
		dw	0fdc8h
		dw	024eh
		dw	04cbh
		dw	027eh
		dw	0fd67h
		dw	0fa92h
		dw	0fd29h
		dw	02fbh
		dw	0644h
		dw	034dh
		dw	0fc82h
		dw	0f898h
		dw	0fc0fh
		dw	0436h
		dw	090dh
		dw	04e3h
		dw	0fab0h
		dw	0f45ch
		dw	0f992h
		dw	0730h
		dw	0104ch
		dw	0966h
		dw	0f4e4h
		dw	0e4d7h
		dw	0ee8ah
		dw	01872h
		dw	0517ch
		dw	07a3ah

ELSE

	; Sampling= 21600  Cutoff= 3600  Taps= 48  Multiplication=  3.00
	; Window= 0
	; Atten @ 3601 Hz = 0.4883
	;
LPInter21100	dw	0fd67h
		dw	0fa92h
		dw	0fd29h
		dw	02fbh
		dw	0644h
		dw	034dh
		dw	0fc82h
		dw	0f898h
		dw	0fc0fh
		dw	0436h
		dw	090dh
		dw	04e3h
		dw	0fab0h
		dw	0f45ch
		dw	0f992h
		dw	0730h
		dw	0104ch
		dw	0966h
		dw	0f4e4h
		dw	0e4d7h
		dw	0ee8ah
		dw	01872h
		dw	0517ch
		dw	07a3ah

ENDIF

ELSE

IFDEF BIG_FILTERS

	; Sampling= 21600  Cutoff= 3170  Taps= 96  Multiplication= 3.00
	; Window= 5  Beta=     6.0000
	; Atten @ 3596 Hz = 0.0014
	;
LPInter21100	dw	0ffffh
		dw	0fff4h
		dw	0ffefh
		dw	0fffch
		dw	015h
		dw	029h
		dw	01bh
		dw	0ffebh
		dw	0ffb8h
		dw	0ffb7h
		dw	0fffeh
		dw	062h
		dw	08dh
		dw	041h
		dw	0ff9eh
		dw	0ff22h
		dw	0ff4eh
		dw	02bh
		dw	0121h
		dw	0153h
		dw	05dh
		dw	0fed3h
		dw	0fdf4h
		dw	0feb5h
		dw	0ceh
		dw	02b0h
		dw	029ch
		dw	030h
		dw	0fd0bh
		dw	0fbcfh
		dw	0fe06h
		dw	027fh
		dw	05c7h
		dw	04adh
		dw	0ff27h
		dw	0f90ch
		dw	0f7a7h
		dw	0fd6ah
		dw	071bh
		dw	0d29h
		dw	08ffh
		dw	0fae8h
		dw	0ebf3h
		dw	0e997h
		dw	0fd0eh
		dw	02402h
		dw	04fd6h
		dw	06cb9h

ELSE

	; Sampling= 21600  Cutoff= 2750  Taps= 48  Multiplication=  3.00
	; Window= 5  Beta=     6.0000
	; Atten @ 3596 Hz = 0.0020
	;
LPInter21100	dw	0ffffh
		dw	0ffe3h
		dw	0ffbeh
		dw	0ffbeh
		dw	010h
		dw	0adh
		dw	012ah
		dw	0eeh
		dw	0ffa8h
		dw	0fdcfh
		dw	0fcaeh
		dw	0fdb0h
		dw	012fh
		dw	059bh
		dw	07d2h
		dw	04e8h
		dw	0fc93h
		dw	0f293h
		dw	0edd6h
		dw	0f525h
		dw	0b03h
		dw	02b19h
		dw	04b19h
		dw	05f12h

ENDIF

ENDIF


IFNDEF Kaiser

IFDEF BIG_FILTERS

	; Sampling= 352800  Cutoff= 3600  Taps= 1568  Multiplication= 49.0
	; Window= 0
	; Atten @ 3528 Hz = 0.8009
	;
LPInter22050	dw	0ffech
		dw	0ffc2h
		dw	0ff98h
		dw	0ff6fh
		dw	0ff46h
		dw	0ff1eh
		dw	0fef6h
		dw	0fed0h
		dw	0feabh
		dw	0fe87h
		dw	0fe65h
		dw	0fe44h
		dw	0fe25h
		dw	0fe07h
		dw	0fdech
		dw	0fdd3h
		dw	0fdbch
		dw	0fda8h
		dw	0fd96h
		dw	0fd86h
		dw	0fd79h
		dw	0fd6fh
		dw	0fd67h
		dw	0fd62h
		dw	0fd60h
		dw	0fd60h
		dw	0fd63h
		dw	0fd69h
		dw	0fd72h
		dw	0fd7eh
		dw	0fd8ch
		dw	0fd9dh
		dw	0fdb0h
		dw	0fdc6h
		dw	0fddeh
		dw	0fdf9h
		dw	0fe15h
		dw	0fe34h
		dw	0fe55h
		dw	0fe78h
		dw	0fe9ch
		dw	0fec2h
		dw	0feeah
		dw	0ff12h
		dw	0ff3ch
		dw	0ff67h
		dw	0ff92h
		dw	0ffbeh
		dw	0ffeah
		dw	016h
		dw	042h
		dw	06fh
		dw	09bh
		dw	0c7h
		dw	0f1h
		dw	011bh
		dw	0144h
		dw	016ch
		dw	0193h
		dw	01b7h
		dw	01dah
		dw	01fch
		dw	021bh
		dw	0238h
		dw	0253h
		dw	026bh
		dw	0281h
		dw	0295h
		dw	02a5h
		dw	02b3h
		dw	02bfh
		dw	02c7h
		dw	02cch
		dw	02cfh
		dw	02ceh
		dw	02cbh
		dw	02c5h
		dw	02bbh
		dw	02afh
		dw	02a0h
		dw	028eh
		dw	0279h
		dw	0262h
		dw	0248h
		dw	022ch
		dw	020dh
		dw	01ech
		dw	01c9h
		dw	01a4h
		dw	017dh
		dw	0154h
		dw	012ah
		dw	0ffh
		dw	0d2h
		dw	0a4h
		dw	076h
		dw	047h
		dw	017h
		dw	0ffe9h
		dw	0ffb9h
		dw	0ff89h
		dw	0ff5ah
		dw	0ff2bh
		dw	0fefdh
		dw	0fed0h
		dw	0fea4h
		dw	0fe79h
		dw	0fe50h
		dw	0fe29h
		dw	0fe03h
		dw	0fddfh
		dw	0fdbeh
		dw	0fd9fh
		dw	0fd82h
		dw	0fd67h
		dw	0fd50h
		dw	0fd3bh
		dw	0fd29h
		dw	0fd1ah
		dw	0fd0eh
		dw	0fd05h
		dw	0fcffh
		dw	0fcfch
		dw	0fcfch
		dw	0fd00h
		dw	0fd07h
		dw	0fd11h
		dw	0fd1eh
		dw	0fd2eh
		dw	0fd41h
		dw	0fd57h
		dw	0fd70h
		dw	0fd8ch
		dw	0fdaah
		dw	0fdcbh
		dw	0fdefh
		dw	0fe14h
		dw	0fe3ch
		dw	0fe66h
		dw	0fe92h
		dw	0febfh
		dw	0feeeh
		dw	0ff1eh
		dw	0ff4fh
		dw	0ff81h
		dw	0ffb4h
		dw	0ffe7h
		dw	019h
		dw	04dh
		dw	080h
		dw	0b3h
		dw	0e5h
		dw	0117h
		dw	0148h
		dw	0177h
		dw	01a5h
		dw	01d1h
		dw	01fch
		dw	0225h
		dw	024bh
		dw	0270h
		dw	0291h
		dw	02b1h
		dw	02cdh
		dw	02e7h
		dw	02fdh
		dw	0311h
		dw	0321h
		dw	032fh
		dw	0338h
		dw	033fh
		dw	0342h
		dw	0342h
		dw	033eh
		dw	0337h
		dw	032ch
		dw	031eh
		dw	030dh
		dw	02f8h
		dw	02e0h
		dw	02c5h
		dw	02a7h
		dw	0286h
		dw	0263h
		dw	023dh
		dw	0214h
		dw	01e9h
		dw	01bch
		dw	018ch
		dw	015bh
		dw	0129h
		dw	0f5h
		dw	0c0h
		dw	08ah
		dw	053h
		dw	01bh
		dw	0ffe5h
		dw	0ffadh
		dw	0ff75h
		dw	0ff3eh
		dw	0ff07h
		dw	0fed1h
		dw	0fe9dh
		dw	0fe69h
		dw	0fe37h
		dw	0fe07h
		dw	0fdd9h
		dw	0fdach
		dw	0fd83h
		dw	0fd5bh
		dw	0fd36h
		dw	0fd14h
		dw	0fcf5h
		dw	0fcd9h
		dw	0fcc1h
		dw	0fcabh
		dw	0fc99h
		dw	0fc8bh
		dw	0fc80h
		dw	0fc79h
		dw	0fc76h
		dw	0fc76h
		dw	0fc7ah
		dw	0fc81h
		dw	0fc8dh
		dw	0fc9ch
		dw	0fcafh
		dw	0fcc5h
		dw	0fcdfh
		dw	0fcfch
		dw	0fd1dh
		dw	0fd40h
		dw	0fd67h
		dw	0fd90h
		dw	0fdbdh
		dw	0fdebh
		dw	0fe1dh
		dw	0fe50h
		dw	0fe85h
		dw	0febch
		dw	0fef5h
		dw	0ff2fh
		dw	0ff6ah
		dw	0ffa6h
		dw	0ffe2h
		dw	01eh
		dw	05bh
		dw	098h
		dw	0d4h
		dw	0110h
		dw	014ah
		dw	0184h
		dw	01bch
		dw	01f3h
		dw	0228h
		dw	025ah
		dw	028bh
		dw	02b9h
		dw	02e4h
		dw	030ch
		dw	0332h
		dw	0354h
		dw	0372h
		dw	038eh
		dw	03a5h
		dw	03b9h
		dw	03c9h
		dw	03d5h
		dw	03ddh
		dw	03e1h
		dw	03e1h
		dw	03ddh
		dw	03d4h
		dw	03c8h
		dw	03b7h
		dw	03a3h
		dw	038bh
		dw	036fh
		dw	034fh
		dw	032bh
		dw	0304h
		dw	02dah
		dw	02adh
		dw	027ch
		dw	0249h
		dw	0213h
		dw	01dbh
		dw	01a0h
		dw	0164h
		dw	0126h
		dw	0e6h
		dw	0a5h
		dw	063h
		dw	021h
		dw	0ffdfh
		dw	0ff9ch
		dw	0ff59h
		dw	0ff17h
		dw	0fed5h
		dw	0fe94h
		dw	0fe55h
		dw	0fe17h
		dw	0fddah
		dw	0fda0h
		dw	0fd68h
		dw	0fd32h
		dw	0fd00h
		dw	0fcd0h
		dw	0fca3h
		dw	0fc7ah
		dw	0fc54h
		dw	0fc32h
		dw	0fc14h
		dw	0fbfah
		dw	0fbe4h
		dw	0fbd2h
		dw	0fbc4h
		dw	0fbbbh
		dw	0fbb7h
		dw	0fbb6h
		dw	0fbbbh
		dw	0fbc4h
		dw	0fbd1h
		dw	0fbe3h
		dw	0fbfah
		dw	0fc14h
		dw	0fc33h
		dw	0fc56h
		dw	0fc7dh
		dw	0fca8h
		dw	0fcd7h
		dw	0fd09h
		dw	0fd3fh
		dw	0fd77h
		dw	0fdb3h
		dw	0fdf1h
		dw	0fe32h
		dw	0fe75h
		dw	0febah
		dw	0ff01h
		dw	0ff49h
		dw	0ff92h
		dw	0ffdbh
		dw	025h
		dw	06fh
		dw	0bah
		dw	0103h
		dw	014dh
		dw	0195h
		dw	01dch
		dw	0221h
		dw	0264h
		dw	02a5h
		dw	02e4h
		dw	031fh
		dw	0358h
		dw	038eh
		dw	03c0h
		dw	03eeh
		dw	0419h
		dw	043fh
		dw	0461h
		dw	047eh
		dw	0497h
		dw	04abh
		dw	04bbh
		dw	04c5h
		dw	04cbh
		dw	04cbh
		dw	04c6h
		dw	04bdh
		dw	04aeh
		dw	049ah
		dw	0482h
		dw	0464h
		dw	0442h
		dw	041bh
		dw	03efh
		dw	03bfh
		dw	038bh
		dw	0353h
		dw	0317h
		dw	02d8h
		dw	0295h
		dw	024fh
		dw	0207h
		dw	01bch
		dw	016eh
		dw	011fh
		dw	0ceh
		dw	07ch
		dw	029h
		dw	0ffd7h
		dw	0ff83h
		dw	0ff2fh
		dw	0fedch
		dw	0fe89h
		dw	0fe38h
		dw	0fde8h
		dw	0fd9ah
		dw	0fd4eh
		dw	0fd04h
		dw	0fcbdh
		dw	0fc79h
		dw	0fc39h
		dw	0fbfch
		dw	0fbc4h
		dw	0fb8fh
		dw	0fb5fh
		dw	0fb33h
		dw	0fb0ch
		dw	0faeah
		dw	0faceh
		dw	0fab7h
		dw	0faa5h
		dw	0fa98h
		dw	0fa92h
		dw	0fa91h
		dw	0fa96h
		dw	0faa0h
		dw	0fab1h
		dw	0fac6h
		dw	0fae2h
		dw	0fb03h
		dw	0fb2ah
		dw	0fb56h
		dw	0fb87h
		dw	0fbbdh
		dw	0fbf8h
		dw	0fc37h
		dw	0fc7bh
		dw	0fcc3h
		dw	0fd0fh
		dw	0fd5eh
		dw	0fdb1h
		dw	0fe06h
		dw	0fe5eh
		dw	0feb8h
		dw	0ff14h
		dw	0ff72h
		dw	0ffd1h
		dw	02fh
		dw	08fh
		dw	0efh
		dw	014eh
		dw	01adh
		dw	020ah
		dw	0266h
		dw	02c0h
		dw	0318h
		dw	036ch
		dw	03beh
		dw	040ch
		dw	0456h
		dw	049dh
		dw	04deh
		dw	051bh
		dw	0553h
		dw	0586h
		dw	05b3h
		dw	05dbh
		dw	05fch
		dw	0618h
		dw	062dh
		dw	063ch
		dw	0644h
		dw	0646h
		dw	0641h
		dw	0636h
		dw	0623h
		dw	060bh
		dw	05ebh
		dw	05c6h
		dw	059ah
		dw	0567h
		dw	052fh
		dw	04f1h
		dw	04adh
		dw	0464h
		dw	0416h
		dw	03c3h
		dw	036bh
		dw	0310h
		dw	02b0h
		dw	024dh
		dw	01e7h
		dw	017eh
		dw	0113h
		dw	0a6h
		dw	037h
		dw	0ffc9h
		dw	0ff59h
		dw	0fee9h
		dw	0fe79h
		dw	0fe0ah
		dw	0fd9dh
		dw	0fd31h
		dw	0fcc7h
		dw	0fc61h
		dw	0fbfdh
		dw	0fb9dh
		dw	0fb40h
		dw	0fae8h
		dw	0fa95h
		dw	0fa47h
		dw	0f9ffh
		dw	0f9bch
		dw	0f97fh
		dw	0f949h
		dw	0f91ah
		dw	0f8f1h
		dw	0f8d0h
		dw	0f8b6h
		dw	0f8a3h
		dw	0f898h
		dw	0f895h
		dw	0f89ah
		dw	0f8a6h
		dw	0f8bbh
		dw	0f8d7h
		dw	0f8fbh
		dw	0f926h
		dw	0f95ah
		dw	0f994h
		dw	0f9d6h
		dw	0fa1fh
		dw	0fa6fh
		dw	0fac5h
		dw	0fb21h
		dw	0fb84h
		dw	0fbebh
		dw	0fc58h
		dw	0fccah
		dw	0fd40h
		dw	0fdbah
		dw	0fe37h
		dw	0feb7h
		dw	0ff39h
		dw	0ffbeh
		dw	042h
		dw	0c9h
		dw	0150h
		dw	01d6h
		dw	025ch
		dw	02e0h
		dw	0363h
		dw	03e3h
		dw	0460h
		dw	04d9h
		dw	054eh
		dw	05bfh
		dw	062bh
		dw	0691h
		dw	06f0h
		dw	074ah
		dw	079ch
		dw	07e8h
		dw	082bh
		dw	0866h
		dw	089ah
		dw	08c4h
		dw	08e6h
		dw	08feh
		dw	090dh
		dw	0913h
		dw	090fh
		dw	0902h
		dw	08ebh
		dw	08cah
		dw	08a0h
		dw	086ch
		dw	082fh
		dw	07e9h
		dw	0799h
		dw	0741h
		dw	06e0h
		dw	0677h
		dw	0607h
		dw	058eh
		dw	050fh
		dw	0489h
		dw	03fdh
		dw	036bh
		dw	02d5h
		dw	023ah
		dw	019ah
		dw	0f8h
		dw	053h
		dw	0ffadh
		dw	0ff04h
		dw	0fe5bh
		dw	0fdb2h
		dw	0fd09h
		dw	0fc62h
		dw	0fbbdh
		dw	0fb1ah
		dw	0fa7ch
		dw	0f9e1h
		dw	0f94bh
		dw	0f8bah
		dw	0f830h
		dw	0f7ach
		dw	0f730h
		dw	0f6bch
		dw	0f650h
		dw	0f5edh
		dw	0f594h
		dw	0f544h
		dw	0f500h
		dw	0f4c6h
		dw	0f497h
		dw	0f474h
		dw	0f45ch
		dw	0f451h
		dw	0f452h
		dw	0f45fh
		dw	0f479h
		dw	0f49fh
		dw	0f4d2h
		dw	0f511h
		dw	0f55dh
		dw	0f5b5h
		dw	0f618h
		dw	0f688h
		dw	0f703h
		dw	0f789h
		dw	0f819h
		dw	0f8b4h
		dw	0f959h
		dw	0fa06h
		dw	0fabdh
		dw	0fb7bh
		dw	0fc41h
		dw	0fd0dh
		dw	0fddfh
		dw	0feb6h
		dw	0ff91h
		dw	06fh
		dw	0151h
		dw	0234h
		dw	0318h
		dw	03fch
		dw	04dfh
		dw	05c0h
		dw	069eh
		dw	0778h
		dw	084eh
		dw	091eh
		dw	09e7h
		dw	0aa9h
		dw	0b63h
		dw	0c13h
		dw	0cb9h
		dw	0d54h
		dw	0de3h
		dw	0e66h
		dw	0edch
		dw	0f43h
		dw	0f9dh
		dw	0fe7h
		dw	01021h
		dw	0104ch
		dw	01065h
		dw	0106eh
		dw	01065h
		dw	0104bh
		dw	0101fh
		dw	0fe2h
		dw	0f92h
		dw	0f31h
		dw	0ebdh
		dw	0e38h
		dw	0da2h
		dw	0cfah
		dw	0c41h
		dw	0b78h
		dw	0a9fh
		dw	09b7h
		dw	08c0h
		dw	07bbh
		dw	06a9h
		dw	058ah
		dw	0460h
		dw	032bh
		dw	01edh
		dw	0a6h
		dw	0ff58h
		dw	0fe04h
		dw	0fcaah
		dw	0fb4dh
		dw	0f9edh
		dw	0f88ch
		dw	0f72ch
		dw	0f5cch
		dw	0f470h
		dw	0f319h
		dw	0f1c7h
		dw	0f07ch
		dw	0ef3bh
		dw	0ee03h
		dw	0ecd7h
		dw	0ebb9h
		dw	0eaa9h
		dw	0e9a8h
		dw	0e8bah
		dw	0e7deh
		dw	0e716h
		dw	0e663h
		dw	0e5c7h
		dw	0e543h
		dw	0e4d7h
		dw	0e486h
		dw	0e44fh
		dw	0e435h
		dw	0e437h
		dw	0e458h
		dw	0e496h
		dw	0e4f4h
		dw	0e572h
		dw	0e610h
		dw	0e6ceh
		dw	0e7aeh
		dw	0e8afh
		dw	0e9d1h
		dw	0eb15h
		dw	0ec7ah
		dw	0ee01h
		dw	0efa9h
		dw	0f171h
		dw	0f35ah
		dw	0f562h
		dw	0f78ah
		dw	0f9d1h
		dw	0fc35h
		dw	0feb6h
		dw	0151h
		dw	0409h
		dw	06dah
		dw	09c3h
		dw	0cc3h
		dw	0fd9h
		dw	01303h
		dw	01640h
		dw	0198dh
		dw	01ceah
		dw	02054h
		dw	023cah
		dw	0274ah
		dw	02ad2h
		dw	02e5fh
		dw	031f1h
		dw	03585h
		dw	03919h
		dw	03cabh
		dw	04039h
		dw	043c1h
		dw	04741h
		dw	04ab7h
		dw	04e20h
		dw	0517ch
		dw	054c7h
		dw	057ffh
		dw	05b24h
		dw	05e33h
		dw	06129h
		dw	06406h
		dw	066c8h
		dw	0696ch
		dw	06bf1h
		dw	06e56h
		dw	0709ah
		dw	072bah
		dw	074b5h
		dw	0768bh
		dw	0783ah
		dw	079c0h
		dw	07b1eh
		dw	07c52h
		dw	07d5ch
		dw	07e3ah
		dw	07eech
		dw	07f72h
		dw	07fcch
		dw	07ff9h
			     
ELSE			     
			     
	; Sampling= 352800  Cutoff= 3600  Taps= 784  Multiplication= 47.00
	; Window= 0
	; Atten @ 3528 Hz = 0.6522
	;		     
LPInter22050	dw	0ffd8h
		dw	0ff88h    
		dw	0ff38h    
		dw	0fee8h    
		dw	0fe99h    
		dw	0fe4ah    
		dw	0fdfeh    
		dw	0fdb3h    
		dw	0fd6ah    
		dw	0fd23h    
		dw	0fcdfh
		dw	0fc9eh
		dw	0fc61h
		dw	0fc26h
		dw	0fbf0h
		dw	0fbbdh
		dw	0fb8fh
		dw	0fb65h
		dw	0fb40h
		dw	0fb20h
		dw	0fb04h
		dw	0faeeh
		dw	0faddh
		dw	0fad1h
		dw	0facbh
		dw	0facah
		dw	0faceh
		dw	0fad8h
		dw	0fae8h
		dw	0fafdh
		dw	0fb18h
		dw	0fb37h
		dw	0fb5ch
		dw	0fb86h
		dw	0fbb6h
		dw	0fbe9h
		dw	0fc22h
		dw	0fc5fh
		dw	0fca0h
		dw	0fce5h
		dw	0fd2eh
		dw	0fd7ah
		dw	0fdc9h
		dw	0fe1bh
		dw	0fe6fh
		dw	0fec6h
		dw	0ff1eh
		dw	0ff78h
		dw	0ffd3h
		dw	02dh
		dw	089h
		dw	0e5h
		dw	0141h
		dw	019ch
		dw	01f5h
		dw	024dh
		dw	02a3h
		dw	02f7h
		dw	0348h
		dw	0397h
		dw	03e2h
		dw	0429h
		dw	046ch
		dw	04abh
		dw	04e6h
		dw	051ch
		dw	054ch
		dw	0578h
		dw	059eh
		dw	05beh
		dw	05d8h
		dw	05ech
		dw	05fbh
		dw	0603h
		dw	0604h
		dw	0600h
		dw	05f5h
		dw	05e3h
		dw	05cch
		dw	05aeh
		dw	0589h
		dw	055fh
		dw	052fh
		dw	04f9h
		dw	04bdh
		dw	047ch
		dw	0436h
		dw	03ebh
		dw	039ch
		dw	0348h
		dw	02f0h
		dw	0294h
		dw	0235h
		dw	01d3h
		dw	016eh
		dw	0107h
		dw	09fh
		dw	035h
		dw	0ffcbh
		dw	0ff60h
		dw	0fef4h
		dw	0fe89h
		dw	0fe1fh
		dw	0fdb6h
		dw	0fd4eh
		dw	0fce9h
		dw	0fc86h
		dw	0fc27h
		dw	0fbcah
		dw	0fb72h
		dw	0fb1eh
		dw	0faceh
		dw	0fa83h
		dw	0fa3dh
		dw	0f9fdh
		dw	0f9c3h
		dw	0f98fh
		dw	0f962h
		dw	0f93bh
		dw	0f91bh
		dw	0f902h
		dw	0f8f0h
		dw	0f8e6h
		dw	0f8e3h
		dw	0f8e7h
		dw	0f8f3h
		dw	0f907h
		dw	0f922h
		dw	0f944h
		dw	0f96eh
		dw	0f99fh
		dw	0f9d7h
		dw	0fa17h
		dw	0fa5dh
		dw	0faa9h
		dw	0fafch
		dw	0fb54h
		dw	0fbb3h
		dw	0fc16h
		dw	0fc7fh
		dw	0fcech
		dw	0fd5dh
		dw	0fdd1h
		dw	0fe4ah
		dw	0fec4h
		dw	0ff42h
		dw	0ffc0h
		dw	040h
		dw	0c1h
		dw	0142h
		dw	01c3h
		dw	0243h
		dw	02c2h
		dw	033fh
		dw	03bah
		dw	0432h
		dw	04a6h
		dw	0517h
		dw	0583h
		dw	05eah
		dw	064ch
		dw	06a8h
		dw	06feh
		dw	074dh
		dw	0795h
		dw	07d6h
		dw	080fh
		dw	0840h
		dw	0868h
		dw	0889h
		dw	08a0h
		dw	08afh
		dw	08b4h
		dw	08b1h
		dw	08a4h
		dw	088eh
		dw	086eh
		dw	0846h
		dw	0814h
		dw	07dah
		dw	0796h
		dw	074ah
		dw	06f5h
		dw	0698h
		dw	0634h
		dw	05c8h
		dw	0554h
		dw	04dah
		dw	045ah
		dw	03d3h
		dw	0348h
		dw	02b7h
		dw	0222h
		dw	018ah
		dw	0eeh
		dw	04fh
		dw	0ffb0h
		dw	0ff0eh
		dw	0fe6ch
		dw	0fdcah
		dw	0fd28h
		dw	0fc88h
		dw	0fbe9h
		dw	0fb4eh
		dw	0fab5h
		dw	0fa21h
		dw	0f991h
		dw	0f906h
		dw	0f881h
		dw	0f803h
		dw	0f78ch
		dw	0f71ch
		dw	0f6b5h
		dw	0f656h
		dw	0f601h
		dw	0f5b5h
		dw	0f573h
		dw	0f53bh
		dw	0f50eh
		dw	0f4edh
		dw	0f4d6h
		dw	0f4cbh
		dw	0f4cch
		dw	0f4d9h
		dw	0f4f2h
		dw	0f516h
		dw	0f547h
		dw	0f584h
		dw	0f5cch
		dw	0f620h
		dw	0f680h
		dw	0f6ebh
		dw	0f761h
		dw	0f7e1h
		dw	0f86ch
		dw	0f900h
		dw	0f99eh
		dw	0fa45h
		dw	0faf4h
		dw	0fbaah
		dw	0fc68h
		dw	0fd2ch
		dw	0fdf5h
		dw	0fec4h
		dw	0ff96h
		dw	06bh
		dw	0143h
		dw	021dh
		dw	02f8h
		dw	03d2h
		dw	04ach
		dw	0584h
		dw	0659h
		dw	072ah
		dw	07f7h
		dw	08bfh
		dw	0980h
		dw	0a3ah
		dw	0aech
		dw	0b95h
		dw	0c34h
		dw	0cc9h
		dw	0d52h
		dw	0dd0h
		dw	0e40h
		dw	0ea4h
		dw	0efah
		dw	0f41h
		dw	0f79h
		dw	0fa1h
		dw	0fbah
		dw	0fc2h
		dw	0fbah
		dw	0fa1h
		dw	0f77h
		dw	0f3ch
		dw	0eefh
		dw	0e92h
		dw	0e23h
		dw	0da4h
		dw	0d13h
		dw	0c72h
		dw	0bc1h
		dw	0b00h
		dw	0a30h
		dw	0952h
		dw	0865h
		dw	076ah
		dw	0663h
		dw	0550h
		dw	0432h
		dw	030ah
		dw	01d9h
		dw	09fh
		dw	0ff5fh
		dw	0fe19h
		dw	0fccdh
		dw	0fb7eh
		dw	0fa2dh
		dw	0f8dah
		dw	0f788h
		dw	0f637h
		dw	0f4e9h
		dw	0f39fh
		dw	0f25bh
		dw	0f11eh
		dw	0efeah
		dw	0eebfh
		dw	0eda0h
		dw	0ec8dh
		dw	0eb88h
		dw	0ea92h
		dw	0e9adh
		dw	0e8dah
		dw	0e81ah
		dw	0e76fh
		dw	0e6d9h
		dw	0e65ah
		dw	0e5f3h
		dw	0e5a5h
		dw	0e571h
		dw	0e557h
		dw	0e55ah
		dw	0e579h
		dw	0e5b5h
		dw	0e60fh
		dw	0e687h
		dw	0e71fh
		dw	0e7d6h
		dw	0e8ach
		dw	0e9a3h
		dw	0eab9h
		dw	0ebf0h
		dw	0ed46h
		dw	0eebdh
		dw	0f053h
		dw	0f209h
		dw	0f3deh
		dw	0f5d1h
		dw	0f7e3h
		dw	0fa11h
		dw	0fc5ch
		dw	0fec3h
		dw	0143h
		dw	03deh
		dw	0692h
		dw	095dh
		dw	0c3eh
		dw	0f34h
		dw	0123ch
		dw	01557h
		dw	01882h
		dw	01bbch
		dw	01f02h
		dw	02254h
		dw	025afh
		dw	02912h
		dw	02c7bh
		dw	02fe7h
		dw	03356h
		dw	036c5h
		dw	03a31h
		dw	03d9ah
		dw	040fdh
		dw	04459h
		dw	047aah
		dw	04af0h
		dw	04e28h
		dw	05151h
		dw	05468h
		dw	0576ch
		dw	05a5ah
		dw	05d32h
		dw	05ff1h
		dw	06296h
		dw	0651eh
		dw	06789h
		dw	069d6h
		dw	06c01h
		dw	06e0bh
		dw	06ff2h
		dw	071b4h
		dw	07351h
		dw	074c8h
		dw	07618h
		dw	0773fh
		dw	0783eh
		dw	07913h
		dw	079beh
		dw	07a3fh
		dw	07a95h
		dw	07ac0h

ENDIF

ELSE

IFDEF BIG_FILTERS

	; Sampling= 352800  Cutoff= 3175  Taps= 1568  Multiplication= 49.0
	; Window= 5  Beta=     6.0000
	; Atten @ 3598 Hz = 0.0012
	;
LPInter22050	dw	03h
		dw	02h
		dw	02h
		dw	01h
		dw	01h
		dw	00h
		dw	00h
		dw	00h
		dw	0ffffh
		dw	0fffeh
		dw	0fffeh
		dw	0fffdh
		dw	0fffch
		dw	0fffbh
		dw	0fffah
		dw	0fffah
		dw	0fff9h
		dw	0fff8h
		dw	0fff7h
		dw	0fff6h
		dw	0fff6h
		dw	0fff5h
		dw	0fff4h
		dw	0fff3h
		dw	0fff3h
		dw	0fff2h
		dw	0fff1h
		dw	0fff0h
		dw	0fff0h
		dw	0ffefh
		dw	0ffefh
		dw	0ffeeh
		dw	0ffeeh
		dw	0ffedh
		dw	0ffedh
		dw	0ffedh
		dw	0ffedh
		dw	0ffech
		dw	0ffech
		dw	0ffech
		dw	0ffech
		dw	0ffech
		dw	0ffedh
		dw	0ffedh
		dw	0ffedh
		dw	0ffeeh
		dw	0ffeeh
		dw	0ffefh
		dw	0fff0h
		dw	0fff0h
		dw	0fff1h
		dw	0fff2h
		dw	0fff3h
		dw	0fff4h
		dw	0fff6h
		dw	0fff7h
		dw	0fff8h
		dw	0fffah
		dw	0fffbh
		dw	0fffdh
		dw	0fffeh
		dw	00h
		dw	01h
		dw	03h
		dw	04h
		dw	06h
		dw	08h
		dw	0ah
		dw	0ch
		dw	0eh
		dw	010h
		dw	012h
		dw	014h
		dw	016h
		dw	018h
		dw	019h
		dw	01bh
		dw	01dh
		dw	01fh
		dw	021h
		dw	022h
		dw	024h
		dw	025h
		dw	026h
		dw	028h
		dw	029h
		dw	02ah
		dw	02bh
		dw	02ch
		dw	02ch
		dw	02dh
		dw	02dh
		dw	02dh
		dw	02dh
		dw	02dh
		dw	02dh
		dw	02dh
		dw	02ch
		dw	02bh
		dw	02ah
		dw	029h
		dw	028h
		dw	027h
		dw	025h
		dw	023h
		dw	021h
		dw	01fh
		dw	01dh
		dw	01bh
		dw	018h
		dw	015h
		dw	012h
		dw	0fh
		dw	0ch
		dw	09h
		dw	06h
		dw	02h
		dw	00h
		dw	0fffch
		dw	0fff8h
		dw	0fff5h
		dw	0fff1h
		dw	0ffedh
		dw	0ffe9h
		dw	0ffe5h
		dw	0ffe2h
		dw	0ffdeh
		dw	0ffdah
		dw	0ffd6h
		dw	0ffd2h
		dw	0ffcfh
		dw	0ffcbh
		dw	0ffc8h
		dw	0ffc4h
		dw	0ffc1h
		dw	0ffbeh
		dw	0ffbbh
		dw	0ffb9h
		dw	0ffb6h
		dw	0ffb4h
		dw	0ffb2h
		dw	0ffb0h
		dw	0ffaeh
		dw	0ffach
		dw	0ffabh
		dw	0ffaah
		dw	0ffaah
		dw	0ffa9h
		dw	0ffa9h
		dw	0ffa9h
		dw	0ffaah
		dw	0ffabh
		dw	0ffach
		dw	0ffadh
		dw	0ffafh
		dw	0ffb1h
		dw	0ffb3h
		dw	0ffb6h
		dw	0ffb9h
		dw	0ffbch
		dw	0ffc0h
		dw	0ffc4h
		dw	0ffc8h
		dw	0ffcch
		dw	0ffd1h
		dw	0ffd6h
		dw	0ffdbh
		dw	0ffe0h
		dw	0ffe6h
		dw	0ffech
		dw	0fff2h
		dw	0fff8h
		dw	0fffeh
		dw	04h
		dw	0ah
		dw	011h
		dw	018h
		dw	01eh
		dw	025h
		dw	02ch
		dw	033h
		dw	03ah
		dw	040h
		dw	047h
		dw	04dh
		dw	054h
		dw	05ah
		dw	060h
		dw	066h
		dw	06bh
		dw	071h
		dw	076h
		dw	07bh
		dw	07fh
		dw	083h
		dw	087h
		dw	08ah
		dw	08dh
		dw	090h
		dw	092h
		dw	093h
		dw	094h
		dw	095h
		dw	095h
		dw	095h
		dw	094h
		dw	093h
		dw	091h
		dw	08fh
		dw	08ch
		dw	089h
		dw	085h
		dw	080h
		dw	07bh
		dw	076h
		dw	070h
		dw	06ah
		dw	063h
		dw	05ch
		dw	054h
		dw	04ch
		dw	043h
		dw	03ah
		dw	031h
		dw	028h
		dw	01eh
		dw	014h
		dw	09h
		dw	00h
		dw	0fff5h
		dw	0ffeah
		dw	0ffdfh
		dw	0ffd4h
		dw	0ffc9h
		dw	0ffbdh
		dw	0ffb2h
		dw	0ffa7h
		dw	0ff9ch
		dw	0ff91h
		dw	0ff87h
		dw	0ff7ch
		dw	0ff72h
		dw	0ff68h
		dw	0ff5fh
		dw	0ff56h
		dw	0ff4dh
		dw	0ff45h
		dw	0ff3dh
		dw	0ff36h
		dw	0ff2fh
		dw	0ff29h
		dw	0ff24h
		dw	0ff1fh
		dw	0ff1bh
		dw	0ff17h
		dw	0ff14h
		dw	0ff12h
		dw	0ff11h
		dw	0ff11h
		dw	0ff11h
		dw	0ff12h
		dw	0ff14h
		dw	0ff16h
		dw	0ff1ah
		dw	0ff1eh
		dw	0ff23h
		dw	0ff29h
		dw	0ff30h
		dw	0ff37h
		dw	0ff40h
		dw	0ff49h
		dw	0ff53h
		dw	0ff5dh
		dw	0ff69h
		dw	0ff74h
		dw	0ff81h
		dw	0ff8eh
		dw	0ff9ch
		dw	0ffaah
		dw	0ffb9h
		dw	0ffc8h
		dw	0ffd8h
		dw	0ffe8h
		dw	0fff9h
		dw	08h
		dw	019h
		dw	02ah
		dw	03ch
		dw	04dh
		dw	05eh
		dw	06fh
		dw	081h
		dw	092h
		dw	0a2h
		dw	0b3h
		dw	0c3h
		dw	0d3h
		dw	0e2h
		dw	0f1h
		dw	0ffh
		dw	010ch
		dw	0119h
		dw	0125h
		dw	0131h
		dw	013bh
		dw	0145h
		dw	014dh
		dw	0155h
		dw	015ch
		dw	0162h
		dw	0166h
		dw	016ah
		dw	016ch
		dw	016dh
		dw	016dh
		dw	016ch
		dw	016ah
		dw	0166h
		dw	0161h
		dw	015bh
		dw	0154h
		dw	014bh
		dw	0141h
		dw	0136h
		dw	012ah
		dw	011dh
		dw	010eh
		dw	0ffh
		dw	0eeh
		dw	0ddh
		dw	0cah
		dw	0b7h
		dw	0a2h
		dw	08dh
		dw	077h
		dw	061h
		dw	049h
		dw	031h
		dw	019h
		dw	00h
		dw	0ffe8h
		dw	0ffcfh
		dw	0ffb5h
		dw	0ff9bh
		dw	0ff82h
		dw	0ff68h
		dw	0ff4eh
		dw	0ff35h
		dw	0ff1ch
		dw	0ff03h
		dw	0feebh
		dw	0fed4h
		dw	0febdh
		dw	0fea7h
		dw	0fe91h
		dw	0fe7dh
		dw	0fe6ah
		dw	0fe57h
		dw	0fe46h
		dw	0fe36h
		dw	0fe27h
		dw	0fe1ah
		dw	0fe0eh
		dw	0fe04h
		dw	0fdfbh
		dw	0fdf3h
		dw	0fdeeh
		dw	0fdeah
		dw	0fde7h
		dw	0fde6h
		dw	0fde8h
		dw	0fdeah
		dw	0fdefh
		dw	0fdf5h
		dw	0fdfeh
		dw	0fe08h
		dw	0fe13h
		dw	0fe21h
		dw	0fe30h
		dw	0fe41h
		dw	0fe54h
		dw	0fe68h
		dw	0fe7eh
		dw	0fe96h
		dw	0feafh
		dw	0fec9h
		dw	0fee5h
		dw	0ff02h
		dw	0ff21h
		dw	0ff40h
		dw	0ff61h
		dw	0ff82h
		dw	0ffa5h
		dw	0ffc8h
		dw	0ffebh
		dw	0fh
		dw	033h
		dw	058h
		dw	07dh
		dw	0a3h
		dw	0c8h
		dw	0edh
		dw	0112h
		dw	0136h
		dw	015ah
		dw	017dh
		dw	019fh
		dw	01c1h
		dw	01e1h
		dw	0200h
		dw	021eh
		dw	023bh
		dw	0256h
		dw	026fh
		dw	0287h
		dw	029dh
		dw	02b1h
		dw	02c3h
		dw	02d3h
		dw	02e1h
		dw	02ech
		dw	02f6h
		dw	02fdh
		dw	0301h
		dw	0303h
		dw	0303h
		dw	0300h
		dw	02fbh
		dw	02f3h
		dw	02e8h
		dw	02dbh
		dw	02cbh
		dw	02b9h
		dw	02a4h
		dw	028dh
		dw	0273h
		dw	0257h
		dw	0239h
		dw	0218h
		dw	01f6h
		dw	01d1h
		dw	01aah
		dw	0181h
		dw	0157h
		dw	012bh
		dw	0fdh
		dw	0ceh
		dw	09eh
		dw	06dh
		dw	03ah
		dw	07h
		dw	0ffd4h
		dw	0ffa0h
		dw	0ff6bh
		dw	0ff36h
		dw	0ff01h
		dw	0fecch
		dw	0fe98h
		dw	0fe64h
		dw	0fe31h
		dw	0fdffh
		dw	0fdcdh
		dw	0fd9dh
		dw	0fd6fh
		dw	0fd41h
		dw	0fd16h
		dw	0fcedh
		dw	0fcc5h
		dw	0fca0h
		dw	0fc7dh
		dw	0fc5dh
		dw	0fc3fh
		dw	0fc24h
		dw	0fc0ch
		dw	0fbf6h
		dw	0fbe4h
		dw	0fbd6h
		dw	0fbcah
		dw	0fbc2h
		dw	0fbbdh
		dw	0fbbch
		dw	0fbbeh
		dw	0fbc4h
		dw	0fbcdh
		dw	0fbdah
		dw	0fbebh
		dw	0fbffh
		dw	0fc17h
		dw	0fc33h
		dw	0fc52h
		dw	0fc74h
		dw	0fc9ah
		dw	0fcc3h
		dw	0fcefh
		dw	0fd1fh
		dw	0fd51h
		dw	0fd87h
		dw	0fdbfh
		dw	0fdf9h
		dw	0fe36h
		dw	0fe76h
		dw	0feb7h
		dw	0fefah
		dw	0ff3fh
		dw	0ff86h
		dw	0ffcdh
		dw	015h
		dw	05fh
		dw	0a9h
		dw	0f4h
		dw	013eh
		dw	0189h
		dw	01d3h
		dw	021dh
		dw	0266h
		dw	02aeh
		dw	02f4h
		dw	0339h
		dw	037ch
		dw	03bdh
		dw	03fch
		dw	0438h
		dw	0472h
		dw	04a8h
		dw	04dch
		dw	050ch
		dw	0538h
		dw	0560h
		dw	0585h
		dw	05a5h
		dw	05c2h
		dw	05d9h
		dw	05ech
		dw	05fbh
		dw	0605h
		dw	0609h
		dw	0609h
		dw	0604h
		dw	05fah
		dw	05eah
		dw	05d6h
		dw	05bch
		dw	059dh
		dw	0579h
		dw	0551h
		dw	0523h
		dw	04f0h
		dw	04b9h
		dw	047dh
		dw	043ch
		dw	03f7h
		dw	03aeh
		dw	0361h
		dw	0310h
		dw	02bbh
		dw	0263h
		dw	0208h
		dw	01aah
		dw	014ah
		dw	0e7h
		dw	082h
		dw	01bh
		dw	0ffb4h
		dw	0ff4ah
		dw	0fee0h
		dw	0fe76h
		dw	0fe0bh
		dw	0fda0h
		dw	0fd36h
		dw	0fccdh
		dw	0fc65h
		dw	0fbffh
		dw	0fb9bh
		dw	0fb39h
		dw	0fadah
		dw	0fa7eh
		dw	0fa25h
		dw	0f9cfh
		dw	0f97eh
		dw	0f931h
		dw	0f8e9h
		dw	0f8a5h
		dw	0f867h
		dw	0f82eh
		dw	0f7fbh
		dw	0f7ceh
		dw	0f7a7h
		dw	0f787h
		dw	0f76dh
		dw	0f75ah
		dw	0f74eh
		dw	0f749h
		dw	0f74bh
		dw	0f754h
		dw	0f765h
		dw	0f77dh
		dw	0f79dh
		dw	0f7c4h
		dw	0f7f2h
		dw	0f828h
		dw	0f865h
		dw	0f8a9h
		dw	0f8f4h
		dw	0f947h
		dw	0f9a0h
		dw	0f9ffh
		dw	0fa66h
		dw	0fad2h
		dw	0fb44h
		dw	0fbbch
		dw	0fc39h
		dw	0fcbbh
		dw	0fd42h
		dw	0fdcdh
		dw	0fe5bh
		dw	0feeeh
		dw	0ff83h
		dw	01bh
		dw	0b5h
		dw	0151h
		dw	01efh
		dw	028dh
		dw	032ch
		dw	03cah
		dw	0468h
		dw	0504h
		dw	059fh
		dw	0637h
		dw	06cdh
		dw	075fh
		dw	07eeh
		dw	0878h
		dw	08feh
		dw	097eh
		dw	09f8h
		dw	0a6ch
		dw	0ad9h
		dw	0b3fh
		dw	0b9dh
		dw	0bf3h
		dw	0c41h
		dw	0c86h
		dw	0cc1h
		dw	0cf3h
		dw	0d1bh
		dw	0d38h
		dw	0d4bh
		dw	0d54h
		dw	0d51h
		dw	0d43h
		dw	0d2ah
		dw	0d06h
		dw	0cd6h
		dw	0c9ah
		dw	0c53h
		dw	0c01h
		dw	0ba3h
		dw	0b39h
		dw	0ac4h
		dw	0a44h
		dw	09bah
		dw	0924h
		dw	0884h
		dw	07dah
		dw	0726h
		dw	0669h
		dw	05a3h
		dw	04d4h
		dw	03feh
		dw	031fh
		dw	023ah
		dw	014fh
		dw	05dh
		dw	0ff67h
		dw	0fe6ch
		dw	0fd6dh
		dw	0fc6bh
		dw	0fb67h
		dw	0fa61h
		dw	0f95ah
		dw	0f853h
		dw	0f74dh
		dw	0f648h
		dw	0f546h
		dw	0f448h
		dw	0f34dh
		dw	0f257h
		dw	0f168h
		dw	0f07fh
		dw	0ef9dh
		dw	0eec4h
		dw	0edf5h
		dw	0ed2fh
		dw	0ec75h
		dw	0ebc6h
		dw	0eb24h
		dw	0ea90h
		dw	0ea0ah
		dw	0e992h
		dw	0e92bh
		dw	0e8d4h
		dw	0e88eh
		dw	0e85ah
		dw	0e838h
		dw	0e82ah
		dw	0e82eh
		dw	0e847h
		dw	0e875h
		dw	0e8b7h
		dw	0e90fh
		dw	0e97ch
		dw	0ea00h
		dw	0ea9ah
		dw	0eb4ah
		dw	0ec11h
		dw	0ecefh
		dw	0ede3h
		dw	0eeefh
		dw	0f011h
		dw	0f14ah
		dw	0f299h
		dw	0f3ffh
		dw	0f57bh
		dw	0f70dh
		dw	0f8b4h
		dw	0fa70h
		dw	0fc41h
		dw	0fe26h
		dw	01eh
		dw	0229h
		dw	0447h
		dw	0676h
		dw	08b7h
		dw	0b07h
		dw	0d66h
		dw	0fd4h
		dw	0124fh
		dw	014d7h
		dw	0176ah
		dw	01a07h
		dw	01caeh
		dw	01f5dh
		dw	02213h
		dw	024cfh
		dw	02790h
		dw	02a55h
		dw	02d1ch
		dw	02fe4h
		dw	032adh
		dw	03574h
		dw	03838h
		dw	03af9h
		dw	03db4h
		dw	0406ah
		dw	04317h
		dw	045bch
		dw	04857h
		dw	04ae7h
		dw	04d6ah
		dw	04fdfh
		dw	05246h
		dw	0549ch
		dw	056e2h
		dw	05915h
		dw	05b35h
		dw	05d40h
		dw	05f36h
		dw	06116h
		dw	062dfh
		dw	0648fh
		dw	06627h
		dw	067a5h
		dw	06908h
		dw	06a51h
		dw	06b7dh
		dw	06c8dh
		dw	06d81h
		dw	06e57h
		dw	06f0fh
		dw	06fa9h
		dw	07024h
		dw	07081h
		dw	070bfh
		dw	070deh

ELSE

	; Sampling= 352800  Cutoff= 2775  Taps= 784  Multiplication= 47.00
	; Window= 5  Beta=     6.0000
	; Atten @ 3598 Hz = 0.0023
	;
LPInter22050	dw	08h
		dw	08h
		dw	07h
		dw	07h
		dw	06h
		dw	05h
		dw	04h
		dw	03h
		dw	02h
		dw	01h
		dw	00h
		dw	0ffffh
		dw	0fffeh
		dw	0fffch
		dw	0fffah
		dw	0fff8h
		dw	0fff6h
		dw	0fff4h
		dw	0fff2h
		dw	0ffefh
		dw	0ffedh
		dw	0ffeah
		dw	0ffe8h
		dw	0ffe5h
		dw	0ffe2h
		dw	0ffe0h
		dw	0ffddh
		dw	0ffdah
		dw	0ffd7h
		dw	0ffd4h
		dw	0ffd1h
		dw	0ffceh
		dw	0ffcbh
		dw	0ffc8h
		dw	0ffc5h
		dw	0ffc2h
		dw	0ffbfh
		dw	0ffbch
		dw	0ffbah
		dw	0ffb7h
		dw	0ffb5h
		dw	0ffb2h
		dw	0ffb0h
		dw	0ffaeh
		dw	0ffach
		dw	0ffabh
		dw	0ffa9h
		dw	0ffa8h
		dw	0ffa7h
		dw	0ffa6h
		dw	0ffa6h
		dw	0ffa5h
		dw	0ffa5h
		dw	0ffa6h
		dw	0ffa6h
		dw	0ffa8h
		dw	0ffa9h
		dw	0ffabh
		dw	0ffadh
		dw	0ffafh
		dw	0ffb2h
		dw	0ffb5h
		dw	0ffb9h
		dw	0ffbdh
		dw	0ffc1h
		dw	0ffc6h
		dw	0ffcbh
		dw	0ffd1h
		dw	0ffd7h
		dw	0ffddh
		dw	0ffe4h
		dw	0ffebh
		dw	0fff3h
		dw	0fffbh
		dw	02h
		dw	0bh
		dw	014h
		dw	01dh
		dw	027h
		dw	031h
		dw	03bh
		dw	046h
		dw	050h
		dw	05bh
		dw	066h
		dw	071h
		dw	07ch
		dw	088h
		dw	093h
		dw	09eh
		dw	0aah
		dw	0b5h
		dw	0c0h
		dw	0cbh
		dw	0d6h
		dw	0e1h
		dw	0ebh
		dw	0f5h
		dw	0ffh
		dw	0108h
		dw	0111h
		dw	011ah
		dw	0122h
		dw	0129h
		dw	0130h
		dw	0136h
		dw	013ch
		dw	0140h
		dw	0144h
		dw	0148h
		dw	014ah
		dw	014ch
		dw	014ch
		dw	014ch
		dw	014bh
		dw	0149h
		dw	0145h
		dw	0141h
		dw	013ch
		dw	0135h
		dw	012eh
		dw	0125h
		dw	011bh
		dw	0111h
		dw	0105h
		dw	0f7h
		dw	0e9h
		dw	0dah
		dw	0c9h
		dw	0b8h
		dw	0a5h
		dw	091h
		dw	07ch
		dw	066h
		dw	050h
		dw	038h
		dw	01fh
		dw	05h
		dw	0ffech
		dw	0ffd1h
		dw	0ffb5h
		dw	0ff99h
		dw	0ff7bh
		dw	0ff5eh
		dw	0ff40h
		dw	0ff21h
		dw	0ff02h
		dw	0fee3h
		dw	0fec4h
		dw	0fea5h
		dw	0fe85h
		dw	0fe66h
		dw	0fe47h
		dw	0fe29h
		dw	0fe0ah
		dw	0fdech
		dw	0fdcfh
		dw	0fdb3h
		dw	0fd97h
		dw	0fd7ch
		dw	0fd62h
		dw	0fd49h
		dw	0fd31h
		dw	0fd1bh
		dw	0fd06h
		dw	0fcf2h
		dw	0fce0h
		dw	0fcd0h
		dw	0fcc2h
		dw	0fcb5h
		dw	0fcaah
		dw	0fca1h
		dw	0fc9bh
		dw	0fc96h
		dw	0fc94h
		dw	0fc94h
		dw	0fc96h
		dw	0fc9bh
		dw	0fca2h
		dw	0fcach
		dw	0fcb8h
		dw	0fcc7h
		dw	0fcd9h
		dw	0fcedh
		dw	0fd04h
		dw	0fd1eh
		dw	0fd3ah
		dw	0fd59h
		dw	0fd7bh
		dw	0fd9fh
		dw	0fdc6h
		dw	0fdf0h
		dw	0fe1bh
		dw	0fe4ah
		dw	0fe7bh
		dw	0feaeh
		dw	0fee4h
		dw	0ff1bh
		dw	0ff55h
		dw	0ff91h
		dw	0ffceh
		dw	0ch
		dw	04dh
		dw	090h
		dw	0d4h
		dw	0119h
		dw	015fh
		dw	01a6h
		dw	01edh
		dw	0235h
		dw	027eh
		dw	02c7h
		dw	030fh
		dw	0358h
		dw	03a0h
		dw	03e7h
		dw	042eh
		dw	0473h
		dw	04b8h
		dw	04fbh
		dw	053ch
		dw	057bh
		dw	05b8h
		dw	05f3h
		dw	062ch
		dw	0661h
		dw	0694h
		dw	06c4h
		dw	06f0h
		dw	0718h
		dw	073dh
		dw	075eh
		dw	077bh
		dw	0793h
		dw	07a7h
		dw	07b6h
		dw	07c0h
		dw	07c6h
		dw	07c6h
		dw	07c1h
		dw	07b7h
		dw	07a7h
		dw	0792h
		dw	0777h
		dw	0757h
		dw	0730h
		dw	0704h
		dw	06d2h
		dw	069ah
		dw	065dh
		dw	0619h
		dw	05d0h
		dw	0581h
		dw	052ch
		dw	04d2h
		dw	0472h
		dw	040dh
		dw	03a2h
		dw	0333h
		dw	02beh
		dw	0245h
		dw	01c7h
		dw	0144h
		dw	0beh
		dw	033h
		dw	0ffa6h
		dw	0ff14h
		dw	0fe7fh
		dw	0fde8h
		dw	0fd4eh
		dw	0fcb1h
		dw	0fc13h
		dw	0fb74h
		dw	0fad3h
		dw	0fa32h
		dw	0f990h
		dw	0f8eeh
		dw	0f84dh
		dw	0f7adh
		dw	0f70eh
		dw	0f671h
		dw	0f5d6h
		dw	0f53dh
		dw	0f4a8h
		dw	0f417h
		dw	0f389h
		dw	0f300h
		dw	0f27ch
		dw	0f1fdh
		dw	0f184h
		dw	0f111h
		dw	0f0a6h
		dw	0f041h
		dw	0efe4h
		dw	0ef90h
		dw	0ef44h
		dw	0ef01h
		dw	0eec7h
		dw	0ee97h
		dw	0ee72h
		dw	0ee57h
		dw	0ee47h
		dw	0ee42h
		dw	0ee49h
		dw	0ee5ch
		dw	0ee7ch
		dw	0eea8h
		dw	0eee1h
		dw	0ef27h
		dw	0ef7ah
		dw	0efdbh
		dw	0f049h
		dw	0f0c6h
		dw	0f151h
		dw	0f1e9h
		dw	0f291h
		dw	0f346h
		dw	0f40ah
		dw	0f4dch
		dw	0f5bch
		dw	0f6abh
		dw	0f7a8h
		dw	0f8b4h
		dw	0f9cdh
		dw	0faf4h
		dw	0fc29h
		dw	0fd6ch
		dw	0febch
		dw	017h
		dw	0181h
		dw	02f7h
		dw	0479h
		dw	0607h
		dw	07a1h
		dw	0945h
		dw	0af4h
		dw	0cach
		dw	0e6fh
		dw	0103ah
		dw	0120eh
		dw	013eah
		dw	015cdh
		dw	017b7h
		dw	019a7h
		dw	01b9ch
		dw	01d97h
		dw	01f96h
		dw	02198h
		dw	0239eh
		dw	025a5h
		dw	027aeh
		dw	029b8h
		dw	02bc1h
		dw	02dcah
		dw	02fd2h
		dw	031d7h
		dw	033d9h
		dw	035d8h
		dw	037d2h
		dw	039c7h
		dw	03bb7h
		dw	03d9fh
		dw	03f80h
		dw	04159h
		dw	04329h
		dw	044f0h
		dw	046ach
		dw	0485eh
		dw	04a04h
		dw	04b9eh
		dw	04d2ah
		dw	04eaah
		dw	0501bh
		dw	0517dh
		dw	052d1h
		dw	05414h
		dw	05548h
		dw	0566ah
		dw	0577ch
		dw	0587bh
		dw	05969h
		dw	05a44h
		dw	05b0dh
		dw	05bc2h
		dw	05c64h
		dw	05cf2h
		dw	05d6ch
		dw	05dd3h
		dw	05e25h
		dw	05e62h
		dw	05e8bh
		dw	05ea0h

ENDIF

ENDIF

		; Delay line for filters
		;
LPdata		dw	LPQORDER dup (0)

		; Index into LPdata
		;
LPIndex		dw	0

		; Decimator and Interpolator step counter
		;
I32Counter	dw	0

	.CODE

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; RVInterpInit
;
; Reset local variables for the interpolating filters.  Public routine.
;
; Function Prototype:
;
; void pascal
;    RVInterpInit(void)
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	EVEN
RVInterpInit	PROC USES SI DI

	mov	ax,@data
	mov	es,ax

	sub	ax,ax

	mov	LPIndex,ax		; clear index into the delay line.
	mov	cx,LPQORDER
	mov	di,OFFSET @data:LPData
	rep 	stosw			; clear delay line.

	sub	ax,ax
	mov	I32Counter,ax

	ret
RVInterpInit	ENDP

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; RVInterpBuf_144
;
; Change sampling rate from 7200 to 14400.  Public Routine.
;
; Function Prototype:
;
; unsigned int pascal
;    RVInterpBuf_144(
;       unsigned int far *Spointer,unsigned int far *Dpointer,
;	   unsigned int NPoints);
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	EVEN

LCYC144	EQU	4
MCYC144	EQU	2

RVInterpBuf_144	PROC USES SI DI, \
		XPtr : FAR PTR WORD, YPtr : FAR PTR WORD, \
		NPoints : WORD
	LOCAL	Counter
	LOCAL	YPtrStart

	; Initialize counter to the number of points to process.
	;
	mov	ax,NPoints
	or	ax,ax
	jnz	I49_ok

	; Number of points is zero -- return.
	;
	sub	ax,ax
	mov	dx,ax
	ret

I49_Done:
	; Restore registers for later call.
	;
	mov	I32Counter,dx
	mov	LPIndex,di
	mov	WORD PTR [Xptr],si

	; All done.  Return number of data points produced.
	;
	mov	ax,WORD PTR [YPtr]
	sub	ax,WORD PTR YPtrStart
	shr	ax,1
	ret

I49_ok:
	; Intialize the point counter.
	;
	inc	ax			; add one because we always subtract
	mov	Counter,ax
	mov	ax,WORD PTR [YPtr]
	mov	YPtrStart,ax
	mov	di,LPIndex		; preload for both branches

I49_Loop:
	mov	dx,I32Counter
	les	si,XPtr

I49_ShortLoop:
	sub	ax,ax
	sub	ax,dx
	jl	I49_Make

	; Check if no more data points to process.
	;
	dec	Counter
	jz	I49_Done

	; include new data point into delay line.
	;
	mov	cx,es:[si]
	add	si,2
	addmsml	di,-2			; subtract 2 from LPIndex

	; Times 2 at the end
	;
	sar	cx,1
	mov	LPData[di],cx

	add	dx,LCYC144		; 2*2
	or	ax,ax
	jne	I49_ShortLoop

	mov	ax,-LCYC144

I49_Make:
	sub	dx,MCYC144		; 2*1
	mov	I32Counter,dx
	mov	WORD PTR [Xptr],si

	add	ax,LCYC144		; 2*2
	mov	si,ax

	; Do the sum of products.  32 multiplications done
	; in line to save time.  Calculate data point due to
	; real data point.
	;
	mov	ax,LPdata[di]			; Y(n-1)
	imul	LPInter14400[si]		; a1(n)
	mov	bx,ax
	mov	cx,dx

	addmsml	di,2				; Y(n-2)
	mov	ax,LPdata[di]
	imul	LPInter14400[si+LCYC144] 	; a2(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-3)
	mov	ax,LPdata[di]
	imul	LPInter14400[si+2*LCYC144]	; a3(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-4)
	mov	ax,LPdata[di]
	imul	LPInter14400[si+3*LCYC144]	; a4(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-5)
	mov	ax,LPdata[di]
	imul	LPInter14400[si+4*LCYC144]	; a5(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-6)
	mov	ax,LPdata[di]
	imul	LPInter14400[si+5*LCYC144]	; a6(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-7)
	mov	ax,LPdata[di]
	imul	LPInter14400[si+6*LCYC144]	; a7(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-8)
	mov	ax,LPdata[di]
	imul	LPInter14400[si+7*LCYC144]	; a8(n)
	add	bx,ax
	adc	cx,dx

IFDEF BIG_FILTERS

	addmsml	di,2				; Y(n-9)
	mov	ax,LPdata[di]
	imul	LPInter14400[si+8*LCYC144] 	; a9(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-10)
	mov	ax,LPdata[di]
	imul	LPInter14400[si+9*LCYC144]	; a10(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-11)
	mov	ax,LPdata[di]
	imul	LPInter14400[si+10*LCYC144]	; a11(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-12)
	mov	ax,LPdata[di]
	imul	LPInter14400[si+11*LCYC144]	; a12(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-13)
	mov	ax,LPdata[di]
	imul	LPInter14400[si+12*LCYC144]	; a13(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-14)
	mov	ax,LPdata[di]
	imul	LPInter14400[si+13*LCYC144]	; a14(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-15)
	mov	ax,LPdata[di]
	imul	LPInter14400[si+14*LCYC144]	; a15(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-16)
	mov	ax,LPdata[di]
	imul	LPInter14400[si+15*LCYC144]	; a16(n)
	add	bx,ax
	adc	cx,dx

	sub	si,16*LCYC144-2
	neg	si

ELSE

	sub	si,8*LCYC144-2
	neg	si

ENDIF

	addmsml	di,2				; Y(n-17)
	mov	ax,LPdata[di]
	imul	LPInter14400[si]		; a17(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-18)
	mov	ax,LPdata[di]
	imul	LPInter14400[si-LCYC144] 	; a18(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-19)
	mov	ax,LPdata[di]
	imul	LPInter14400[si-2*LCYC144]	; a19(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-20)
	mov	ax,LPdata[di]
	imul	LPInter14400[si-3*LCYC144]	; a20(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-21)
	mov	ax,LPdata[di]
	imul	LPInter14400[si-4*LCYC144]	; a21(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-22)
	mov	ax,LPdata[di]
	imul	LPInter14400[si-5*LCYC144]	; a22(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-23)
	mov	ax,LPdata[di]
	imul	LPInter14400[si-6*LCYC144]	; a23(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-24)
	mov	ax,LPdata[di]
	imul	LPInter14400[si-7*LCYC144]	; a24(n)
	add	bx,ax
	adc	cx,dx

IFDEF BIG_FILTERS

	addmsml	di,2				; Y(n-25)
	mov	ax,LPdata[di]
	imul	LPInter14400[si-8*LCYC144]	; a25(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-26)
	mov	ax,LPdata[di]
	imul	LPInter14400[si-9*LCYC144]	; a26(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-27)
	mov	ax,LPdata[di]
	imul	LPInter14400[si-10*LCYC144]	; a27(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-28)
	mov	ax,LPdata[di]
	imul	LPInter14400[si-11*LCYC144]	; a28(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-29)
	mov	ax,LPdata[di]
	imul	LPInter14400[si-12*LCYC144]	; a29(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-30)
	mov	ax,LPdata[di]
	imul	LPInter14400[si-13*LCYC144]	; a30(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-31)
	mov	ax,LPdata[di]
	imul	LPInter14400[si-14*LCYC144] 	; a31(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-32)
	mov	ax,LPdata[di]
	imul	LPInter14400[si-15*LCYC144]	; a32(n)
	add	bx,ax
	adc	cx,dx

ENDIF
	
	; Decimal point adjust
 	;
	adjust	cx,bx
	clip	cx,bx,Ov2
	adjust	cx,bx
	clip	cx,bx

Ov2:
	addmsml	di,2				; complete circle

	; Store the data point.
	;
	les	si,YPtr
	mov	es:[si],cx
	add	si,2
	mov	WORD PTR [YPtr],si

	jmp	I49_Loop

RVInterpBuf_144	ENDP

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; RVInterpBuf_211
;
; Change sampling rate from 7200 to 21100.  Public Routine.
;
; Function Prototype:
;
; unsigned int pascal
;    RVInterpBuf_211(
;       unsigned int far *Spointer,unsigned int far *Dpointer,
;	   unsigned int NPoints);
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	EVEN

LCYC211	EQU	6
MCYC211	EQU	2

RVInterpBuf_211	PROC USES SI DI, \
		XPtr : FAR PTR WORD, YPtr : FAR PTR WORD, \
		NPoints : WORD
	LOCAL	Counter
	LOCAL	YPtrStart

	; Initialize counter to the number of points to process.
	;
	mov	ax,NPoints
	or	ax,ax
	jnz	I49_ok

	; Number of points is zero -- return.
	;
	sub	ax,ax
	mov	dx,ax
	ret

I49_Done:
	; Restore registers for later call.
	;
	mov	I32Counter,dx
	mov	LPIndex,di
	mov	WORD PTR [Xptr],si

	; All done.  Return number of data points produced.
	;
	mov	ax,WORD PTR [YPtr]
	sub	ax,WORD PTR YPtrStart
	shr	ax,1
	ret

I49_ok:
	; Intialize the point counter.
	;
	inc	ax			; add one because we always subtract
	mov	Counter,ax
	mov	ax,WORD PTR [YPtr]
	mov	YPtrStart,ax
	mov	di,LPIndex		; preload for both branches

I49_Loop:
	mov	dx,I32Counter
	les	si,XPtr

I49_ShortLoop:
	sub	ax,ax
	sub	ax,dx
	jl	I49_Make

	; Check if no more data points to process.
	;
	dec	Counter
	jz	I49_Done

	; include new data point into delay line.
	;
	mov	cx,es:[si]
	add	si,2
	addmsml	di,-2			; subtract 2 from LPIndex

	; Times 2 at the end
	;
	sar	cx,1
	mov	LPData[di],cx

	add	dx,LCYC211		; 2*2
	or	ax,ax
	jne	I49_ShortLoop

	mov	ax,-LCYC211

I49_Make:
	sub	dx,MCYC211		; 2*1
	mov	I32Counter,dx
	mov	WORD PTR [Xptr],si

	add	ax,LCYC211		; 2*2
	mov	si,ax

	; Do the sum of products.  32 multiplications done
	; in line to save time.  Calculate data point due to
	; real data point.
	;
	mov	ax,LPdata[di]			; Y(n-1)
	imul	LPInter21100[si]		; a1(n)
	mov	bx,ax
	mov	cx,dx

	addmsml	di,2				; Y(n-2)
	mov	ax,LPdata[di]
	imul	LPInter21100[si+LCYC211] 	; a2(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-3)
	mov	ax,LPdata[di]
	imul	LPInter21100[si+2*LCYC211]	; a3(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-4)
	mov	ax,LPdata[di]
	imul	LPInter21100[si+3*LCYC211]	; a4(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-5)
	mov	ax,LPdata[di]
	imul	LPInter21100[si+4*LCYC211]	; a5(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-6)
	mov	ax,LPdata[di]
	imul	LPInter21100[si+5*LCYC211]	; a6(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-7)
	mov	ax,LPdata[di]
	imul	LPInter21100[si+6*LCYC211]	; a7(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-8)
	mov	ax,LPdata[di]
	imul	LPInter21100[si+7*LCYC211]	; a8(n)
	add	bx,ax
	adc	cx,dx

IFDEF BIG_FILTERS

	addmsml	di,2				; Y(n-9)
	mov	ax,LPdata[di]
	imul	LPInter21100[si+8*LCYC211] 	; a9(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-10)
	mov	ax,LPdata[di]
	imul	LPInter21100[si+9*LCYC211]	; a10(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-11)
	mov	ax,LPdata[di]
	imul	LPInter21100[si+10*LCYC211]	; a11(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-12)
	mov	ax,LPdata[di]
	imul	LPInter21100[si+11*LCYC211]	; a12(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-13)
	mov	ax,LPdata[di]
	imul	LPInter21100[si+12*LCYC211]	; a13(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-14)
	mov	ax,LPdata[di]
	imul	LPInter21100[si+13*LCYC211]	; a14(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-15)
	mov	ax,LPdata[di]
	imul	LPInter21100[si+14*LCYC211]	; a15(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-16)
	mov	ax,LPdata[di]
	imul	LPInter21100[si+15*LCYC211]	; a16(n)
	add	bx,ax
	adc	cx,dx

	sub	si,16*LCYC211-2
	neg	si

ELSE

	sub	si,8*LCYC211-2
	neg	si

ENDIF

	addmsml	di,2				; Y(n-17)
	mov	ax,LPdata[di]
	imul	LPInter21100[si]		; a17(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-18)
	mov	ax,LPdata[di]
	imul	LPInter21100[si-LCYC211] 	; a18(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-19)
	mov	ax,LPdata[di]
	imul	LPInter21100[si-2*LCYC211]	; a19(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-20)
	mov	ax,LPdata[di]
	imul	LPInter21100[si-3*LCYC211]	; a20(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-21)
	mov	ax,LPdata[di]
	imul	LPInter21100[si-4*LCYC211]	; a21(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-22)
	mov	ax,LPdata[di]
	imul	LPInter21100[si-5*LCYC211]	; a22(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-23)
	mov	ax,LPdata[di]
	imul	LPInter21100[si-6*LCYC211]	; a23(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-24)
	mov	ax,LPdata[di]
	imul	LPInter21100[si-7*LCYC211]	; a24(n)
	add	bx,ax
	adc	cx,dx

IFDEF BIG_FILTERS

	addmsml	di,2				; Y(n-25)
	mov	ax,LPdata[di]
	imul	LPInter21100[si-8*LCYC211]	; a25(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-26)
	mov	ax,LPdata[di]
	imul	LPInter21100[si-9*LCYC211]	; a26(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-27)
	mov	ax,LPdata[di]
	imul	LPInter21100[si-10*LCYC211]	; a27(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-28)
	mov	ax,LPdata[di]
	imul	LPInter21100[si-11*LCYC211]	; a28(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-29)
	mov	ax,LPdata[di]
	imul	LPInter21100[si-12*LCYC211]	; a29(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-30)
	mov	ax,LPdata[di]
	imul	LPInter21100[si-13*LCYC211]	; a30(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-31)
	mov	ax,LPdata[di]
	imul	LPInter21100[si-14*LCYC211] 	; a31(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-32)
	mov	ax,LPdata[di]
	imul	LPInter21100[si-15*LCYC211]	; a32(n)
	add	bx,ax
	adc	cx,dx

ENDIF

	; Decimal point adjust
 	;
	adjust	cx,bx
	clip	cx,bx,Ov2
	adjust	cx,bx
	clip	cx,bx

Ov2:
	addmsml	di,2				; complete circle

	; Store the data point.
	;
	les	si,YPtr
	mov	es:[si],cx
	add	si,2
	mov	WORD PTR [YPtr],si

	jmp	I49_Loop

RVInterpBuf_211	ENDP

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; RVInterpBuf_11025
;
; Change sampling rate from 7200 to 11025.  Public Routine.
;
; Function Prototype:
;
; unsigned int pascal
;    RVInterpBuf_11025(
;       unsigned int far *Spointer,unsigned int far *Dpointer,
;	   unsigned int NPoints);
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	EVEN
RVInterpBuf_11025	PROC USES SI DI, \
		XPtr : FAR PTR WORD, YPtr : FAR PTR WORD, \
		NPoints : WORD
	LOCAL	Counter
	LOCAL	YPtrStart

	; Initialize counter to the number of points to process.
	;
	mov	ax,NPoints
	or	ax,ax
	jnz	I49_ok

	; Number of points is zero -- return.
	;
	sub	ax,ax
	mov	dx,ax
	ret

I49_Done:
	; Restore registers for later call.
	;
	mov	I32Counter,dx
	mov	LPIndex,di
	mov	WORD PTR [Xptr],si

	; All done.  Return number of data points produced.
	;
	mov	ax,WORD PTR [YPtr]
	sub	ax,WORD PTR YPtrStart
	shr	ax,1
	ret

I49_ok:
	; Intialize the point counter.
	;
	inc	ax			; add one because we always subtract
	mov	Counter,ax
	mov	ax,WORD PTR [YPtr]
	mov	YPtrStart,ax
	mov	di,LPIndex		; preload for both branches

I49_Loop:
	mov	dx,I32Counter
	les	si,XPtr

I49_ShortLoop:
	sub	ax,ax
	sub	ax,dx
	jl	I49_Make

	; Check if no more data points to process.
	;
	dec	Counter
	jz	I49_Done

	; include new data point into delay line.
	;
	mov	cx,es:[si]
	add	si,2
	addmsml	di,-2			; subtract 2 from LPIndex

	; Times 2 at the end
	;
	sar	cx,1
	mov	LPData[di],cx

	add	dx,LCYCLE		; 2*49
	or	ax,ax
	jne	I49_ShortLoop

	mov	ax,-LCYCLE

I49_Make:
	sub	dx,MCYCLE		; 2*32
	mov	I32Counter,dx
	mov	WORD PTR [Xptr],si

	add	ax,LCYCLE		; 2*49
	mov	si,ax

	; Do the sum of products.  32 multiplications done
	; in line to save time.  Calculate data point due to
	; real data point.
	;
	mov	ax,LPdata[di]			; Y(n-1)
	imul	LPInter22050[si]		; a1(n)
	mov	bx,ax
	mov	cx,dx

	addmsml	di,2				; Y(n-2)
	mov	ax,LPdata[di]
	imul	LPInter22050[si+LCYCLE] 	; a2(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-3)
	mov	ax,LPdata[di]
	imul	LPInter22050[si+2*LCYCLE]	; a3(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-4)
	mov	ax,LPdata[di]
	imul	LPInter22050[si+3*LCYCLE]	; a4(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-5)
	mov	ax,LPdata[di]
	imul	LPInter22050[si+4*LCYCLE]	; a5(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-6)
	mov	ax,LPdata[di]
	imul	LPInter22050[si+5*LCYCLE]	; a6(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-7)
	mov	ax,LPdata[di]
	imul	LPInter22050[si+6*LCYCLE]	; a7(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-8)
	mov	ax,LPdata[di]
	imul	LPInter22050[si+7*LCYCLE]	; a8(n)
	add	bx,ax
	adc	cx,dx

IFDEF BIG_FILTERS

	addmsml	di,2				; Y(n-9)
	mov	ax,LPdata[di]
	imul	LPInter22050[si+8*LCYCLE] 	; a9(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-10)
	mov	ax,LPdata[di]
	imul	LPInter22050[si+9*LCYCLE]	; a10(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-11)
	mov	ax,LPdata[di]
	imul	LPInter22050[si+10*LCYCLE]	; a11(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-12)
	mov	ax,LPdata[di]
	imul	LPInter22050[si+11*LCYCLE]	; a12(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-13)
	mov	ax,LPdata[di]
	imul	LPInter22050[si+12*LCYCLE]	; a13(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-14)
	mov	ax,LPdata[di]
	imul	LPInter22050[si+13*LCYCLE]	; a14(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-15)
	mov	ax,LPdata[di]
	imul	LPInter22050[si+14*LCYCLE]	; a15(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-16)
	mov	ax,LPdata[di]
	imul	LPInter22050[si+15*LCYCLE]	; a16(n)
	add	bx,ax
	adc	cx,dx

	sub	si,16*LCYCLE-2
	neg	si

ELSE

	sub	si,8*LCYCLE-2
	neg	si

ENDIF

	addmsml	di,2				; Y(n-17)
	mov	ax,LPdata[di]
	imul	LPInter22050[si]		; a17(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-18)
	mov	ax,LPdata[di]
	imul	LPInter22050[si-LCYCLE]		; a18(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-19)
	mov	ax,LPdata[di]
	imul	LPInter22050[si-2*LCYCLE]	; a19(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-20)
	mov	ax,LPdata[di]
	imul	LPInter22050[si-3*LCYCLE]	; a20(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-21)
	mov	ax,LPdata[di]
	imul	LPInter22050[si-4*LCYCLE]	; a21(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-22)
	mov	ax,LPdata[di]
	imul	LPInter22050[si-5*LCYCLE]	; a22(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-23)
	mov	ax,LPdata[di]
	imul	LPInter22050[si-6*LCYCLE]	; a23(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-24)
	mov	ax,LPdata[di]
	imul	LPInter22050[si-7*LCYCLE]	; a24(n)
	add	bx,ax
	adc	cx,dx

IFDEF BIG_FILTERS

	addmsml	di,2				; Y(n-25)
	mov	ax,LPdata[di]
	imul	LPInter22050[si-8*LCYCLE]	; a25(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-26)
	mov	ax,LPdata[di]
	imul	LPInter22050[si-9*LCYCLE]	; a26(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-27)
	mov	ax,LPdata[di]
	imul	LPInter22050[si-10*LCYCLE]	; a27(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-28)
	mov	ax,LPdata[di]
	imul	LPInter22050[si-11*LCYCLE]	; a28(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-29)
	mov	ax,LPdata[di]
	imul	LPInter22050[si-12*LCYCLE]	; a29(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-30)
	mov	ax,LPdata[di]
	imul	LPInter22050[si-13*LCYCLE]	; a30(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-31)
	mov	ax,LPdata[di]
	imul	LPInter22050[si-14*LCYCLE] 	; a31(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-32)
	mov	ax,LPdata[di]
	imul	LPInter22050[si-15*LCYCLE]	; a32(n)
	add	bx,ax
	adc	cx,dx

ENDIF

	; Decimal point adjust
 	;
	adjust	cx,bx
	clip	cx,bx,Ov2
	adjust	cx,bx
	clip	cx,bx

Ov2:
	addmsml	di,2				; complete circle

	; Store the data point.
	;
	les	si,YPtr
	mov	es:[si],cx
	add	si,2
	mov	WORD PTR [YPtr],si

	jmp	I49_Loop

RVInterpBuf_11025	ENDP

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; RVInterpBuf_22050
;
; Change sampling rate from 7200 to 22050.  Public Routine.
;
; Function Prototype:
;
; unsigned int pascal
;    RVInterpBuf_22050(
;       unsigned int far *Spointer,unsigned int far *Dpointer,
;	   unsigned int NPoints);
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	EVEN
RVInterpBuf_22050	PROC USES SI DI, \
		XPtr : FAR PTR WORD, YPtr : FAR PTR WORD, \
		NPoints : WORD
	LOCAL	Counter
	LOCAL	YPtrStart

	; Initialize counter to the number of points to process.
	;
	mov	ax,NPoints
	or	ax,ax
	jnz	I49_ok

	; Number of points is zero -- return.
	;
	sub	ax,ax
	mov	dx,ax
	ret

I49_Done:
	; Restore registers for later call.
	;
	mov	I32Counter,dx
	mov	LPIndex,di
	mov	WORD PTR [Xptr],si

	; All done.  Return number of data points produced.
	;
	mov	ax,WORD PTR [YPtr]
	sub	ax,WORD PTR YPtrStart
	shr	ax,1
	ret

I49_ok:
	; Intialize the point counter.
	;
	inc	ax			; add one because we always subtract
	mov	Counter,ax
	mov	ax,WORD PTR [YPtr]
	mov	YPtrStart,ax
	mov	di,LPIndex		; preload for both branches

I49_Loop:
	mov	dx,I32Counter
	les	si,XPtr

I49_ShortLoop:
	sub	ax,ax
	sub	ax,dx
	jl	I49_Make

	; Check if no more data points to process.
	;
	dec	Counter
	jz	I49_Done

	; include new data point into delay line.
	;
	mov	cx,es:[si]
	add	si,2
	addmsml	di,-2			; subtract 2 from LPIndex

	; Times 2 at the end
	;
	sar	cx,1
	mov	LPData[di],cx

	add	dx,LCYCLE		; 2*49
	or	ax,ax
	jne	I49_ShortLoop

	mov	ax,-LCYCLE

I49_Make:
	sub	dx,MCYCLE/2		; 2*32/2
	mov	I32Counter,dx
	mov	WORD PTR [Xptr],si

	add	ax,LCYCLE		; 2*49
	mov	si,ax

	; Do the sum of products.  32 multiplications done
	; in line to save time.  Calculate data point due to
	; real data point.
	;
	mov	ax,LPdata[di]			; Y(n-1)
	imul	LPInter22050[si]		; a1(n)
	mov	bx,ax
	mov	cx,dx

	addmsml	di,2				; Y(n-2)
	mov	ax,LPdata[di]
	imul	LPInter22050[si+LCYCLE] 	; a2(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-3)
	mov	ax,LPdata[di]
	imul	LPInter22050[si+2*LCYCLE]	; a3(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-4)
	mov	ax,LPdata[di]
	imul	LPInter22050[si+3*LCYCLE]	; a4(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-5)
	mov	ax,LPdata[di]
	imul	LPInter22050[si+4*LCYCLE]	; a5(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-6)
	mov	ax,LPdata[di]
	imul	LPInter22050[si+5*LCYCLE]	; a6(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-7)
	mov	ax,LPdata[di]
	imul	LPInter22050[si+6*LCYCLE]	; a7(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-8)
	mov	ax,LPdata[di]
	imul	LPInter22050[si+7*LCYCLE]	; a8(n)
	add	bx,ax
	adc	cx,dx

IFDEF BIG_FILTERS

	addmsml	di,2				; Y(n-9)
	mov	ax,LPdata[di]
	imul	LPInter22050[si+8*LCYCLE] 	; a9(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-10)
	mov	ax,LPdata[di]
	imul	LPInter22050[si+9*LCYCLE]	; a10(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-11)
	mov	ax,LPdata[di]
	imul	LPInter22050[si+10*LCYCLE]	; a11(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-12)
	mov	ax,LPdata[di]
	imul	LPInter22050[si+11*LCYCLE]	; a12(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-13)
	mov	ax,LPdata[di]
	imul	LPInter22050[si+12*LCYCLE]	; a13(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-14)
	mov	ax,LPdata[di]
	imul	LPInter22050[si+13*LCYCLE]	; a14(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-15)
	mov	ax,LPdata[di]
	imul	LPInter22050[si+14*LCYCLE]	; a15(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-16)
	mov	ax,LPdata[di]
	imul	LPInter22050[si+15*LCYCLE]	; a16(n)
	add	bx,ax
	adc	cx,dx

	sub	si,16*LCYCLE-2
	neg	si

ELSE

	sub	si,8*LCYCLE-2
	neg	si

ENDIF

	addmsml	di,2				; Y(n-17)
	mov	ax,LPdata[di]
	imul	LPInter22050[si]		; a17(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-18)
	mov	ax,LPdata[di]
	imul	LPInter22050[si-LCYCLE]		; a18(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-19)
	mov	ax,LPdata[di]
	imul	LPInter22050[si-2*LCYCLE]	; a19(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-20)
	mov	ax,LPdata[di]
	imul	LPInter22050[si-3*LCYCLE]	; a20(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-21)
	mov	ax,LPdata[di]
	imul	LPInter22050[si-4*LCYCLE]	; a21(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-22)
	mov	ax,LPdata[di]
	imul	LPInter22050[si-5*LCYCLE]	; a22(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-23)
	mov	ax,LPdata[di]
	imul	LPInter22050[si-6*LCYCLE]	; a23(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-24)
	mov	ax,LPdata[di]
	imul	LPInter22050[si-7*LCYCLE]	; a24(n)
	add	bx,ax
	adc	cx,dx

IFDEF BIG_FILTERS

	addmsml	di,2				; Y(n-25)
	mov	ax,LPdata[di]
	imul	LPInter22050[si-8*LCYCLE]	; a25(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-26)
	mov	ax,LPdata[di]
	imul	LPInter22050[si-9*LCYCLE]	; a26(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-27)
	mov	ax,LPdata[di]
	imul	LPInter22050[si-10*LCYCLE]	; a27(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-28)
	mov	ax,LPdata[di]
	imul	LPInter22050[si-11*LCYCLE]	; a28(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-29)
	mov	ax,LPdata[di]
	imul	LPInter22050[si-12*LCYCLE]	; a29(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-30)
	mov	ax,LPdata[di]
	imul	LPInter22050[si-13*LCYCLE]	; a30(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-31)
	mov	ax,LPdata[di]
	imul	LPInter22050[si-14*LCYCLE] 	; a31(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-32)
	mov	ax,LPdata[di]
	imul	LPInter22050[si-15*LCYCLE]	; a32(n)
	add	bx,ax
	adc	cx,dx

ENDIF

	; Decimal point adjust
 	;
	adjust	cx,bx
	clip	cx,bx,Ov2
	adjust	cx,bx
	clip	cx,bx

Ov2:
	addmsml	di,2				; complete circle

	; Store the data point.
	;
	les	si,YPtr
	mov	es:[si],cx
	add	si,2
	mov	WORD PTR [YPtr],si

	jmp	I49_Loop

RVInterpBuf_22050	ENDP


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; RVInterpCheck
;
; Check line delay for non-zero elements.  Return zero if all lines
; are zero.
;
; Function Prototype:
;
; unsigned int pascal
;    RVInterpCheck(void)
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	EVEN
RVInterpCheck	PROC USES DI

	mov	ax,@data
	mov	es,ax

	sub	ax,ax
	mov	cx,LPQORDER
	mov	di,OFFSET @data:LPData
	repz 	scasw			; check delay line.

	mov	ax,cx
	ret
RVInterpCheck	ENDP

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Decimation!!  Change sample rate from 14.4k, 21.1k,
;	        11.025k, or 22.05k to 7200 Hz
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	.DATA

IFNDEF Kaiser

IFDEF BIG_FILTERS

	; Sampling= 352800  Cutoff= 3600  Taps= 1024  Multiplication= 29.00
	; Window= 0
	; Atten @ 3528 Hz = 0.7067
	;
LPDecim11025   	dw	0244h
        	dw	023dh
        	dw	0233h
        	dw	0227h
        	dw	0218h
        	dw	0208h
        	dw	01f5h
        	dw	01e0h
        	dw	01c9h
	       	dw	01b0h
       		dw	0195h
        	dw	0178h
        	dw	015ah
        	dw	013ah
        	dw	0119h
        	dw	0f6h
        	dw	0d2h
        	dw	0aeh
        	dw	088h
        	dw	062h
        	dw	03bh
        	dw	013h
        	dw	0ffedh
        	dw	0ffc5h
         	dw	0ff9dh
         	dw	0ff76h
         	dw	0ff4fh
         	dw	0ff29h
         	dw	0ff03h
         	dw	0fedfh
         	dw	0febbh
         	dw	0fe98h
        	dw	0fe77h
        	dw	0fe58h
        	dw	0fe39h
        	dw	0fe1dh
        	dw	0fe03h
        	dw	0fdeah
        	dw	0fdd4h
        	dw	0fdc0h
        	dw	0fdaeh
        	dw	0fd9eh
        	dw	0fd91h
        	dw	0fd87h
        	dw	0fd7fh
        	dw	0fd79h
        	dw	0fd77h
        	dw	0fd77h
        	dw	0fd79h
        	dw	0fd7fh
        	dw	0fd87h
        	dw	0fd91h
        	dw	0fd9eh
        	dw	0fdaeh
        	dw	0fdc0h
         	dw	0fdd5h
         	dw	0fdech
         	dw	0fe06h
         	dw	0fe21h
        	dw	0fe3fh
        	dw	0fe5fh
        	dw	0fe80h
        	dw	0fea4h
       		dw	0fec9h
       		dw	0feefh
       		dw	0ff17h
       		dw	0ff3fh
       		dw	0ff69h
       		dw	0ff94h
       		dw	0ffbfh
       		dw	0ffebh
       		dw	016h
       		dw	042h
       		dw	06eh
       		dw	099h
       		dw	0c5h
       		dw	0efh
       		dw	0119h
       		dw	0142h
       		dw	016ah
       		dw	0191h
       		dw	01b6h
       		dw	01d9h
       		dw	01fbh
       		dw	021ah
       		dw	0238h
       		dw	0253h
       		dw	026ch
       		dw	0283h
       		dw	0297h
       		dw	02a9h
       		dw	02b7h
        	dw	02c3h
         	dw	02cch
         	dw	02d2h
        	dw	02d6h
        	dw	02d6h
        	dw	02d3h
        	dw	02ceh
        	dw	02c5h
        	dw	02b9h
        	dw	02aah
        	dw	0299h
        	dw	0285h
        	dw	026eh
        	dw	0254h
       		dw	0237h
       		dw	0219h
       		dw	01f7h
       		dw	01d4h
       		dw	01afh
       		dw	0187h
       		dw	015eh
       		dw	0133h
       		dw	0106h
       		dw	0d9h
       		dw	0aah
       		dw	07ah
       		dw	049h
       		dw	018h
       		dw	0ffe8h
       		dw	0ffb6h
       		dw	0ff85h
       		dw	0ff53h
       		dw	0ff22h
       		dw	0fef2h
       		dw	0fec3h
       		dw	0fe95h
       		dw	0fe68h
       		dw	0fe3ch
       		dw	0fe12h
       		dw	0fdeah
       		dw	0fdc4h
       		dw	0fda0h
       		dw	0fd7eh
       		dw	0fd5fh
       		dw	0fd43h
       		dw	0fd29h
       		dw	0fd12h
       		dw	0fcfeh
       		dw	0fcedh
       		dw	0fcdfh
       		dw	0fcd5h
       		dw	0fccdh
       		dw	0fcc9h
       		dw	0fcc9h
       		dw	0fccch
       		dw	0fcd2h
       		dw	0fcdch
       		dw	0fce9h
        	dw	0fcf9h
        	dw	0fd0dh
        	dw	0fd23h
        	dw	0fd3dh
        	dw	0fd5ah
        	dw	0fd7ah
        	dw	0fd9dh
        	dw	0fdc3h
        	dw	0fdebh
        	dw	0fe16h
        	dw	0fe42h
         	dw	0fe71h
         	dw	0fea2h
        	dw	0fed5h
       		dw	0ff09h
       		dw	0ff3eh
       		dw	0ff75h
       		dw	0ffach
       		dw	0ffe4h
       		dw	01ch
       		dw	055h
       		dw	08dh
       		dw	0c6h
       		dw	0feh
       		dw	0135h
       		dw	016bh
       		dw	01a1h
       		dw	01d4h
       		dw	0206h
       		dw	0237h
       		dw	0265h
       		dw	0291h
       		dw	02bah
       		dw	02e1h
       		dw	0306h
       		dw	0327h
       		dw	0345h
       		dw	035fh
       		dw	0377h
       		dw	038bh
       		dw	039bh
       		dw	03a8h
       		dw	03b0h
        	dw	03b5h
        	dw	03b6h
        	dw	03b3h
        	dw	03adh
         	dw	03a2h
         	dw	0393h
         	dw	0381h
         	dw	036ah
        	dw	0350h
        	dw	0333h
        	dw	0311h
        	dw	02ech
        	dw	02c4h
        	dw	0299h
        	dw	026bh
        	dw	023ah
        	dw	0206h
        	dw	01d0h
        	dw	0197h
        	dw	015ch
        	dw	0120h
        	dw	0e2h
        	dw	0a2h
        	dw	062h
        	dw	020h
        	dw	0ffdfh
        	dw	0ff9dh
        	dw	0ff5bh
        	dw	0ff19h
        	dw	0fed7h
        	dw	0fe96h
         	dw	0fe57h
         	dw	0fe18h
         	dw	0fddbh
         	dw	0fda0h
         	dw	0fd67h
         	dw	0fd31h
         	dw	0fcfdh
         	dw	0fccbh
        	dw	0fc9dh
        	dw	0fc72h
        	dw	0fc4bh
        	dw	0fc27h
        	dw	0fc07h
        	dw	0fbebh
        	dw	0fbd3h
        	dw	0fbbfh
        	dw	0fbb0h
        	dw	0fba5h
        	dw	0fb9eh
        	dw	0fb9ch
        	dw	0fb9fh
	       	dw	0fba6h
       		dw	0fbb3h
        	dw	0fbc3h
        	dw	0fbd8h
        	dw	0fbf2h
        	dw	0fc11h
        	dw	0fc33h
        	dw	0fc5ah
        	dw	0fc86h
        	dw	0fcb5h
        	dw	0fce8h
	   	dw	0fd1eh
        	dw	0fd59h
        	dw	0fd96h
        	dw	0fdd6h
        	dw	0fe1ah
        	dw	0fe5fh
        	dw	0fea8h
        	dw	0fef2h
        	dw	0ff3dh
	       	dw	0ff8bh
       		dw	0ffd9h
        	dw	027h
        	dw	077h
        	dw	0c7h
        	dw	0116h
        	dw	0165h
        	dw	01b4h
        	dw	0201h
        	dw	024dh
        	dw	0296h
        	dw	02deh
        	dw	0324h
        	dw	0366h
        	dw	03a6h
         	dw	03e2h
         	dw	041bh
         	dw	0450h
         	dw	0481h
         	dw	04adh
         	dw	04d5h
         	dw	04f9h
         	dw	0517h
        	dw	0530h
        	dw	0544h
        	dw	0552h
        	dw	055bh
        	dw	055fh
        	dw	055ch
        	dw	0555h
        	dw	0547h
        	dw	0534h
        	dw	051bh
        	dw	04fch
        	dw	04d8h
        	dw	04aeh
        	dw	047fh
        	dw	044bh
        	dw	0412h
        	dw	03d4h
        	dw	0391h
        	dw	034ah
        	dw	02feh
        	dw	02afh
        	dw	025ch
        	dw	0206h
         	dw	01adh
         	dw	0151h
         	dw	0f3h
         	dw	093h
        	dw	031h
        	dw	0ffcfh
        	dw	0ff6bh
        	dw	0ff07h
       		dw	0fea3h
       		dw	0fe3fh
       		dw	0fddch
       		dw	0fd7ah
       		dw	0fd1ah
       		dw	0fcbch
       		dw	0fc61h
       		dw	0fc08h
       		dw	0fbb2h
       		dw	0fb60h
       		dw	0fb12h
       		dw	0fac9h
       		dw	0fa84h
       		dw	0fa44h
       		dw	0fa0ah
       		dw	0f9d5h
       		dw	0f9a6h
       		dw	0f97dh
       		dw	0f95bh
       		dw	0f93fh
       		dw	0f92bh
       		dw	0f91dh
       		dw	0f916h
       		dw	0f917h
       		dw	0f91fh
       		dw	0f92eh
       		dw	0f944h
       		dw	0f962h
       		dw	0f988h
        	dw	0f9b5h
         	dw	0f9e9h
         	dw	0fa24h
        	dw	0fa66h
        	dw	0faaeh
        	dw	0fafeh
        	dw	0fb53h
        	dw	0fbafh
        	dw	0fc10h
        	dw	0fc77h
        	dw	0fce3h
        	dw	0fd53h
        	dw	0fdc8h
        	dw	0fe41h
       		dw	0febeh
       		dw	0ff3dh
       		dw	0ffbfh
       		dw	042h
       		dw	0c7h
       		dw	014eh
       		dw	01d5h
       		dw	025bh
       		dw	02e2h
       		dw	0367h
       		dw	03eah
       		dw	046ch
       		dw	04eah
       		dw	0565h
       		dw	05dch
       		dw	064fh
       		dw	06bdh
       		dw	0725h
       		dw	0787h
       		dw	07e3h
       		dw	0838h
       		dw	0885h
       		dw	08cbh
       		dw	0908h
       		dw	093dh
       		dw	0969h
       		dw	098ch
       		dw	09a5h
       		dw	09b4h
       		dw	09b9h
       		dw	09b4h
       		dw	09a5h
       		dw	098bh
       		dw	0966h
       		dw	0937h
       		dw	08fdh
       		dw	08b9h
       		dw	086ah
       		dw	0811h
       		dw	07aeh
       		dw	0741h
       		dw	06cah
       		dw	0649h
       		dw	05c0h
        	dw	052eh
        	dw	0493h
        	dw	03f1h
        	dw	0347h
        	dw	0297h
        	dw	01e0h
        	dw	0123h
        	dw	062h
        	dw	0ff9dh
        	dw	0fed4h
        	dw	0fe07h
         	dw	0fd38h
         	dw	0fc68h
        	dw	0fb97h
       		dw	0fac6h
       		dw	0f9f7h
       		dw	0f929h
       		dw	0f85dh
       		dw	0f795h
       		dw	0f6d2h
       		dw	0f613h
       		dw	0f55bh
       		dw	0f4a9h
       		dw	0f400h
       		dw	0f35fh
       		dw	0f2c7h
       		dw	0f23ah
       		dw	0f1b8h
       		dw	0f141h
       		dw	0f0d8h
       		dw	0f07bh
       		dw	0f02dh
       		dw	0efedh
       		dw	0efbdh
       		dw	0ef9dh
       		dw	0ef8dh
       		dw	0ef8fh
       		dw	0efa2h
       		dw	0efc7h
       		dw	0effeh
       		dw	0f049h
       		dw	0f0a6h
       		dw	0f117h
        	dw	0f19bh
        	dw	0f233h
        	dw	0f2dfh
        	dw	0f39fh
         	dw	0f472h
         	dw	0f55ah
         	dw	0f654h
         	dw	0f762h
        	dw	0f884h
        	dw	0f9b8h
        	dw	0fafeh
        	dw	0fc57h
        	dw	0fdc1h
        	dw	0ff3dh
        	dw	0c7h
        	dw	0263h
        	dw	040eh
        	dw	05c7h
        	dw	078dh
        	dw	0961h
        	dw	0b40h
        	dw	0d2bh
        	dw	0f1fh
        	dw	0111ch
        	dw	01322h
        	dw	0152eh
        	dw	01740h
        	dw	01957h
        	dw	01b72h
        	dw	01d8fh
        	dw	01fadh
         	dw	021cbh
         	dw	023e8h
         	dw	02602h
         	dw	02819h
         	dw	02a2ch
         	dw	02c38h
         	dw	02e3dh
         	dw	03039h
        	dw	0322ch
        	dw	03414h
        	dw	035f1h
        	dw	037c0h
        	dw	03981h
        	dw	03b32h
        	dw	03cd4h
        	dw	03e64h
        	dw	03fe2h
        	dw	0414dh
        	dw	042a4h
        	dw	043e6h
        	dw	04512h
	       	dw	04628h
       		dw	04727h
        	dw	0480eh
        	dw	048ddh
        	dw	04994h
        	dw	04a31h
        	dw	04ab4h
        	dw	04b1eh
        	dw	04b6dh
        	dw	04ba2h
        	dw	04bbdh
			     
ELSE			     
			     
	; Sampling= 352800  Cutoff= 3600  Taps= 512  Multiplication= 31.00
	; Window= 0
	; Atten @ 3439 Hz = 0.7178
	;		     
LPDecim11025   	dw	0fcebh
        	dw	0fd2ah    
        	dw	0fd6bh    
        	dw	0fdb0h    
        	dw	0fdf8h    
        	dw	0fe43h    
        	dw	0fe90h    
        	dw	0fedfh    
        	dw	0ff30h    
	       	dw	0ff83h    
       		dw	0ffd6h    
        	dw	02ah
        	dw	07fh
        	dw	0d4h
        	dw	0129h    
        	dw	017eh    
        	dw	01d2h    
        	dw	0224h    
        	dw	0275h    
        	dw	02c4h    
        	dw	0311h    
        	dw	035bh    
        	dw	03a2h    
        	dw	03e6h    
         	dw	0427h    
         	dw	0464h    
         	dw	049ch    
         	dw	04d0h    
         	dw	0500h    
         	dw	052bh    
         	dw	0550h    
         	dw	0571h    
        	dw	058bh    
        	dw	05a1h    
        	dw	05b0h    
        	dw	05bah    
        	dw	05beh    
        	dw	05bbh    
        	dw	05b3h    
        	dw	05a4h    
        	dw	058fh    
        	dw	0575h    
        	dw	0554h    
        	dw	052dh    
        	dw	0501h    
        	dw	04ceh    
        	dw	0497h    
        	dw	0459h    
        	dw	0417h    
        	dw	03d0h    
        	dw	0384h    
        	dw	0333h    
        	dw	02deh    
        	dw	0286h    
        	dw	022ah    
         	dw	01cah    
         	dw	0168h    
         	dw	0103h    
         	dw	09dh
        	dw	034h
        	dw	0ffcbh    
        	dw	0ff61h    
        	dw	0fef6h    
       		dw	0fe8bh    
       		dw	0fe20h    
       		dw	0fdb6h    
       		dw	0fd4eh    
       		dw	0fce7h    
       		dw	0fc83h    
       		dw	0fc21h    
       		dw	0fbc2h    
       		dw	0fb66h    
       		dw	0fb0fh    
       		dw	0fabbh    
       		dw	0fa6dh    
       		dw	0fa23h    
       		dw	0f9dfh    
       		dw	0f9a1h    
       		dw	0f968h    
       		dw	0f936h    
       		dw	0f90ah    
       		dw	0f8e6h    
       		dw	0f8c8h    
       		dw	0f8b2h    
       		dw	0f8a3h    
       		dw	0f89ch    
       		dw	0f89dh    
       		dw	0f8a5h    
       		dw	0f8b5h    
       		dw	0f8cdh    
       		dw	0f8eeh    
       		dw	0f916h    
        	dw	0f945h    
         	dw	0f97dh    
         	dw	0f9bch    
        	dw	0fa03h    
        	dw	0fa50h    
        	dw	0faa5h    
        	dw	0fb01h    
        	dw	0fb62h    
        	dw	0fbcbh    
        	dw	0fc39h    
        	dw	0fcach    
        	dw	0fd24h    
        	dw	0fda1h    
        	dw	0fe22h    
       		dw	0fea7h    
       		dw	0ff2fh    
       		dw	0ffbah    
       		dw	046h
       		dw	0d5h
       		dw	0165h    
       		dw	01f5h    
       		dw	0285h    
       		dw	0315h    
       		dw	03a3h    
       		dw	0430h    
       		dw	04bah    
       		dw	0541h    
       		dw	05c4h    
       		dw	0644h    
       		dw	06beh    
       		dw	0734h    
       		dw	07a3h    
       		dw	080ch    
       		dw	086eh    
       		dw	08c9h    
       		dw	091ch    
       		dw	0966h    
       		dw	09a8h    
       		dw	09e0h    
       		dw	0a0fh    
       		dw	0a34h    
       		dw	0a4fh    
       		dw	0a5fh    
       		dw	0a65h    
       		dw	0a5fh    
       		dw	0a4fh    
       		dw	0a33h    
       		dw	0a0ch    
       		dw	09dah    
       		dw	099ch    
       		dw	0953h    
       		dw	08ffh    
       		dw	08a0h    
       		dw	0835h    
       		dw	07c1h    
       		dw	0742h    
       		dw	06b8h    
       		dw	0625h    
        	dw	0589h    
        	dw	04e4h    
        	dw	0436h    
        	dw	0381h    
        	dw	02c4h    
        	dw	0201h    
        	dw	0138h    
        	dw	069h
        	dw	0ff96h    
        	dw	0febfh    
        	dw	0fde4h    
         	dw	0fd07h    
         	dw	0fc29h    
        	dw	0fb49h    
       		dw	0fa6ah    
       		dw	0f98ch    
       		dw	0f8b0h    
       		dw	0f7d6h    
       		dw	0f701h    
       		dw	0f62fh    
       		dw	0f564h    
       		dw	0f49fh    
       		dw	0f3e1h    
       		dw	0f32ch    
       		dw	0f280h    
       		dw	0f1deh    
       		dw	0f147h    
       		dw	0f0bbh    
       		dw	0f03dh    
       		dw	0efcch    
       		dw	0ef69h    
       		dw	0ef15h    
       		dw	0eed1h    
       		dw	0ee9eh    
       		dw	0ee7ch    
       		dw	0ee6bh    
       		dw	0ee6ch    
       		dw	0ee81h    
       		dw	0eea8h    
       		dw	0eee4h    
       		dw	0ef33h    
       		dw	0ef97h    
       		dw	0f010h    
        	dw	0f09dh    
        	dw	0f140h    
        	dw	0f1f8h    
        	dw	0f2c4h    
         	dw	0f3a6h    
         	dw	0f49dh    
         	dw	0f5aah    
         	dw	0f6cah    
        	dw	0f800h    
        	dw	0f949h    
        	dw	0faa6h    
        	dw	0fc16h    
        	dw	0fd9ah    
        	dw	0ff2fh    
        	dw	0d5h
        	dw	028dh    
        	dw	0455h
        	dw	062dh
        	dw	0813h
        	dw	0a07h
        	dw	0c07h
        	dw	0e13h
        	dw	0102ah
        	dw	0124bh
        	dw	01474h
        	dw	016a4h
        	dw	018dbh
        	dw	01b17h
        	dw	01d56h
        	dw	01f98h
        	dw	021dch
         	dw	0241fh
         	dw	02662h
         	dw	028a2h
         	dw	02addh
         	dw	02d14h
         	dw	02f45h
         	dw	0316dh
         	dw	0338dh
        	dw	035a2h
        	dw	037ach
        	dw	039a9h
        	dw	03b98h
        	dw	03d78h
        	dw	03f48h
        	dw	04106h
        	dw	042b2h
        	dw	0444ah
        	dw	045ceh
        	dw	0473dh
        	dw	04895h
        	dw	049d6h
	       	dw	04affh
       		dw	04c0fh
        	dw	04d07h
        	dw	04de4h
        	dw	04ea7h
        	dw	04f4fh
        	dw	04fdbh
        	dw	0504ch
        	dw	050a1h
        	dw	050dah
        	dw	050f6h

ENDIF

ELSE

IFDEF BIG_FILTERS

	; Sampling= 352800  Cutoff= 2950  Taps= 1024  Multiplication= 29.00
	; Window= 5  Beta=     6.0000
	; Atten @ 3598 Hz = 0.0014
	;
LPDecim11025   	dw	08h
        	dw	09h
        	dw	09h
        	dw	09h
        	dw	0ah
        	dw	0ah
        	dw	0ah
        	dw	0bh
        	dw	0bh
	       	dw	0bh
       		dw	0bh
        	dw	0bh
        	dw	0bh
        	dw	0bh
        	dw	0bh
        	dw	0bh
        	dw	0bh
        	dw	0bh
        	dw	0bh
        	dw	0ah
        	dw	0ah
        	dw	0ah
        	dw	09h
        	dw	09h
         	dw	08h
         	dw	07h
         	dw	07h
         	dw	06h
         	dw	05h
         	dw	04h
         	dw	03h
         	dw	02h
        	dw	01h
        	dw	00h
        	dw	0ffffh
        	dw	0fffeh
        	dw	0fffdh
        	dw	0fffbh
        	dw	0fffah
        	dw	0fff8h
        	dw	0fff7h
        	dw	0fff5h
        	dw	0fff4h
        	dw	0fff2h
        	dw	0fff0h
        	dw	0ffefh
        	dw	0ffedh
        	dw	0ffebh
        	dw	0ffe9h
        	dw	0ffe8h
        	dw	0ffe6h
        	dw	0ffe4h
        	dw	0ffe3h
        	dw	0ffe1h
        	dw	0ffdfh
         	dw	0ffdeh
         	dw	0ffdch
         	dw	0ffdbh
         	dw	0ffd9h
        	dw	0ffd8h
        	dw	0ffd7h
        	dw	0ffd6h
        	dw	0ffd5h
       		dw	0ffd4h
       		dw	0ffd3h
       		dw	0ffd2h
       		dw	0ffd2h
       		dw	0ffd1h
       		dw	0ffd1h
       		dw	0ffd1h
       		dw	0ffd1h
       		dw	0ffd1h
       		dw	0ffd1h
       		dw	0ffd2h
       		dw	0ffd3h
       		dw	0ffd3h
       		dw	0ffd4h
       		dw	0ffd6h
       		dw	0ffd7h
       		dw	0ffd8h
       		dw	0ffdah
       		dw	0ffdch
       		dw	0ffdeh
       		dw	0ffe0h
       		dw	0ffe3h
       		dw	0ffe6h
       		dw	0ffe8h
       		dw	0ffebh
       		dw	0ffefh
       		dw	0fff2h
       		dw	0fff5h
       		dw	0fff9h
        	dw	0fffdh
         	dw	00h
         	dw	04h
        	dw	08h
        	dw	0ch
        	dw	011h
        	dw	015h
        	dw	01ah
        	dw	01eh
        	dw	023h
        	dw	028h
        	dw	02dh
        	dw	031h
        	dw	036h
       		dw	03bh
       		dw	03fh
       		dw	044h
       		dw	049h
       		dw	04dh
       		dw	052h
       		dw	056h
       		dw	05ah
       		dw	05eh
       		dw	062h
       		dw	065h
       		dw	069h
       		dw	06ch
       		dw	06fh
       		dw	071h
       		dw	074h
       		dw	076h
       		dw	078h
       		dw	079h
       		dw	07bh
       		dw	07bh
       		dw	07ch
       		dw	07ch
       		dw	07ch
       		dw	07bh
       		dw	07ah
       		dw	079h
       		dw	077h
       		dw	074h
       		dw	072h
       		dw	06fh
       		dw	06bh
       		dw	067h
       		dw	063h
       		dw	05eh
       		dw	059h
       		dw	053h
       		dw	04dh
       		dw	047h
       		dw	040h
       		dw	039h
       		dw	031h
       		dw	029h
       		dw	021h
        	dw	018h
        	dw	0fh
        	dw	06h
        	dw	0fffeh
        	dw	0fff4h
        	dw	0ffeah
        	dw	0ffe0h
        	dw	0ffd6h
        	dw	0ffcch
        	dw	0ffc1h
        	dw	0ffb7h
         	dw	0ffach
         	dw	0ffa2h
        	dw	0ff97h
       		dw	0ff8ch
       		dw	0ff82h
       		dw	0ff78h
       		dw	0ff6dh
       		dw	0ff63h
       		dw	0ff5ah
       		dw	0ff50h
       		dw	0ff47h
       		dw	0ff3eh
       		dw	0ff36h
       		dw	0ff2dh
       		dw	0ff26h
       		dw	0ff1eh
       		dw	0ff18h
       		dw	0ff11h
       		dw	0ff0ch
       		dw	0ff07h
       		dw	0ff02h
       		dw	0fefeh
       		dw	0fefbh
       		dw	0fef9h
       		dw	0fef7h
       		dw	0fef6h
       		dw	0fef6h
       		dw	0fef7h
       		dw	0fef8h
       		dw	0fefah
       		dw	0fefdh
       		dw	0ff01h
        	dw	0ff06h
        	dw	0ff0bh
        	dw	0ff12h
        	dw	0ff19h
         	dw	0ff21h
         	dw	0ff2ah
         	dw	0ff34h
         	dw	0ff3eh
        	dw	0ff4ah
        	dw	0ff56h
        	dw	0ff63h
        	dw	0ff71h
        	dw	0ff7fh
        	dw	0ff8fh
        	dw	0ff9eh
        	dw	0ffafh
        	dw	0ffc0h
        	dw	0ffd2h
        	dw	0ffe4h
        	dw	0fff7h
        	dw	09h
        	dw	01ch
        	dw	030h
        	dw	044h
        	dw	059h
        	dw	06dh
        	dw	082h
        	dw	097h
        	dw	0ach
        	dw	0c0h
        	dw	0d5h
         	dw	0e9h
         	dw	0feh
         	dw	0111h
         	dw	0125h
         	dw	0138h
         	dw	014ah
         	dw	015ch
         	dw	016eh
        	dw	017eh
        	dw	018eh
        	dw	019dh
        	dw	01abh
        	dw	01b9h
        	dw	01c5h
        	dw	01d0h
        	dw	01dah
        	dw	01e3h
        	dw	01eah
        	dw	01f0h
        	dw	01f5h
        	dw	01f9h
	       	dw	01fbh
       		dw	01fch
        	dw	01fbh
        	dw	01f9h
        	dw	01f6h
        	dw	01f0h
        	dw	01eah
        	dw	01e1h
        	dw	01d7h
        	dw	01cch
        	dw	01bfh
	   	dw	01b0h
        	dw	01a0h
        	dw	018fh
        	dw	017bh
        	dw	0167h
        	dw	0151h
        	dw	0139h
        	dw	0120h
        	dw	0106h
	       	dw	0eah
       		dw	0cdh
        	dw	0afh
        	dw	090h
        	dw	070h
        	dw	04fh
        	dw	02dh
        	dw	0bh
        	dw	0ffe8h
        	dw	0ffc4h
        	dw	0ff9fh
        	dw	0ff7ah
        	dw	0ff55h
        	dw	0ff2fh
        	dw	0ff09h
         	dw	0fee4h
         	dw	0febeh
         	dw	0fe98h
         	dw	0fe73h
         	dw	0fe4eh
         	dw	0fe2ah
         	dw	0fe07h
         	dw	0fde4h
        	dw	0fdc2h
        	dw	0fda1h
        	dw	0fd81h
        	dw	0fd62h
        	dw	0fd45h
        	dw	0fd29h
        	dw	0fd0fh
        	dw	0fcf7h
        	dw	0fce0h
        	dw	0fccbh
        	dw	0fcb8h
        	dw	0fca7h
        	dw	0fc98h
        	dw	0fc8ch
        	dw	0fc81h
        	dw	0fc79h
        	dw	0fc74h
        	dw	0fc71h
        	dw	0fc71h
        	dw	0fc73h
        	dw	0fc78h
        	dw	0fc80h
        	dw	0fc8ah
         	dw	0fc97h
         	dw	0fca7h
         	dw	0fcbah
         	dw	0fccfh
        	dw	0fce8h
        	dw	0fd03h
        	dw	0fd20h
        	dw	0fd41h
       		dw	0fd64h
       		dw	0fd89h
       		dw	0fdb1h
       		dw	0fddch
       		dw	0fe09h
       		dw	0fe38h
       		dw	0fe6ah
       		dw	0fe9eh
       		dw	0fed3h
       		dw	0ff0bh
       		dw	0ff44h
       		dw	0ff7fh
       		dw	0ffbch
       		dw	0fffah
       		dw	038h
       		dw	078h
       		dw	0b9h
       		dw	0fbh
       		dw	013dh
       		dw	0180h
       		dw	01c3h
       		dw	0205h
       		dw	0248h
       		dw	028ah
       		dw	02cch
       		dw	030dh
       		dw	034dh
       		dw	038ch
       		dw	03c9h
        	dw	0405h
         	dw	043fh
         	dw	0477h
        	dw	04adh
        	dw	04e1h
        	dw	0512h
        	dw	0540h
        	dw	056bh
        	dw	0593h
        	dw	05b8h
        	dw	05dah
        	dw	05f7h
        	dw	0611h
        	dw	0627h
       		dw	0639h
       		dw	0647h
       		dw	0651h
       		dw	0656h
       		dw	0656h
       		dw	0652h
       		dw	0649h
       		dw	063bh
       		dw	0629h
       		dw	0611h
       		dw	05f5h
       		dw	05d3h
       		dw	05adh
       		dw	0582h
       		dw	0552h
       		dw	051dh
       		dw	04e3h
       		dw	04a4h
       		dw	0460h
       		dw	0418h
       		dw	03cch
       		dw	037bh
       		dw	0325h
       		dw	02cch
       		dw	026eh
       		dw	020dh
       		dw	01a8h
       		dw	0140h
       		dw	0d4h
       		dw	066h
       		dw	0fff6h
       		dw	0ff82h
       		dw	0ff0ch
       		dw	0fe95h
       		dw	0fe1bh
       		dw	0fda1h
       		dw	0fd25h
       		dw	0fca8h
       		dw	0fc2bh
       		dw	0fbaeh
       		dw	0fb32h
       		dw	0fab6h
       		dw	0fa3bh
       		dw	0f9c1h
        	dw	0f949h
        	dw	0f8d3h
        	dw	0f860h
        	dw	0f7efh
        	dw	0f781h
        	dw	0f717h
        	dw	0f6b2h
        	dw	0f650h
        	dw	0f5f3h
        	dw	0f59bh
        	dw	0f548h
         	dw	0f4fch
         	dw	0f4b5h
        	dw	0f475h
       		dw	0f43ch
       		dw	0f409h
       		dw	0f3dfh
       		dw	0f3bch
       		dw	0f3a1h
       		dw	0f38eh
       		dw	0f384h
       		dw	0f383h
       		dw	0f38bh
       		dw	0f39dh
       		dw	0f3b8h
       		dw	0f3dch
       		dw	0f40bh
       		dw	0f444h
       		dw	0f487h
       		dw	0f4d5h
       		dw	0f52dh
       		dw	0f590h
       		dw	0f5fdh
       		dw	0f675h
       		dw	0f6f8h
       		dw	0f785h
       		dw	0f81eh
       		dw	0f8c1h
       		dw	0f96eh
       		dw	0fa27h
       		dw	0fae9h
       		dw	0fbb6h
       		dw	0fc8eh
        	dw	0fd6fh
        	dw	0fe5ah
        	dw	0ff4fh
        	dw	04ch
         	dw	0153h
         	dw	0263h
         	dw	037ch
         	dw	049dh
        	dw	05c5h
        	dw	06f6h
        	dw	082dh
        	dw	096bh
        	dw	0ab0h
        	dw	0bfah
        	dw	0d4ah
        	dw	0ea0h
        	dw	0ff9h
        	dw	01157h
        	dw	012b8h
        	dw	0141dh
        	dw	01584h
        	dw	016edh
        	dw	01857h
        	dw	019c3h
        	dw	01b2eh
        	dw	01c9ah
        	dw	01e05h
        	dw	01f6fh
        	dw	020d6h
        	dw	0223ch
        	dw	0239eh
         	dw	024fdh
         	dw	02658h
         	dw	027aeh
         	dw	028feh
         	dw	02a49h
         	dw	02b8eh
         	dw	02ccch
         	dw	02e02h
        	dw	02f30h
        	dw	03056h
        	dw	03173h
        	dw	03287h
        	dw	03391h
        	dw	03490h
        	dw	03585h
        	dw	0366eh
        	dw	0374ch
        	dw	0381eh
        	dw	038e4h
        	dw	0399dh
        	dw	03a49h
	       	dw	03ae8h
       		dw	03b79h
        	dw	03bfch
        	dw	03c72h
        	dw	03cd9h
        	dw	03d32h
        	dw	03d7ch
        	dw	03db7h
        	dw	03de4h
        	dw	03e02h
        	dw	03e11h

ELSE

	; Sampling= 352800  Cutoff= 2350  Taps= 512  Multiplication= 31.00
	; Window= 5  Beta=     6.0000
	; Atten @ 3598 Hz = 0.0028
	;
LPDecim11025   	dw	0ffefh
        	dw	0ffedh
        	dw	0ffech
        	dw	0ffebh
        	dw	0ffeah
        	dw	0ffe9h
        	dw	0ffe9h
        	dw	0ffe8h
        	dw	0ffe7h
	       	dw	0ffe6h
       		dw	0ffe6h
        	dw	0ffe5h
        	dw	0ffe5h
        	dw	0ffe5h
        	dw	0ffe5h
        	dw	0ffe5h
        	dw	0ffe5h
        	dw	0ffe5h
        	dw	0ffe6h
        	dw	0ffe7h
        	dw	0ffe8h
        	dw	0ffe9h
        	dw	0ffebh
        	dw	0ffech
         	dw	0ffeeh
         	dw	0fff0h
         	dw	0fff3h
         	dw	0fff5h
         	dw	0fff8h
         	dw	0fffch
         	dw	0ffffh
         	dw	02h
        	dw	06h
        	dw	0bh
        	dw	010h
        	dw	015h
        	dw	01ah
        	dw	020h
        	dw	026h
        	dw	02ch
        	dw	033h
        	dw	03ah
        	dw	041h
        	dw	049h
        	dw	051h
        	dw	059h
        	dw	061h
        	dw	06ah
        	dw	073h
        	dw	07ch
        	dw	085h
        	dw	08fh
        	dw	099h
        	dw	0a2h
        	dw	0ach
         	dw	0b6h
         	dw	0c1h
         	dw	0cbh
         	dw	0d5h
        	dw	0dfh
        	dw	0e9h
        	dw	0f3h
        	dw	0fdh
       		dw	0107h
       		dw	0111h
       		dw	011ah
       		dw	0124h
       		dw	012ch
       		dw	0135h
       		dw	013dh
       		dw	0145h
       		dw	014ch
       		dw	0153h
       		dw	0159h
       		dw	015fh
       		dw	0164h
       		dw	0168h
       		dw	016ch
       		dw	016fh
       		dw	0170h
       		dw	0172h
       		dw	0172h
       		dw	0171h
       		dw	016fh
       		dw	016dh
       		dw	0169h
       		dw	0164h
       		dw	015eh
       		dw	0156h
       		dw	014eh
       		dw	0144h
       		dw	0139h
        	dw	012ch
         	dw	011fh
         	dw	0110h
        	dw	0ffh
        	dw	0edh
        	dw	0dah
        	dw	0c5h
        	dw	0afh
        	dw	097h
        	dw	07eh
        	dw	064h
        	dw	048h
        	dw	02ah
        	dw	0bh
       		dw	0ffech
       		dw	0ffcah
       		dw	0ffa7h
       		dw	0ff83h
       		dw	0ff5dh
       		dw	0ff36h
       		dw	0ff0eh
       		dw	0fee5h
       		dw	0febbh
       		dw	0fe8fh
       		dw	0fe63h
       		dw	0fe35h
       		dw	0fe07h
       		dw	0fdd8h
       		dw	0fda8h
       		dw	0fd78h
       		dw	0fd47h
       		dw	0fd15h
       		dw	0fce4h
       		dw	0fcb2h
       		dw	0fc80h
       		dw	0fc4dh
       		dw	0fc1bh
       		dw	0fbeah
       		dw	0fbb8h
       		dw	0fb87h
       		dw	0fb57h
       		dw	0fb27h
       		dw	0faf8h
       		dw	0facah
       		dw	0fa9dh
       		dw	0fa72h
       		dw	0fa48h
       		dw	0fa1fh
       		dw	0f9f9h
       		dw	0f9d4h
       		dw	0f9b1h
       		dw	0f990h
       		dw	0f972h
       		dw	0f956h
       		dw	0f93dh
       		dw	0f927h
       		dw	0f913h
       		dw	0f903h
        	dw	0f8f6h
        	dw	0f8ech
        	dw	0f8e6h
        	dw	0f8e4h
        	dw	0f8e6h
        	dw	0f8ebh
        	dw	0f8f5h
        	dw	0f903h
        	dw	0f915h
        	dw	0f92ch
        	dw	0f947h
         	dw	0f968h
         	dw	0f98dh
        	dw	0f9b7h
       		dw	0f9e6h
       		dw	0fa1ah
       		dw	0fa54h
       		dw	0fa93h
       		dw	0fad7h
       		dw	0fb21h
       		dw	0fb71h
       		dw	0fbc6h
       		dw	0fc20h
       		dw	0fc81h
       		dw	0fce7h
       		dw	0fd52h
       		dw	0fdc4h
       		dw	0fe3bh
       		dw	0feb8h
       		dw	0ff3bh
       		dw	0ffc3h
       		dw	050h
       		dw	0e4h
       		dw	017dh
       		dw	021ch
       		dw	02c0h
       		dw	0369h
       		dw	0418h
       		dw	04cch
       		dw	0584h
       		dw	0642h
       		dw	0704h
       		dw	07cbh
        	dw	0897h
        	dw	0967h
        	dw	0a3bh
        	dw	0b12h
         	dw	0beeh
         	dw	0ccdh
         	dw	0db0h
         	dw	0e95h
        	dw	0f7eh
        	dw	01069h
        	dw	01157h
        	dw	01247h
        	dw	01339h
        	dw	0142dh
        	dw	01522h
        	dw	01619h
        	dw	01710h
        	dw	01808h
        	dw	01900h
        	dw	019f9h
        	dw	01af1h
        	dw	01be9h
        	dw	01ce0h
        	dw	01dd6h
        	dw	01ecbh
        	dw	01fbeh
        	dw	020afh
        	dw	0219dh
        	dw	0228ah
        	dw	02373h
        	dw	02459h
         	dw	0253ch
         	dw	0261bh
         	dw	026f7h
         	dw	027ceh
         	dw	028a0h
         	dw	0296dh
         	dw	02a36h
         	dw	02af9h
        	dw	02bb6h
        	dw	02c6eh
        	dw	02d20h
        	dw	02dcbh
        	dw	02e6fh
        	dw	02f0dh
        	dw	02fa4h
        	dw	03034h
        	dw	030bch
        	dw	0313dh
        	dw	031b5h
        	dw	03226h
        	dw	0328fh
	       	dw	032f0h
       		dw	03348h
        	dw	03398h
        	dw	033dfh
        	dw	0341eh
        	dw	03453h
        	dw	03480h
        	dw	034a4h
        	dw	034bfh
        	dw	034d2h
        	dw	034dbh

ENDIF

ENDIF


IFNDEF Kaiser

IF 0

	; Sampling= 22050 Cutoff= 6000  Taps= 32  window= 0
	; (times 2)
	; Atten @ 5512 Hz=1.0460, 72% @ 5848 Hz
	;
LPDecHalf22050	dw	0526h
		dw	0fe1eh
		dw	0faa7h
		dw	03c9h
		dw	0524h
		dw	0f9efh
		dw	0fba2h
		dw	08d8h
		dw	02c1h
		dw	0f38dh
		dw	051h
		dw	011e0h
		dw	0f924h
		dw	0e27dh
		dw	01da2h
		dw	07af4h

ENDIF

	; Sampling= 22050 Cutoff= 5500  Taps= 32  window= 0
	; (times 2)
	; Atten @ 5512 Hz=0.4719, 72% @ 5336 Hz
	;
LPDecHalf22050	dw	0fc16h
		dw	0fc3dh
		dw	0477h
		dw	0466h
		dw	0facah
		dw	0fab9h
		dw	0644h
		dw	0692h
		dw	0f81eh
		dw	0f758h
		dw	0aadh
		dw	0c98h
		dw	0ef56h
		dw	0e929h
		dw	0269dh
		dw	07308h

ELSE

	; Sampling= 22050 Cutoff= 4200  Taps= 32  window= 5
	; Beta=     6.0000(times 2)
	; Atten @ 5512 Hz=0.0020, 72% @ 4029 Hz
	;
LPDecHalf22050	dw	0fffbh
		dw	0ffcch
		dw	0ffd2h
		dw	081h
		dw	0121h
		dw	00h
		dw	0fd6fh
		dw	0fd52h
		dw	0264h
		dw	0791h
		dw	030fh
		dw	0f4d0h
		dw	0ee7fh
		dw	0485h
		dw	0339eh
		dw	05b8ah

ENDIF


IFNDEF Kaiser

	; Sampling= 44100 Cutoff= 5500  Taps= 32  window= 0
	; (times 2)
	; Atten @ 5501 Hz=0.4849, 72% @ 5170 Hz
	;
LPDecQrtr44100	dw	0fddbh
		dw	0fac2h
		dw	0fa7ch
		dw	0fda4h
		dw	02d8h
		dw	0739h
		dw	07deh
		dw	0388h
		dw	0fbb6h
		dw	0f45eh
		dw	0f25fh
		dw	0f935h
		dw	090bh
		dw	01e2bh
		dw	03221h
		dw	03e3bh

ELSE

	; Sampling= 44100 Cutoff= 4200  Taps= 32  window= 5
	; Beta=     6.0000(times 2)
	; Atten @ 5501 Hz=0.1186, 72% @ 3539 Hz
	;
LPDecQrtr44100	dw	02h
		dw	024h
		dw	068h
		dw	0b1h
		dw	0afh
		dw	00h
		dw	0fe73h
		dw	0fc54h
		dw	0faa1h
		dw	0fad7h
		dw	0fe74h
		dw	0635h
		dw	01181h
		dw	01e41h
		dw	02964h
		dw	02fe6h

ENDIF

	.CODE

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; RVDecimInit
;
; Reset local variables for the decimation filters.  Public routine.
;
; Function Prototype:
;
; void pascal
;    RVDecimInit(void)
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	EVEN
RVDecimInit	PROC
	call	RVInterpInit
	ret
RVDecimInit	ENDP

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; RVDecimBuf_11025
;
; Change sampling rate from 11025 to 7200.  Public Routine.
;
; Function Prototype:
;
; unsigned int pascal
;    RVDecimBuf_11025(
;       unsigned int far *Spointer,unsigned int far *Dpointer,
;	   unsigned int NPoints);
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	EVEN
RVDecimBuf_11025	PROC USES SI DI, \
		XPtr : FAR PTR WORD, YPtr : FAR PTR WORD, \
		NPoints : WORD
	LOCAL	Counter
	LOCAL	YPtrStart

	; Initialize counter to the number of points to process.
	;
	mov	ax,NPoints
	or	ax,ax
	jnz	I49_ok

	; Number of points is zero -- return.
	;
	sub	ax,ax
	mov	dx,ax
	ret

I49_Done:
	; Restore registers for later call.
	;
	mov	I32Counter,dx
	mov	LPIndex,di
	mov	WORD PTR [Xptr],si

	; All done.  Return number of data points produced.
	;
	mov	ax,WORD PTR [YPtr]
	sub	ax,WORD PTR YPtrStart
	shr	ax,1
	ret

I49_ok:
	; Intialize the point counter.
	;
	inc	ax			; add one because we always subtract
	mov	Counter,ax
	mov	ax,WORD PTR [YPtr]
	mov	YPtrStart,ax
	mov	di,LPIndex		; preload for both branches

I49_Loop:
	mov	dx,I32Counter
	les	si,XPtr

I49_ShortLoop:
	sub	ax,ax
	sub	ax,dx
	jl	I49_Make

	; Check if no more data points to process.
	;
	dec	Counter
	jz	I49_Done

	; include new data point into delay line.
	;
	mov	cx,es:[si]
	add	si,2
	addmsml	di,-2			; subtract 2 from LPIndex

	; Times 2 at the end
	;
	sar	cx,1
	mov	LPData[di],cx

	add	dx,MCYCLE		; 2*32
	or	ax,ax
	jne	I49_ShortLoop

	mov	ax,-MCYCLE

I49_Make:
	sub	dx,LCYCLE		; 2*49
	mov	I32Counter,dx
	mov	WORD PTR [Xptr],si

	add	ax,MCYCLE		; 2*32
	mov	si,ax

	; Do the sum of products.  32 multiplications done
	; in line to save time.  Calculate data point due to
	; real data point.
	;
	mov	ax,LPdata[di]			; Y(n-1)
	imul	LPDecim11025[si]		; a1(n)
	mov	bx,ax
	mov	cx,dx

	addmsml	di,2				; Y(n-2)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si+MCYCLE] 	; a2(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-3)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si+2*MCYCLE]	; a3(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-4)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si+3*MCYCLE]	; a4(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-5)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si+4*MCYCLE]	; a5(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-6)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si+5*MCYCLE]	; a6(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-7)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si+6*MCYCLE]	; a7(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-8)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si+7*MCYCLE]	; a8(n)
	add	bx,ax
	adc	cx,dx

IFDEF BIG_FILTERS

	addmsml	di,2				; Y(n-9)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si+8*MCYCLE] 	; a9(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-10)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si+9*MCYCLE]	; a10(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-11)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si+10*MCYCLE]	; a11(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-12)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si+11*MCYCLE]	; a12(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-13)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si+12*MCYCLE]	; a13(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-14)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si+13*MCYCLE]	; a14(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-15)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si+14*MCYCLE]	; a15(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-16)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si+15*MCYCLE]	; a16(n)
	add	bx,ax
	adc	cx,dx

	sub	si,16*MCYCLE-2
	neg	si

ELSE

	sub	si,8*MCYCLE-2
	neg	si

ENDIF

	addmsml	di,2				; Y(n-17)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si]		; a17(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-18)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si-MCYCLE]		; a18(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-19)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si-2*MCYCLE]	; a19(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-20)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si-3*MCYCLE]	; a20(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-21)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si-4*MCYCLE]	; a21(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-22)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si-5*MCYCLE]	; a22(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-23)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si-6*MCYCLE]	; a23(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-24)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si-7*MCYCLE]	; a24(n)
	add	bx,ax
	adc	cx,dx

IFDEF BIG_FILTERS

	addmsml	di,2				; Y(n-25)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si-8*MCYCLE]	; a25(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-26)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si-9*MCYCLE]	; a26(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-27)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si-10*MCYCLE]	; a27(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-28)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si-11*MCYCLE]	; a28(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-29)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si-12*MCYCLE]	; a29(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-30)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si-13*MCYCLE]	; a30(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-31)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si-14*MCYCLE] 	; a31(n)
	add	bx,ax
	adc	cx,dx

	addmsml	di,2				; Y(n-32)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si-15*MCYCLE]	; a32(n)
	add	bx,ax
	adc	cx,dx

ENDIF

	; Decimal point adjust
 	;
	adjust	cx,bx
	clip	cx,bx,Ov2
	adjust	cx,bx
	clip	cx,bx

Ov2:
	addmsml	di,2				; complete circle

	; Store the data point.
	;
	les	si,YPtr
	mov	es:[si],cx
	add	si,2
	mov	WORD PTR [YPtr],si

	jmp	I49_Loop

RVDecimBuf_11025	ENDP

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; RVDecimBuf_22050
;
; Change sampling rate from 22050 to 7200.  Public Routine.
;
; Function Prototype:
;
; unsigned int pascal
;    RVDecimBuf_22050(
;       unsigned int far *Spointer,unsigned int far *Dpointer,
;	   unsigned int NPoints);
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	EVEN
RVDecimBuf_22050	PROC USES SI DI, \
		XPtr : FAR PTR WORD, YPtr : FAR PTR WORD, \
		NPoints : WORD
	LOCAL	Counter
	LOCAL	YPtrStart

	; Initialize counter to the number of points to process.
	;
	mov	ax,NPoints
	or	ax,ax
	jnz	I49_ok

	; Number of points is zero -- return.
	;
	sub	ax,ax
	mov	dx,ax
	ret

I49_Done:
	; Restore registers for later call.
	;
	mov	I32Counter,dx
	mov	LPIndex,di
	mov	WORD PTR [Xptr],si

	; All done.  Return number of data points produced.
	;
	mov	ax,WORD PTR [YPtr]
	sub	ax,WORD PTR YPtrStart
	shr	ax,1
	ret

I49_ok:
	; Intialize the point counter.
	;
	inc	ax			; add one because we always subtract
	mov	Counter,ax
	mov	ax,WORD PTR [YPtr]
	mov	YPtrStart,ax
	mov	di,LPIndex		; preload for both branches

I49_Loop:
	mov	dx,I32Counter
	les	si,XPtr

I49_ShortLoop:
	sub	ax,ax
	sub	ax,dx
	jl	I49_Make

	; Check if no more data points to process.
	;
	dec	Counter
	jz	I49_Done

	; include new data point into delay line.
	;
	mov	cx,es:[si]
	add	si,2
	addmlrg	di,-2			; subtract 2 from LPIndex

	; Coeff. have a times 2
	;
	sar	cx,1
	mov	LPData[di],cx

	add	dx,MCYCLE/2		; 2*32/2
	or	ax,ax
	jne	I49_ShortLoop

	mov	ax,-MCYCLE/2

I49_Make:
	sub	dx,LCYCLE		; 2*49
	mov	I32Counter,dx
	mov	WORD PTR [Xptr],si

	add	ax,MCYCLE/2		; 2*32/2
	mov	si,ax

	; Do the sum of products.  32 multiplications done
	; in line to save time.  Calculate data point due to
	; real data point.
	;
	mov	ax,LPdata[di]			; Y(n-1)
	imul	LPDecim11025[si]		; a1(n)
	mov	bx,ax
	mov	cx,dx

	addmlrg	di,2				; Y(n-2)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si+MCYCLE/2] 	; a2(n)
	add	bx,ax
	adc	cx,dx

	addmlrg	di,2				; Y(n-3)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si+MCYCLE]		; a3(n)
	add	bx,ax
	adc	cx,dx

	addmlrg	di,2				; Y(n-4)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si+3*MCYCLE/2]	; a4(n)
	add	bx,ax
	adc	cx,dx

	addmlrg	di,2				; Y(n-5)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si+2*MCYCLE]	; a5(n)
	add	bx,ax
	adc	cx,dx

	addmlrg	di,2				; Y(n-6)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si+5*MCYCLE/2]	; a6(n)
	add	bx,ax
	adc	cx,dx

	addmlrg	di,2				; Y(n-7)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si+3*MCYCLE]	; a7(n)
	add	bx,ax
	adc	cx,dx

	addmlrg	di,2				; Y(n-8)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si+7*MCYCLE/2]	; a8(n)
	add	bx,ax
	adc	cx,dx

	addmlrg	di,2				; Y(n-9)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si+4*MCYCLE] 	; a9(n)
	add	bx,ax
	adc	cx,dx

	addmlrg	di,2				; Y(n-10)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si+9*MCYCLE/2]	; a10(n)
	add	bx,ax
	adc	cx,dx

	addmlrg	di,2				; Y(n-11)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si+5*MCYCLE]	; a11(n)
	add	bx,ax
	adc	cx,dx

	addmlrg	di,2				; Y(n-12)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si+11*MCYCLE/2]	; a12(n)
	add	bx,ax
	adc	cx,dx

	addmlrg	di,2				; Y(n-13)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si+6*MCYCLE]	; a13(n)
	add	bx,ax
	adc	cx,dx

	addmlrg	di,2				; Y(n-14)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si+13*MCYCLE/2]	; a14(n)
	add	bx,ax
	adc	cx,dx

	addmlrg	di,2				; Y(n-15)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si+7*MCYCLE]	; a15(n)
	add	bx,ax
	adc	cx,dx

	addmlrg	di,2				; Y(n-16)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si+15*MCYCLE/2]	; a16(n)
	add	bx,ax
	adc	cx,dx

IFDEF BIG_FILTERS

	addmlrg	di,2				; Y(n-17)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si+8*MCYCLE] 	; a17(n)
	add	bx,ax
	adc	cx,dx

	addmlrg	di,2				; Y(n-18)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si+17*MCYCLE/2] 	; a18(n)
	add	bx,ax
	adc	cx,dx

	addmlrg	di,2				; Y(n-19)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si+9*MCYCLE]	; a19(n)
	add	bx,ax
	adc	cx,dx

	addmlrg	di,2				; Y(n-20)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si+19*MCYCLE/2]	; a20(n)
	add	bx,ax
	adc	cx,dx

	addmlrg	di,2				; Y(n-21)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si+10*MCYCLE]	; a21(n)
	add	bx,ax
	adc	cx,dx

	addmlrg	di,2				; Y(n-22)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si+21*MCYCLE/2]	; a22(n)
	add	bx,ax
	adc	cx,dx

	addmlrg	di,2				; Y(n-23)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si+11*MCYCLE]	; a23(n)
	add	bx,ax
	adc	cx,dx

	addmlrg	di,2				; Y(n-24)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si+23*MCYCLE/2]	; a24(n)
	add	bx,ax
	adc	cx,dx

	addmlrg	di,2				; Y(n-25)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si+12*MCYCLE] 	; a25(n)
	add	bx,ax
	adc	cx,dx

	addmlrg	di,2				; Y(n-26)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si+25*MCYCLE/2]	; a26(n)
	add	bx,ax
	adc	cx,dx

	addmlrg	di,2				; Y(n-27)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si+13*MCYCLE]	; a27(n)
	add	bx,ax
	adc	cx,dx

	addmlrg	di,2				; Y(n-28)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si+27*MCYCLE/2]	; a28(n)
	add	bx,ax
	adc	cx,dx

	addmlrg	di,2				; Y(n-29)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si+14*MCYCLE]	; a29(n)
	add	bx,ax
	adc	cx,dx

	addmlrg	di,2				; Y(n-30)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si+29*MCYCLE/2]	; a30(n)
	add	bx,ax
	adc	cx,dx

	addmlrg	di,2				; Y(n-31)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si+15*MCYCLE]	; a31(n)
	add	bx,ax
	adc	cx,dx

	addmlrg	di,2				; Y(n-32)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si+31*MCYCLE/2]	; a32(n)
	add	bx,ax
	adc	cx,dx

	sub	si,16*MCYCLE-2
	neg	si

ELSE

	sub	si,8*MCYCLE-2
	neg	si

ENDIF

	addmlrg	di,2				; Y(n-33)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si]		; a33(n)
	add	bx,ax
	adc	cx,dx

	addmlrg	di,2				; Y(n-34)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si-MCYCLE/2]	; a34(n)
	add	bx,ax
	adc	cx,dx

	addmlrg	di,2				; Y(n-35)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si-MCYCLE]		; a35(n)
	add	bx,ax
	adc	cx,dx

	addmlrg	di,2				; Y(n-36)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si-3*MCYCLE/2]	; a36(n)
	add	bx,ax
	adc	cx,dx

	addmlrg	di,2				; Y(n-37)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si-2*MCYCLE]	; a37(n)
	add	bx,ax
	adc	cx,dx

	addmlrg	di,2				; Y(n-38)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si-5*MCYCLE/2]	; a38(n)
	add	bx,ax
	adc	cx,dx

	addmlrg	di,2				; Y(n-39)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si-3*MCYCLE]	; a39(n)
	add	bx,ax
	adc	cx,dx

	addmlrg	di,2				; Y(n-40)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si-7*MCYCLE/2]	; a40(n)
	add	bx,ax
	adc	cx,dx

	addmlrg	di,2				; Y(n-41)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si-4*MCYCLE]	; a41(n)
	add	bx,ax
	adc	cx,dx

	addmlrg	di,2				; Y(n-42)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si-9*MCYCLE/2]	; a42(n)
	add	bx,ax
	adc	cx,dx

	addmlrg	di,2				; Y(n-43)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si-5*MCYCLE]	; a43(n)
	add	bx,ax
	adc	cx,dx

	addmlrg	di,2				; Y(n-44)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si-11*MCYCLE/2]	; a44(n)
	add	bx,ax
	adc	cx,dx

	addmlrg	di,2				; Y(n-45)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si-6*MCYCLE]	; a45(n)
	add	bx,ax
	adc	cx,dx

	addmlrg	di,2				; Y(n-46)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si-13*MCYCLE/2]	; a46(n)
	add	bx,ax
	adc	cx,dx

	addmlrg	di,2				; Y(n-47)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si-7*MCYCLE] 	; a47(n)
	add	bx,ax
	adc	cx,dx

	addmlrg	di,2				; Y(n-48)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si-15*MCYCLE/2]	; a48(n)
	add	bx,ax
	adc	cx,dx

IFDEF BIG_FILTERS

	addmlrg	di,2				; Y(n-49)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si-8*MCYCLE]	; a49(n)
	add	bx,ax
	adc	cx,dx

	addmlrg	di,2				; Y(n-50)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si-17*MCYCLE/2]	; a50(n)
	add	bx,ax
	adc	cx,dx

	addmlrg	di,2				; Y(n-51)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si-9*MCYCLE]	; a51(n)
	add	bx,ax
	adc	cx,dx

	addmlrg	di,2				; Y(n-52)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si-19*MCYCLE/2]	; a52(n)
	add	bx,ax
	adc	cx,dx

	addmlrg	di,2				; Y(n-53)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si-10*MCYCLE]	; a53(n)
	add	bx,ax
	adc	cx,dx

	addmlrg	di,2				; Y(n-54)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si-21*MCYCLE/2]	; a54(n)
	add	bx,ax
	adc	cx,dx

	addmlrg	di,2				; Y(n-55)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si-11*MCYCLE]	; a55(n)
	add	bx,ax
	adc	cx,dx

	addmlrg	di,2				; Y(n-56)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si-23*MCYCLE/2]	; a56(n)
	add	bx,ax
	adc	cx,dx

	addmlrg	di,2				; Y(n-57)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si-12*MCYCLE]	; a57(n)
	add	bx,ax
	adc	cx,dx

	addmlrg	di,2				; Y(n-58)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si-25*MCYCLE/2]	; a58(n)
	add	bx,ax
	adc	cx,dx

	addmlrg	di,2				; Y(n-59)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si-13*MCYCLE]	; a59(n)
	add	bx,ax
	adc	cx,dx

	addmlrg	di,2				; Y(n-60)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si-27*MCYCLE/2]	; a60(n)
	add	bx,ax
	adc	cx,dx

	addmlrg	di,2				; Y(n-61)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si-14*MCYCLE]	; a61(n)
	add	bx,ax
	adc	cx,dx

	addmlrg	di,2				; Y(n-62)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si-29*MCYCLE/2]	; a62(n)
	add	bx,ax
	adc	cx,dx

	addmlrg	di,2				; Y(n-63)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si-15*MCYCLE] 	; a63(n)
	add	bx,ax
	adc	cx,dx

	addmlrg	di,2				; Y(n-64)
	mov	ax,LPdata[di]
	imul	LPDecim11025[si-31*MCYCLE/2]	; a64(n)
	add	bx,ax
	adc	cx,dx

ENDIF

	; Decimal point adjust
 	;
	adjust	cx,bx
	clip	cx,bx

	addmlrg	di,2				; complete circle

	; Store the data point.
	;
	les	si,YPtr
	mov	es:[si],cx
	add	si,2
	mov	WORD PTR [YPtr],si

	jmp	I49_Loop

RVDecimBuf_22050	ENDP

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; RVDecimBufHalf22050
;
; Change sampling rate from 22050 to 11025.  Public Routine.
;
; Function Prototype:
;
; unsigned int pascal
;    RVDecimBuf_22050_to_11025(
;       unsigned int far *Spointer,unsigned int far *Dpointer,
;	   unsigned int NPoints);
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	EVEN
RVDecimBufHalf22050	PROC USES SI DI, \
		XPtr : FAR PTR WORD, YPtr : FAR PTR WORD, \
		NPoints : WORD
	LOCAL	Counter
	LOCAL	YPtrStart

	; Initialize counter to the number of points to process.
	;
	mov	ax,NPoints
	or	ax,ax
	jnz	I49_ok

	; Number of points is zero -- return.
	;
	sub	ax,ax
	mov	dx,ax
	ret

I49_Done:
	; Restore registers for later call.
	;
	mov	I32Counter,ax
	mov	LPIndex,di
	mov	WORD PTR [Xptr],si

	; All done.  Return number of data points produced.
	;
	mov	ax,WORD PTR [YPtr]
	sub	ax,WORD PTR YPtrStart
	shr	ax,1
	ret

I49_ok:
	; Intialize the point counter.
	;
	inc	ax			; add one because we always subtract
	mov	Counter,ax
	mov	ax,WORD PTR [YPtr]
	mov	YPtrStart,ax
	mov	di,LPIndex		; preload for both branches
	les	si,XPtr

I49_Loop:
	mov	ax,I32Counter

I49_ShortLoop:
	; Check if no more data points to process.
	;
	dec	Counter
	jz	I49_Done

	; include new data point into delay line.
	;
	mov	cx,es:[si]
	add	si,2
	addm64	di,-2			; subtract 2 from LPIndex

	; Coeff. have a times 2
	;
	sar	cx,1
	mov	LPData[di],cx

	inc	ax
	test	ax,1
	jz	I49_ShortLoop

Ifs_a:
	; Save I49 counter.
	;
	mov	I32Counter,ax

	; Do the sum of products.  16 multiplications done
	; in line to save time.  Calculate data point due to
	; real data point.
	;
	mov	ax,LPdata[di]			; Y(n-1)
	addm64	di,2*(LPQORDER-1)		; Y(n-33)
	add	ax,LPdata[di]
	imul	LPDecHalf22050[0]		; a1(n)
	mov	bx,ax
	mov	cx,dx

	addm64	di,-2				; Y(n-32)
	mov	ax,LPdata[di]
	addm64	di,-2*(LPQORDER-3)		; Y(n-2)
	add	ax,LPdata[di]
	imul	LPDecHalf22050[2]		; a2(n)
	add	bx,ax
	adc	cx,dx

	addm64	di,2				; Y(n-3)
	mov	ax,LPdata[di]
	addm64	di,2*(LPQORDER-5)		; Y(n-31)
	add	ax,LPdata[di]
	imul	LPDecHalf22050[4]		; a3(n)
	add	bx,ax
	adc	cx,dx

	addm64	di,-2				; Y(n-30)
	mov	ax,LPdata[di]
	addm64	di,-2*(LPQORDER-7)		; Y(n-4)
	add	ax,LPdata[di]
	imul	LPDecHalf22050[6]		; a4(n)
	add	bx,ax
	adc	cx,dx

	addm64	di,2				; Y(n-5)
	mov	ax,LPdata[di]
	addm64	di,2*(LPQORDER-9)		; Y(n-29)
	add	ax,LPdata[di]
	imul	LPDecHalf22050[8]		; a5(n)
	add	bx,ax
	adc	cx,dx

	addm64	di,-2				; Y(n-28)
	mov	ax,LPdata[di]
	addm64	di,-2*(LPQORDER-11)		; Y(n-6)
	add	ax,LPdata[di]
	imul	LPDecHalf22050[10]		; a6(n)
	add	bx,ax
	adc	cx,dx

	addm64	di,2				; Y(n-7)
	mov	ax,LPdata[di]
	addm64	di,2*(LPQORDER-13)		; Y(n-27)
	add	ax,LPdata[di]
	imul	LPDecHalf22050[12]		; a7(n)
	add	bx,ax
	adc	cx,dx

	addm64	di,-2				; Y(n-26)
	mov	ax,LPdata[di]
	addm64	di,-2*(LPQORDER-15)		; Y(n-8)
	add	ax,LPdata[di]
	imul	LPDecHalf22050[14]		; a8(n)
	add	bx,ax
	adc	cx,dx

	addm64	di,2				; Y(n-9)
	mov	ax,LPdata[di]
	addm64	di,2*(LPQORDER-17)		; Y(n-25)
	add	ax,LPdata[di]
	imul	LPDecHalf22050[16]		; a9(n)
	add	bx,ax
	adc	cx,dx

	addm64	di,-2				; Y(n-24)
	mov	ax,LPdata[di]
	addm64	di,-2*(LPQORDER-19)		; Y(n-10)
	add	ax,LPdata[di]
	imul	LPDecHalf22050[18]		; a10(n)
	add	bx,ax
	adc	cx,dx

	addm64	di,2				; Y(n-11)
	mov	ax,LPdata[di]
	addm64	di,2*(LPQORDER-21)		; Y(n-23)
	add	ax,LPdata[di]
	imul	LPDecHalf22050[20]		; a11(n)
	add	bx,ax
	adc	cx,dx

	addm64	di,-2				; Y(n-22)
	mov	ax,LPdata[di]
	addm64	di,-2*(LPQORDER-23)		; Y(n-12)
	add	ax,LPdata[di]
	imul	LPDecHalf22050[22]		; a12(n)
	add	bx,ax
	adc	cx,dx

	addm64	di,2				; Y(n-13)
	mov	ax,LPdata[di]
	addm64	di,2*(LPQORDER-25)		; Y(n-21)
	add	ax,LPdata[di]
	imul	LPDecHalf22050[24]		; a13(n)
	add	bx,ax
	adc	cx,dx

	addm64	di,-2				; Y(n-20)
	mov	ax,LPdata[di]
	addm64	di,-2*(LPQORDER-27)		; Y(n-14)
	add	ax,LPdata[di]
	imul	LPDecHalf22050[26]		; a14(n)
	add	bx,ax
	adc	cx,dx

	addm64	di,2				; Y(n-15)
	mov	ax,LPdata[di]
	addm64	di,2*(LPQORDER-29)		; Y(n-19)
	add	ax,LPdata[di]
	imul	LPDecHalf22050[28]		; a15(n)
	add	bx,ax
	adc	cx,dx

	addm64	di,-2				; Y(n-18)
	mov	ax,LPdata[di]
	addm64	di,-2*(LPQORDER-31)		; Y(n-16)
	add	ax,LPdata[di]
	imul	LPDecHalf22050[30]		; a14(n)
	add	bx,ax
	adc	cx,dx

	; Decimal point adjust
 	;
	adjust	cx,bx
	clip	cx,bx

	addm64	di,2*(17)			; complete circle

	; Store the data point.
	;
	les	bx,YPtr
	mov	es:[bx],cx
	mov	es,WORD PTR [XPtr+2]		; restore ES
	add	bx,2
	mov	WORD PTR [YPtr],bx

	jmp	I49_Loop

RVDecimBufHalf22050	ENDP

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; RVDecimBufQrtr44100
;
; Change sampling rate from 44100 to 11025.  Public Routine.
;
; Function Prototype:
;
; unsigned int pascal
;    RVDecimBuf_44100_to_11025(
;       unsigned int far *Spointer,unsigned int far *Dpointer,
;	   unsigned int NPoints);
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	EVEN
RVDecimBufQrtr44100	PROC USES SI DI, \
		XPtr : FAR PTR WORD, YPtr : FAR PTR WORD, \
		NPoints : WORD
	LOCAL	Counter
	LOCAL	YPtrStart

	; Initialize counter to the number of points to process.
	;
	mov	ax,NPoints
	or	ax,ax
	jnz	I49_ok

	; Number of points is zero -- return.
	;
	sub	ax,ax
	mov	dx,ax
	ret

I49_Done:
	; Restore registers for later call.
	;
	mov	I32Counter,ax
	mov	LPIndex,di
	mov	WORD PTR [Xptr],si

	; All done.  Return number of data points produced.
	;
	mov	ax,WORD PTR [YPtr]
	sub	ax,WORD PTR YPtrStart
	shr	ax,1
	ret

I49_ok:
	; Intialize the point counter.
	;
	inc	ax			; add one because we always subtract
	mov	Counter,ax
	mov	ax,WORD PTR [YPtr]
	mov	YPtrStart,ax
	mov	di,LPIndex		; preload for both branches
	les	si,XPtr

I49_Loop:
	mov	ax,I32Counter

I49_ShortLoop:
	; Check if no more data points to process.
	;
	dec	Counter
	jz	I49_Done

	; include new data point into delay line.
	;
	mov	cx,es:[si]
	add	si,2
	addm64	di,-2			; subtract 2 from LPIndex

	; Coeff. have a times 2
	;
	sar	cx,1
	mov	LPData[di],cx

	inc	ax
	mov	bx,ax
	and	bx,3
	jnz	I49_ShortLoop

Ifs_a:
	; Save I49 counter.
	;
	mov	I32Counter,ax

	; Do the sum of products.  16 multiplications done
	; in line to save time.  Calculate data point due to
	; real data point.
	;
	mov	ax,LPdata[di]			; Y(n-1)
	addm64	di,2*(LPQORDER-1)		; Y(n-33)
	add	ax,LPdata[di]
	imul	LPDecQrtr44100[0]		; a1(n)
	mov	bx,ax
	mov	cx,dx

	addm64	di,-2				; Y(n-32)
	mov	ax,LPdata[di]
	addm64	di,-2*(LPQORDER-3)		; Y(n-2)
	add	ax,LPdata[di]
	imul	LPDecQrtr44100[2]		; a2(n)
	add	bx,ax
	adc	cx,dx

	addm64	di,2				; Y(n-3)
	mov	ax,LPdata[di]
	addm64	di,2*(LPQORDER-5)		; Y(n-31)
	add	ax,LPdata[di]
	imul	LPDecQrtr44100[4]		; a3(n)
	add	bx,ax
	adc	cx,dx

	addm64	di,-2				; Y(n-30)
	mov	ax,LPdata[di]
	addm64	di,-2*(LPQORDER-7)		; Y(n-4)
	add	ax,LPdata[di]
	imul	LPDecQrtr44100[6]		; a4(n)
	add	bx,ax
	adc	cx,dx

	addm64	di,2				; Y(n-5)
	mov	ax,LPdata[di]
	addm64	di,2*(LPQORDER-9)		; Y(n-29)
	add	ax,LPdata[di]
	imul	LPDecQrtr44100[8]		; a5(n)
	add	bx,ax
	adc	cx,dx

	addm64	di,-2				; Y(n-28)
	mov	ax,LPdata[di]
	addm64	di,-2*(LPQORDER-11)		; Y(n-6)
	add	ax,LPdata[di]
	imul	LPDecQrtr44100[10]		; a6(n)
	add	bx,ax
	adc	cx,dx

	addm64	di,2				; Y(n-7)
	mov	ax,LPdata[di]
	addm64	di,2*(LPQORDER-13)		; Y(n-27)
	add	ax,LPdata[di]
	imul	LPDecQrtr44100[12]		; a7(n)
	add	bx,ax
	adc	cx,dx

	addm64	di,-2				; Y(n-26)
	mov	ax,LPdata[di]
	addm64	di,-2*(LPQORDER-15)		; Y(n-8)
	add	ax,LPdata[di]
	imul	LPDecQrtr44100[14]		; a8(n)
	add	bx,ax
	adc	cx,dx

	addm64	di,2				; Y(n-9)
	mov	ax,LPdata[di]
	addm64	di,2*(LPQORDER-17)		; Y(n-25)
	add	ax,LPdata[di]
	imul	LPDecQrtr44100[16]		; a9(n)
	add	bx,ax
	adc	cx,dx

	addm64	di,-2				; Y(n-24)
	mov	ax,LPdata[di]
	addm64	di,-2*(LPQORDER-19)		; Y(n-10)
	add	ax,LPdata[di]
	imul	LPDecQrtr44100[18]		; a10(n)
	add	bx,ax
	adc	cx,dx

	addm64	di,2				; Y(n-11)
	mov	ax,LPdata[di]
	addm64	di,2*(LPQORDER-21)		; Y(n-23)
	add	ax,LPdata[di]
	imul	LPDecQrtr44100[20]		; a11(n)
	add	bx,ax
	adc	cx,dx

	addm64	di,-2				; Y(n-22)
	mov	ax,LPdata[di]
	addm64	di,-2*(LPQORDER-23)		; Y(n-12)
	add	ax,LPdata[di]
	imul	LPDecQrtr44100[22]		; a12(n)
	add	bx,ax
	adc	cx,dx

	addm64	di,2				; Y(n-13)
	mov	ax,LPdata[di]
	addm64	di,2*(LPQORDER-25)		; Y(n-21)
	add	ax,LPdata[di]
	imul	LPDecQrtr44100[24]		; a13(n)
	add	bx,ax
	adc	cx,dx

	addm64	di,-2				; Y(n-20)
	mov	ax,LPdata[di]
	addm64	di,-2*(LPQORDER-27)		; Y(n-14)
	add	ax,LPdata[di]
	imul	LPDecQrtr44100[26]		; a14(n)
	add	bx,ax
	adc	cx,dx

	addm64	di,2				; Y(n-15)
	mov	ax,LPdata[di]
	addm64	di,2*(LPQORDER-29)		; Y(n-19)
	add	ax,LPdata[di]
	imul	LPDecQrtr44100[28]		; a15(n)
	add	bx,ax
	adc	cx,dx

	addm64	di,-2				; Y(n-18)
	mov	ax,LPdata[di]
	addm64	di,-2*(LPQORDER-31)		; Y(n-16)
	add	ax,LPdata[di]
	imul	LPDecQrtr44100[30]		; a14(n)
	add	bx,ax
	adc	cx,dx

	; Decimal point adjust
 	;
	adjust	cx,bx
	clip	cx,bx

	addm64	di,2*(17)			; complete circle

	; Store the data point.
	;
	les	bx,YPtr
	mov	es:[bx],cx
	mov	es,WORD PTR [XPtr+2]		; restore ES
	add	bx,2
	mov	WORD PTR [YPtr],bx

	jmp	I49_Loop

RVDecimBufQrtr44100	ENDP

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Return Version Identification Number.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; RVVerAPDPC
;
; Version Identification Number for this release.
;
; Function Prototype:
;
; unsigned int pascal
;    RVVerADPCM(void)
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	EVEN
RVVerADPCM	PROC

	; Major number is in AH, and the minor number is in AL.
	;

IFDEF BIG_FILTERS

	mov	ax,0151h

ELSE

	mov	ax,0150h

ENDIF

	ret
RVVerADPCM	ENDP

END

